@@ -34,8 +34,10 @@
 
 #include <application.h>
 #include <expr.h>
+#include <expr-name.h>
 #include <value.h>
 #include <sheet.h>
+#include <sheet-view.h>
 #include <number-match.h>
 #include <cell.h>
 #include <parse-util.h>
@@ -47,9 +49,11 @@
 #include <position.h>
 #include <ranges.h>
 #include <io-context.h>
+#include <command-context.h>
 #include <workbook-view.h>
 #include <workbook.h>
 #include <error-info.h>
+#include <parse-util.h>
 
 #include <gsf/gsf-input-textline.h>
 #include <ctype.h>
@@ -70,6 +74,8 @@ typedef struct {
 	size_t buffer_size;
 	size_t line_len;
 	int zoom;
+	GSList *sheet_order;
+	GSList *std_names, *real_names;
 } ApplixReadState;
 
 /* #define NO_DEBUG_APPLIX */
@@ -141,6 +147,76 @@ applix_parse_value (char *buf, char **follow)
 	return buf;
 }
 
+static char const *
+applix_sheetref_parse (char const *start, Sheet **sheet, Workbook const *wb)
+{
+	char const *end, *begin;
+	char *name;
+
+	begin = end = (*start == '$') ? start + 1 : start;
+	while (*end && g_ascii_isalnum (*end))
+		end++;
+
+	if (*end != ':') {
+		*sheet = NULL;
+		return start;
+	}
+
+	name = g_alloca (1 + end - begin);
+	strncpy (name, begin, end-begin);
+	name [end-begin] = '\0';
+	*sheet = workbook_sheet_by_name (wb, name);
+	return *sheet != NULL ? end : start;
+}
+
+static char const *
+applix_rangeref_parse (RangeRef *res, char const *start, ParsePos const *pp)
+{
+	char const *ptr = start, *tmp1, *tmp2;
+	Workbook *wb = pp->wb;
+
+	g_return_val_if_fail (start != NULL, start);
+	g_return_val_if_fail (pp != NULL, start);
+
+	/* TODO : Does not handle external references */
+
+	ptr = applix_sheetref_parse (start, &res->a.sheet, wb);
+	if (ptr == NULL)
+		return start; /* TODO error unknown sheet */
+	if (*ptr == ':') ptr++;
+	tmp1 = col_parse (ptr, &res->a.col, &res->a.col_relative);
+	if (tmp1 == ptr)
+		return start;
+	tmp2 = row_parse (tmp1, &res->a.row, &res->a.row_relative);
+	if (tmp2 == tmp1)
+		return start;
+	if (res->a.col_relative)
+		res->a.col -= pp->eval.col;
+	if (res->a.row_relative)
+		res->a.row -= pp->eval.row;
+	if (tmp2[0] != '.' || tmp2[1] != '.') {
+		res->b = res->a;
+		return tmp2;
+	}
+
+	start = tmp2;
+	ptr = applix_sheetref_parse (start+2, &res->b.sheet, wb);
+	if (ptr == NULL)
+		return start; /* TODO error unknown sheet */
+	if (*ptr == ':') ptr++;
+	tmp1 = col_parse (ptr, &res->b.col, &res->b.col_relative);
+	if (tmp1 == ptr)
+		return start;
+	tmp2 = row_parse (tmp1, &res->b.row, &res->b.row_relative);
+	if (tmp2 == tmp1)
+		return start;
+	if (res->b.col_relative)
+		res->b.col -= pp->eval.col;
+	if (res->b.row_relative)
+		res->b.row -= pp->eval.row;
+	return tmp2;
+}
+
 static unsigned char *
 applix_get_line (ApplixReadState *state)
 {
@@ -343,8 +419,8 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 		/* Allocate the new style */
 		style = mstyle_new_default ();
 
-		mstyle_set_content_hidden (style, is_protected);
-		mstyle_set_content_locked (style, is_invisible);
+		mstyle_set_content_locked (style, is_protected);
+		mstyle_set_content_hidden (style, is_invisible);
 
 		if (sep[1] == '\'')
 			sep += 2;
@@ -454,10 +530,10 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 					/* comma 'CO' */
 					if (sep[1] == 'O') {
 						++sep;
-						format_prefix = "#,###";
+						format_prefix = "#,##0";
 					} else
 						/* FIXME : what currency to use for differnt locales */
-						format_prefix = "$ #,###";
+						format_prefix = "$ #,##0";
 
 					format_suffix = "";
 
@@ -729,8 +805,23 @@ applix_read_attributes (ApplixReadState *state)
 }
 
 static Sheet *
-applix_get_sheet (ApplixReadState *state, unsigned char **buffer,
-		  char const separator)
+applix_fetch_sheet (ApplixReadState *state, char const *name)
+{
+	Sheet *sheet = workbook_sheet_by_name (state->wb, name);
+
+	if (sheet == NULL) {
+		sheet = sheet_new (state->wb, name);
+		workbook_sheet_attach (state->wb, sheet, NULL);
+		sheet_set_zoom_factor (sheet, (double )(state->zoom) / 100., FALSE, FALSE);
+		sheet_flag_recompute_spans (sheet);
+	}
+
+	return sheet;
+}
+
+static Sheet *
+applix_parse_sheet (ApplixReadState *state, unsigned char **buffer,
+		    char const separator)
 {
 	Sheet *sheet;
 
@@ -743,14 +834,7 @@ applix_get_sheet (ApplixReadState *state, unsigned char **buffer,
 	}
 
 	*tmp = '\0';
-	sheet = workbook_sheet_by_name (state->wb, *buffer);
-	if (sheet == NULL) {
-		sheet = sheet_new (state->wb, *buffer);
-		workbook_sheet_attach (state->wb, sheet, NULL);
-		sheet_set_zoom_factor (sheet, (double )(state->zoom) / 100., FALSE, FALSE);
-		sheet_flag_recompute_spans (sheet);
-	}
-
+	sheet = applix_fetch_sheet (state, *buffer);
 	*buffer = tmp+1;
 	return sheet;
 }
@@ -762,10 +846,10 @@ applix_parse_cellref (ApplixReadState *state, unsigned char *buffer,
 {
 	int len;
 
-	*sheet = applix_get_sheet (state, &buffer, separator);
+	*sheet = applix_parse_sheet (state, &buffer, separator);
 
 	/* Get cell addr */
-	if (*sheet && parse_cell_name (buffer, pos, FALSE, &len))
+	if (*sheet && cellpos_parse (buffer, pos, FALSE, &len))
 		return buffer + len;
 
 	*sheet = NULL;
@@ -800,21 +884,18 @@ applix_read_view (ApplixReadState *state, unsigned char *buffer)
 {
 	Sheet *sheet = NULL;
 	unsigned char *name = buffer + 19;
+	unsigned char *tmp;
 	gboolean ignore;
-	int len = strlen (name);
 
-	g_return_val_if_fail (len > 1 && name [len-1] == '~', -1);
+	tmp = strchr (name, ':');
+	if (tmp == NULL)
+		return 0;
+	*tmp =  '\0';
 
-	/* I wish there were some docs on this, names sometimes appear to be
-	 * SHEET_LETTER:name~ and sometimes
-	 * SHEET_LETTER:name:~ and sometimes
-	 */
-	if (name [len-2] == ':')
-		name [len-2] = '\0';
-	else
-		name [len-1] = '\0';
-
-	ignore = !strcmp (name, "Current"); /* ignore current */
+	ignore = tmp[1] != '~';
+	if (!ignore)
+		state->sheet_order = g_slist_prepend (state->sheet_order,
+			applix_fetch_sheet (state, name));
 
 	while (NULL != (buffer = applix_get_line (state))) {
 		if (!a_strncmp (buffer, "View End, Name: ~"))
@@ -875,15 +956,16 @@ applix_read_view (ApplixReadState *state, unsigned char *buffer)
 							  TRUE);
 			} while (ptr[0] == ' ' && isdigit ((unsigned char) ptr[1]));
 		} else if (!a_strncmp (buffer, "View Column Widths: ")) {
-			char *ptr = buffer + 19;
-			do {
-				int col, width;
-				char *tmp;
+			char const *ptr = buffer + 19;
+			char const *tmp;
+			int col, width;
+			unsigned char dummy;
 
-				col = parse_col_name (tmp = ptr + 1, (char const **)&ptr);
-				if (tmp == ptr || col < 0 || *ptr != ':')
+			do {
+				ptr = col_parse (tmp = ptr + 1, &col, &dummy);
+				if (tmp == ptr || *ptr != ':')
 					return applix_parse_error (state, "Invalid column");
-				width = strtol (tmp = ptr + 1, &ptr, 10);
+				width = strtol (tmp = ptr + 1, (char **)&ptr, 10);
 				if (tmp == ptr || width <= 0)
 					return applix_parse_error (state, "Invalid column size");
 
@@ -1002,14 +1084,15 @@ applix_read_cells (ApplixReadState *state)
 				} else
 					expr = gnm_expr_parse_str (expr_string+1,
 						parse_pos_init_cell (&pos, cell),
-						GNM_EXPR_PARSE_USE_APPLIX_REFERENCE_CONVENTIONS |
+						GNM_EXPR_PARSE_USE_APPLIX_CONVENTIONS |
 						GNM_EXPR_PARSE_CREATE_PLACEHOLDER_FOR_UNKNOWN_FUNC,
-						&perr);
+						&applix_rangeref_parse,
+						parse_error_init (&perr));
 
 				if (expr == NULL) {
 					(void) applix_parse_error (state, _("%s!%s : unable to parse '%s'\n     %s"),
 								   cell->base.sheet->name_quoted, cell_name (cell),
-								   expr_string, perr.message);
+								   expr_string, perr.err->message);
 					parse_error_free (&perr);
 					expr = gnm_expr_new_constant (value_new_string (expr_string));
 				} else if (is_array) {
@@ -1079,7 +1162,7 @@ applix_read_row_list (ApplixReadState *state, unsigned char *ptr)
 {
 	unsigned char *tmp;
 	Range	r;
-	Sheet *sheet = applix_get_sheet (state, &ptr, ' ');
+	Sheet *sheet = applix_parse_sheet (state, &ptr, ' ');
 
 	if (ptr == NULL)
 		return -1;
@@ -1118,10 +1201,29 @@ static gboolean
 applix_read_sheet_table (ApplixReadState *state)
 {
 	unsigned char *ptr;
-	while (NULL != (ptr = applix_get_line (state)))
+	unsigned char *std_name, *real_name;
+	while (NULL != (ptr = applix_get_line (state))) {
 	       if (!a_strncmp (ptr, "END SHEETS TABLE"))
 		       return FALSE;
-#warning TODO
+
+	       /* Sheet A: ~Foo~ */
+	       std_name = ptr + 6;
+	       ptr = strchr (std_name, ':');
+	       if (ptr == NULL)
+		       continue;
+	       *ptr = '\0';
+
+	       real_name = ptr + 3;
+	       ptr = strchr (real_name, '~');
+	       if (ptr == NULL)
+		       continue;
+	       *ptr = '\0';
+
+	       state->std_names  = g_slist_prepend (state->std_names,
+						    g_strdup (std_name));
+	       state->real_names = g_slist_prepend (state->real_names,
+						    g_strdup (real_name));
+	}
 	return TRUE;
 }
 
@@ -1136,9 +1238,12 @@ applix_read_header_footer (ApplixReadState *state)
 }
 
 static gboolean
-applix_read_relative_name (ApplixReadState *state, char *buffer)
+applix_read_absolute_name (ApplixReadState *state, char *buffer)
 {
 	char *end;
+	RangeRef ref;
+	ParsePos pp;
+	GnmExpr const *expr;
 
 	/* .ABCDe. Coordinate: A:B2..A:C4 */
 	/* Spec guarantees that there are no dots in the name */
@@ -1149,23 +1254,55 @@ applix_read_relative_name (ApplixReadState *state, char *buffer)
 	if (end == NULL)
 		return TRUE;
 	*end = '\0';
-#if 0
-	GnmExpr *expr = gnm_expr_parse_str (expr_string+1,
-		parse_pos_init_cell (&pos, cell),
-		GNM_EXPR_PARSE_USE_APPLIX_REFERENCE_CONVENTIONS |
-		GNM_EXPR_PARSE_CREATE_PLACEHOLDER_FOR_UNKNOWN_FUNC,
-		&perr);
-#endif
-	puts (buffer);
+	end = strchr (end + 1, ':');
+	if (end == NULL)
+		return TRUE;
+	applix_rangeref_parse (&ref, end+2,
+		parse_pos_init (&pp, state->wb, NULL, 0, 0));
+	ref.a.col_relative = ref.b.col_relative =
+		ref.a.row_relative = ref.b.row_relative = FALSE;
+
+	expr = gnm_expr_new_constant (value_new_cellrange_unsafe (&ref.a, &ref.b));
+	expr_name_add (&pp, buffer, expr, NULL, TRUE);
+
 	return FALSE;
 }
 
 static gboolean
-applix_read_absolute_name (ApplixReadState *state, char *buffer)
+applix_read_relative_name (ApplixReadState *state, char *buffer)
 {
+	int dummy;
+	char *end;
+	RangeRef ref, flag;
+	ParsePos pp;
+	GnmExpr const *expr;
+
 	/* .abcdE. tCol:0 tRow:0 tSheet:0 bCol:1 bRow:2 bSheet: 0 tColAbs:0 tRowAbs:0 tSheetAbs:1 bColAbs:0 bRowAbs:0 bSheetAbs:1 */
 	/* Spec guarantees that there are no dots in the name */
-	puts (buffer);
+	buffer = strchr (buffer, '.');
+	if (buffer == NULL)
+		return TRUE;
+	end = strchr (++buffer, '.');
+	if (end == NULL)
+		return TRUE;
+	*end = '\0';
+	if (12 != sscanf (end + 2,
+			  " tCol:%d tRow:%d tSheet:%d bCol:%d bRow:%d bSheet: %d tColAbs:%d tRowAbs:%d tSheetAbs:%d bColAbs:%d bRowAbs:%d bSheetAbs:%d",
+			  &ref.a.col, &ref.a.row, &dummy, &ref.b.col, &ref.b.row, &dummy,
+			  &flag.a.col, &flag.a.row, &dummy, &flag.b.col, &flag.b.row, &dummy))
+		return TRUE;
+
+	ref.a.col_relative = (flag.a.col == 0);
+	ref.b.col_relative = (flag.b.col == 0);
+	ref.a.row_relative = (flag.a.row == 0);
+	ref.b.row_relative = (flag.b.row == 0);
+
+	ref.a.sheet = ref.b.sheet = NULL;
+	expr = gnm_expr_new_constant (value_new_cellrange_unsafe (&ref.a, &ref.b));
+	parse_pos_init (&pp, state->wb, NULL,
+		MAX (-ref.a.col, 0), MAX (-ref.a.row, 0));
+	expr_name_add (&pp, buffer, expr, NULL, TRUE);
+
 	return FALSE;
 }
 
@@ -1220,7 +1357,8 @@ applix_read_impl (ApplixReadState *state)
 			d (0, printf ("Applix load : Saved with revision %d.%d",
 				      major_rev, minor_rev););
 		} else if (!a_strncmp (buffer, "Current Doc Real Name:")) {
-			real_name = g_strdup (buffer + 22);
+			g_free (real_name);
+			real_name = NULL;  /* FIXME? g_strdup (buffer + 22); */
 
 		} else if (!strcmp (buffer, "COLORMAP")) {
 			if (applix_read_colormap (state))
@@ -1278,7 +1416,7 @@ applix_read_impl (ApplixReadState *state)
 				return applix_parse_error (state, "Absolute named range");
 		} else if (!a_strncmp (buffer, REL_NAMED_RANGE)) {
 			if (applix_read_relative_name (state, buffer + sizeof (REL_NAMED_RANGE)))
-				return applix_parse_error (state, "Absolute named range");
+				return applix_parse_error (state, "Relative named range");
 		} else if (!a_strncmp (buffer, "Row List")) {
 			if (applix_read_row_list (state, buffer + sizeof ("Row List")))
 				return applix_parse_error (state, "row list");
@@ -1321,6 +1459,7 @@ applix_read (IOContext *io_context, WorkbookView *wb_view, GsfInput *src)
 	int i;
 	int res;
 	ApplixReadState	state;
+	GSList *ptr, *renamed_sheets;
 
 	/* Init the state variable */
 	state.input	  = gsf_input_textline_new (src);
@@ -1335,6 +1474,9 @@ applix_read (IOContext *io_context, WorkbookView *wb_view, GsfInput *src)
 	state.buffer      = NULL;
 	state.buffer_size = 0;
 	state.line_len    = 80;
+	state.sheet_order = NULL;
+	state.std_names   = NULL;
+	state.real_names  = NULL;
 
 	/* Actually read the workbook */
 	res = applix_read_impl (&state);
@@ -1343,6 +1485,25 @@ applix_read (IOContext *io_context, WorkbookView *wb_view, GsfInput *src)
 	if (state.buffer)
 		g_free (state.buffer);
 
+	state.sheet_order = g_slist_reverse (state.sheet_order);
+	workbook_sheet_reorder (state.wb, state.sheet_order,  NULL);
+	g_slist_free (state.sheet_order);
+
+	renamed_sheets = NULL;
+	for (ptr = state.std_names; ptr != NULL ; ptr = ptr->next)
+		renamed_sheets = g_slist_prepend (renamed_sheets,
+			workbook_sheet_by_name (state.wb, ptr->data));
+	renamed_sheets = g_slist_reverse (renamed_sheets);
+	workbook_sheet_reorganize (state.wb, renamed_sheets, NULL,
+		state.real_names, state.std_names,
+		NULL, NULL, NULL, NULL, NULL, NULL,
+		COMMAND_CONTEXT (io_context));
+	g_slist_free (renamed_sheets);
+	g_slist_foreach (state.std_names, (GFunc)g_free, NULL);
+	g_slist_free (state.std_names);
+	g_slist_foreach (state.real_names, (GFunc)g_free, NULL);
+	g_slist_free (state.real_names);
+
 	/* Release the shared expressions and styles */
 	g_hash_table_foreach_remove (state.exprs, &cb_remove_expr, NULL);
 	g_hash_table_destroy (state.exprs);
