@@ -16,21 +16,18 @@
 #include "ms-container.h"
 #include <expr.h>
 
-typedef struct _ExcelSheet
-{
+typedef struct _ExcelSheet	ExcelSheet;
+
+struct _ExcelSheet {
 	MSContainer container;
 
-	Sheet *gnum_sheet;
-	struct _ExcelWorkbook *wb;
-	GHashTable *shared_formulae;
-	double base_char_width;
-	double base_char_width_default;
+	Sheet		*sheet;
+	GHashTable	*shared_formulae, *tables;
 
-	gboolean freeze_panes : 1;
-} ExcelSheet;
+	gboolean freeze_panes;
+};
 
-typedef struct _BiffBoundsheetData
-{
+typedef struct {
 	guint16 index;
 	guint32 streamStartPos;
 	MsBiffFileType type;
@@ -44,14 +41,21 @@ typedef struct {
 	guint8 *data;
 	guint32 data_len;
 	gboolean is_array;
-} BiffSharedFormula;
+} XLSharedFormula;
+
+typedef struct {
+	CellPos key;
+	guint8 *data;
+	guint32 data_len;
+} XLDataTable;
 
 /* Use the upper left corner as the key to a collection of shared formulas */
-BiffSharedFormula *ms_excel_sheet_shared_formula (ExcelSheet const *sheet,
-						  CellPos const    *key);
+XLSharedFormula *excel_sheet_shared_formula (ExcelSheet const *sheet,
+					     CellPos const    *key);
+XLDataTable	*excel_sheet_data_table	    (ExcelSheet const *esheet,
+					     CellPos const    *key);
 
-typedef struct _ExcelPalette
-{
+typedef struct {
 	int *red;
 	int *green;
 	int *blue;
@@ -59,8 +63,7 @@ typedef struct _ExcelPalette
 	StyleColor **gnum_cols;
 } ExcelPalette;
 
-typedef struct _BiffFontData
-{
+typedef struct {
 	guint16 index;
 	int height;         /* in 1/20ths of a point   */
 	int italic;         /* boolean                 */
@@ -72,69 +75,56 @@ typedef struct _BiffFontData
 	char *fontname;
 } BiffFontData;
 
-typedef struct _BiffExternSheetData {
-	Sheet *first_sheet;
-	Sheet *last_sheet;
-} XLExternSheetV8;
-
-typedef struct _BiffFormatData {
+typedef struct {
 	guint16 idx;
 	char *name;
 } BiffFormatData;
 
-typedef struct _ExcelWorkbook
-{
+struct _ExcelWorkbook {
 	MSContainer	  container;
 	IOContext	 *context;
 	WorkbookView	 *wbv;
 
 	GPtrArray	 *excel_sheets;
 	GHashTable	 *boundsheet_data_by_stream;
-	GHashTable	 *boundsheet_data_by_index;
+	GPtrArray	 *boundsheet_sheet_by_index;
 	GPtrArray	 *XF_cell_records;
 	GHashTable	 *font_data;
 	GHashTable	 *format_data; /* leave as a hash */
-	GPtrArray	 *names;
-	GArray		 *extern_sheet_v8;
-	GPtrArray	 *extern_sheet_v7;
+	struct {
+		GPtrArray	 *supbooks;
+		GArray		 *externsheet;
+	} v8; /* biff8 does this in the workbook */
 	ExcelPalette	 *palette;
 	char		**global_strings;
 	guint32		  global_string_max;
 
 	gboolean          warn_unsupported_graphs;
-	GSList		 *delayed_names;
 
 	ExprTreeSharer   *expr_sharer;
 
-	/**
-	 * Gnumeric parallel workbook
-   	 **/
 	Workbook            *gnum_wb;
-} ExcelWorkbook;
+};
 
 char       *biff_get_text (guint8 const *ptr, guint32 length, guint32 *byte_length);
 char const *biff_get_error_text (guint8 err);
 
-GnmExpr const	 	*ms_excel_workbook_get_name  (ExcelWorkbook const *ewb, int idx);
-ExcelSheet		*ms_excel_workbook_get_sheet (ExcelWorkbook const *wb, guint idx);
-XLExternSheetV8 const	*ms_excel_workbook_get_externsheet_v8 (ExcelWorkbook const *wb,
-							       guint idx);
-Sheet 			*ms_excel_workbook_get_externsheet_v7 (ExcelWorkbook const *wb,
-							       int idx, int sheet_index);
+Sheet		*excel_externsheet_v7	 (MSContainer const *container, gint16 i);
+void		 excel_externsheet_v8	 (ExcelWorkbook const *wb,  gint16 i,
+					  Sheet **first, Sheet **last);
 
-MsBiffBofData * ms_biff_bof_data_new (BiffQuery * q);
-void ms_biff_bof_data_destroy (MsBiffBofData * data);
+void		excel_read_EXTERNSHEET_v7 (BiffQuery const *q, MSContainer *container);
+MsBiffBofData  *ms_biff_bof_data_new     (BiffQuery * q);
+void		ms_biff_bof_data_destroy (MsBiffBofData * data);
 
-StyleColor  *ms_excel_palette_get (ExcelPalette const *pal, gint idx);
+StyleColor  *excel_palette_get (ExcelPalette const *pal, gint idx);
 
-void	    ms_excel_read_imdata (BiffQuery *q);
+void	    excel_read_IMDATA (BiffQuery *q);
 
 /* A utility routine to handle unexpected BIFF records */
-void          ms_excel_unexpected_biff (BiffQuery *q,
-					char const *state,
-					int debug_level);
+void excel_unexpected_biff (BiffQuery *q, char const *state, int debug_level);
 
-void ms_excel_read_cleanup (void);
-void ms_excel_read_init (void);
+void excel_read_cleanup (void);
+void excel_read_init (void);
 
 #endif
