@@ -1192,33 +1192,36 @@ static const char *help_mod = {
 	   "@SEEALSO=INT,FLOOR,CEIL")
 };
 
+/*
+ * MOD(-1,-3) = -1
+ * MOD(2,-3) = -2
+ * MOD(10.6,2) = 0.6
+ * MOD(-10.6,2) = 1.4
+ * MOD(10.6,-2) = -0.6
+ * MOD(-10.6,-2) = -1.4
+ */
+
 static Value *
 gnumeric_mod (FunctionEvalInfo *ei, Value **argv)
 {
-	int a, b;
+	gnum_float a, b, babs, r;
 
-	a = value_get_as_int (argv[0]);
-	b = value_get_as_int (argv[1]);
+	a = value_get_as_float (argv[0]);
+	b = value_get_as_float (argv[1]);
 
 	if (b == 0)
 		return value_new_error (ei->pos, gnumeric_err_DIV0);
-	else if (b > 0) {
-		if (a >= 0)
-			return value_new_int (a % b);
-		else {
-			int c = (-a) % b;
-			return value_new_int (c ? b - c : 0);
-		}
-	} else {
-		b = -b;
-		a = -a;
-		if (a >= 0)
-			return value_new_int (-(a % b));
-		else {
-			int c = (-a) % b;
-			return value_new_int (c ? c - b : 0);
-		}
+
+	babs = gnumabs (b);
+	r = fmodgnum (gnumabs (a), babs);
+	if (r > 0) {
+		if ((a < 0) != (b < 0))
+			r = babs - r;
+		if (b < 0)
+			r = -r;
 	}
+
+	return value_new_float (r);
 }
 
 /***************************************************************************/
@@ -1575,7 +1578,7 @@ gnumeric_even (FunctionEvalInfo *ei, Value **argv)
 		number = -number;
 	}
 	ceiled = ceilgnum (number);
-	if (fmod (ceiled, 2) == 0)
+	if (fmodgnum (ceiled, 2) == 0)
 	        if (number > ceiled)
 		        return value_new_int ((int) (sign * (ceiled + 2)));
 		else
@@ -1613,7 +1616,7 @@ gnumeric_odd (FunctionEvalInfo *ei, Value **argv)
 		number = -number;
 	}
 	ceiled = ceilgnum (number);
-	if (fmod (ceiled, 2) == 1)
+	if (fmodgnum (ceiled, 2) == 1)
 	        if (number > ceiled)
 		        return value_new_int ((int) (sign * (ceiled + 2)));
 		else
@@ -1662,6 +1665,54 @@ gnumeric_factdouble (FunctionEvalInfo *ei, Value **argv)
 
 /***************************************************************************/
 
+static const char *help_fib = {
+	N_("@FUNCTION=FIB\n"
+	   "@SYNTAX=FIB(number)\n"
+
+	   "@DESCRIPTION="
+	   "FIB function computes Fibonacci numbers.\n"
+	   "\n"
+	   "* If @number is not an integer, it is truncated.\n"
+	   "* If @number is negative or zero FIB returns #NUM! error.\n"
+	   "\n"
+	   "@EXAMPLES=\n"
+	   "FIB(12) equals 144.\n"
+	   "\n"
+	   "@SEEALSO=")
+};
+
+static Value *
+gnumeric_fib (FunctionEvalInfo *ei, Value **argv)
+
+{
+	static int fibs[47];
+	static int fib_count = sizeof (fibs) / sizeof (fibs[0]);
+	static gboolean inited = FALSE;
+
+	int n = value_get_as_int (argv[0]);
+	if (n <= 0)
+		return value_new_error (ei->pos, gnumeric_err_NUM );
+
+	if (n < fib_count) {
+		if (!inited) {
+			int i;
+			fibs[1] = fibs[2] = 1;
+			for (i = 3; i < fib_count; i++)
+				fibs[i] = fibs[i - 1] + fibs[i - 2];
+			inited = TRUE;
+		}
+		return value_new_int (fibs[n]);
+	} else {
+		gnum_float s5 = sqrtgnum (5.0);
+		gnum_float r1 = (1 + s5) / 2;
+		gnum_float r2 = (1 - s5) / 2;
+		// Use the Binet form.
+		return value_new_float ((powgnum (r1, n) - powgnum (r2, n)) / s5);
+	}
+}
+
+/***************************************************************************/
+
 static const char *help_quotient = {
 	N_("@FUNCTION=QUOTIENT\n"
 	   "@SYNTAX=QUOTIENT(numerator,denumerator)\n"
@@ -1917,6 +1968,10 @@ gnumeric_mround (FunctionEvalInfo *ei, Value **argv)
 	number = value_get_as_float (argv[0]);
 	multiple = value_get_as_float (argv[1]);
 
+	/* Weird, but XL compatible.  */
+	if (multiple == 0)
+		return value_new_int (0);
+
 	if ((number > 0 && multiple < 0)
 	    || (number < 0 && multiple > 0))
 		return value_new_error (ei->pos, gnumeric_err_NUM);
@@ -1927,8 +1982,8 @@ gnumeric_mround (FunctionEvalInfo *ei, Value **argv)
 		multiple = -multiple;
 	}
 
-	mod = fmod (number, multiple);
-	div = number-mod;
+	mod = fmodgnum (number, multiple);
+	div = number - mod;
 
         return value_new_float (sign * (
 		div + ((mod + accuracy_limit >= multiple / 2) ? multiple : 0)));
@@ -2601,8 +2656,9 @@ gnumeric_seriessum (FunctionEvalInfo *ei, GnmExprList *nodes)
 	if (tree == NULL)
 		return value_new_error (ei->pos, gnumeric_err_NUM);
 
-	val = gnm_expr_eval (tree, ei->pos, GNM_EXPR_EVAL_STRICT);
+	val = gnm_expr_eval (tree, ei->pos, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
 	if (!val) return NULL;
+	/* FIXME: What if val is an error?  */
 	if (!VALUE_IS_NUMBER (val)) {
 		value_release (val);
 		return value_new_error (ei->pos, gnumeric_err_VALUE);
@@ -2617,8 +2673,10 @@ gnumeric_seriessum (FunctionEvalInfo *ei, GnmExprList *nodes)
 	if (tree == NULL)
 		return value_new_error (ei->pos, gnumeric_err_NUM);
 
-	val = gnm_expr_eval (tree, ei->pos, GNM_EXPR_EVAL_STRICT);
+	val = gnm_expr_eval (tree, ei->pos, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
 	if (!val) return NULL;
+	/* FIXME: what if val is an error.  */
+	/* DOUBLE FIXME: what if val is an error and "x" was a string?  */
 	if (! VALUE_IS_NUMBER (val)) {
 		value_release (val);
 		return value_new_error (ei->pos, gnumeric_err_VALUE);
@@ -2636,7 +2694,7 @@ gnumeric_seriessum (FunctionEvalInfo *ei, GnmExprList *nodes)
 	if (tree == NULL)
 		return value_new_error (ei->pos, gnumeric_err_NUM);
 
-	val = gnm_expr_eval (tree, ei->pos, GNM_EXPR_EVAL_STRICT);
+	val = gnm_expr_eval (tree, ei->pos, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
 	if (!val) return NULL;
 	if (! VALUE_IS_NUMBER (val)) {
 		value_release (val);
@@ -2703,8 +2761,8 @@ validate_range_numeric_matrix (const EvalPos *ep, Value * matrix,
 	Value *res;
 	int cell_count = 0;
 
-	*cols = value_area_get_width (ep, matrix);
-	*rows = value_area_get_height (ep, matrix);
+	*cols = value_area_get_width (matrix, ep);
+	*rows = value_area_get_height (matrix, ep);
 
 	/* No checking needed for arrays */
 	if (matrix->type == VALUE_ARRAY)
@@ -2762,7 +2820,7 @@ gnumeric_minverse (FunctionEvalInfo *ei, Value **argv)
 	for (c = 0; c < cols; c++)
 	        for (r = 0; r < rows; r++) {
 		        Value const * a =
-			      value_area_get_x_y (ep, values, c, r);
+			      value_area_get_x_y (values, c, r, ep);
 		        *(matrix + r + c * cols) = value_get_as_float (a);
 		}
 
@@ -2839,14 +2897,14 @@ gnumeric_mmult (FunctionEvalInfo *ei, Value **argv)
 	for (c = 0; c < cols_a; c++)
 	        for (r = 0; r < rows_a; r++) {
 		        Value const * a =
-			     value_area_get_x_y (ep, values_a, c, r);
+			     value_area_get_x_y (values_a, c, r, ep);
 		        A[r + c * rows_a] = value_get_as_float (a);
 		}
 
 	for (c = 0; c < cols_b; c++)
 	        for (r = 0; r < rows_b; r++) {
 		        Value const * b =
-			     value_area_get_x_y (ep, values_b, c, r);
+			     value_area_get_x_y (values_b, c, r, ep);
 		        B[r + c * rows_b] = value_get_as_float (b);
 		}
 
@@ -2913,7 +2971,7 @@ gnumeric_mdeterm (FunctionEvalInfo *ei, Value **argv)
 	for (c = 0; c < cols; c++)
 	        for (r = 0; r < rows; r++) {
 		        Value const * a =
-			      value_area_get_x_y (ep, values, c, r);
+			      value_area_get_x_y (values, c, r, ep);
 		        *(matrix + r + c * cols) = value_get_as_float (a);
 		}
 
@@ -2970,8 +3028,8 @@ gnumeric_sumproduct (FunctionEvalInfo *ei, GnmExprList *args)
 					   GNM_EXPR_EVAL_PERMIT_NON_SCALAR |
 					   GNM_EXPR_EVAL_PERMIT_EMPTY);
 
-		thissizex = value_area_get_width (ei->pos, val);
-		thissizey = value_area_get_height (ei->pos, val);
+		thissizex = value_area_get_width (val, ei->pos);
+		thissizey = value_area_get_height (val, ei->pos);
 
 		if (i == 0) {
 			sizex = thissizex;
@@ -2983,8 +3041,7 @@ gnumeric_sumproduct (FunctionEvalInfo *ei, GnmExprList *args)
 		for (y = 0; y < thissizey; y++) {
 			for (x = 0; x < thissizex; x++) {
 				/* FIXME: efficiency worries?  */
-				const Value *v = value_area_fetch_x_y (ei->pos,
-								       val, x, y);
+				Value const *v = value_area_fetch_x_y (val, x, y, ei->pos);
 				if (v->type == VALUE_ERROR) {
 					/*
 					 * We carefully tranverse the argument
@@ -3038,125 +3095,193 @@ gnumeric_sumproduct (FunctionEvalInfo *ei, GnmExprList *args)
 
 /***************************************************************************/
 
-const ModulePluginFunctionInfo math_functions[] = {
+const GnmFuncDescriptor math_functions[] = {
 	{ "abs",     "f", N_("number"),    &help_abs,
-	  gnumeric_abs, NULL, NULL, NULL },
+	  gnumeric_abs, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "acos",    "f", N_("number"),    &help_acos,
-	  gnumeric_acos, NULL, NULL, NULL },
+	  gnumeric_acos, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "acosh",   "f", N_("number"),    &help_acosh,
-	  gnumeric_acosh, NULL, NULL, NULL },
+	  gnumeric_acosh, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "asin",    "f", N_("number"),    &help_asin,
-	  gnumeric_asin, NULL, NULL, NULL },
+	  gnumeric_asin, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "asinh",   "f", N_("number"),    &help_asinh,
-	  gnumeric_asinh, NULL, NULL, NULL },
+	  gnumeric_asinh, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "atan",    "f", N_("number"),    &help_atan,
-	  gnumeric_atan, NULL, NULL, NULL },
+	  gnumeric_atan, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "atanh",   "f", N_("number"),    &help_atanh,
-	  gnumeric_atanh, NULL, NULL, NULL },
+	  gnumeric_atanh, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "atan2",   "ff", N_("xnum,ynum"), &help_atan2,
-	  gnumeric_atan2, NULL, NULL, NULL },
+	  gnumeric_atan2, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "cos",     "f", N_("number"),    &help_cos,
-	  gnumeric_cos, NULL, NULL, NULL },
+	  gnumeric_cos, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "cosh",    "f", N_("number"),    &help_cosh,
-	  gnumeric_cosh, NULL, NULL, NULL },
-	{ "countif", "r?", N_("range,criteria"), &help_countif,
-	  gnumeric_countif, NULL, NULL, NULL },
+	  gnumeric_cosh, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
+
+/* MS Excel puts this in statistical */
+	{ "countif", "rS", N_("range,criteria"), &help_countif,
+	  gnumeric_countif, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+
 	{ "ceil",    "f", N_("number"),    &help_ceil,
-	  gnumeric_ceil, NULL, NULL, NULL },
+	  gnumeric_ceil, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "ceiling", "ff", N_("number,significance"), &help_ceiling,
-	  gnumeric_ceiling, NULL, NULL, NULL },
+	  gnumeric_ceiling, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "degrees", "f", N_("number"),    &help_degrees,
-	  gnumeric_degrees, NULL, NULL, NULL },
+	  gnumeric_degrees, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "even",    "f", N_("number"),    &help_even,
-	  gnumeric_even, NULL, NULL, NULL },
+	  gnumeric_even, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "exp",     "f", N_("number"),    &help_exp,
-	  gnumeric_exp, NULL, NULL, NULL },
+	  gnumeric_exp, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "fact",    "f", N_("number"),    &help_fact,
-	  gnumeric_fact, NULL, NULL, NULL },
+	  gnumeric_fact, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+
+/* MS Excel puts this in the engineering functions */
 	{ "factdouble", "f", N_("number"), &help_factdouble,
-	  gnumeric_factdouble, NULL, NULL, NULL },
+	  gnumeric_factdouble, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+
+	{ "fib", "f", N_("number"), &help_fib,
+	  gnumeric_fib, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "combin",  "ff", N_("n,k"),      &help_combin,
-	  gnumeric_combin, NULL, NULL, NULL },
+	  gnumeric_combin, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "floor",   "f|f", N_("number"),  &help_floor,
-	  gnumeric_floor, NULL, NULL, NULL },
+	  gnumeric_floor, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "gcd",     0, N_("number,number"), &help_gcd,
-	  NULL, gnumeric_gcd, NULL, NULL },
+	  NULL, gnumeric_gcd, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "int",     "f", N_("number"),    &help_int,
-	  gnumeric_int, NULL, NULL, NULL },
+	  gnumeric_int, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "lcm",     0, "",            &help_lcm,
-	  NULL, gnumeric_lcm, NULL, NULL },
+	  NULL, gnumeric_lcm, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "ln",      "f", N_("number"),    &help_ln,
-	  gnumeric_ln, NULL, NULL, NULL },
+	  gnumeric_ln, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "log",     "f|f", N_("number,base"), &help_log,
-	  gnumeric_log, NULL, NULL, NULL },
+	  gnumeric_log, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "log2",    "f", N_("number"),    &help_log2,
-	  gnumeric_log2, NULL, NULL, NULL },
+	  gnumeric_log2, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "log10",   "f", N_("number"),    &help_log10,
-	  gnumeric_log10, NULL, NULL, NULL },
+	  gnumeric_log10, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "mod",     "ff", N_("numerator,denominator"), &help_mod,
-	  gnumeric_mod, NULL, NULL, NULL },
+	  gnumeric_mod, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "mround",  "ff", N_("number,multiple"), &help_mround,
-	  gnumeric_mround, NULL, NULL, NULL },
+	  gnumeric_mround, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "multinomial", 0, "", &help_multinomial,
-	  NULL, gnumeric_multinomial, NULL, NULL },
+	  NULL, gnumeric_multinomial, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "odd" ,    "f", N_("number"),    &help_odd,
-	  gnumeric_odd, NULL, NULL, NULL },
+	  gnumeric_odd, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "power",   "ff", N_("base,exponent"),      &help_power,
-	  gnumeric_power, NULL, NULL, NULL },
+	  gnumeric_power, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "g_product", 0, N_("number"),    &help_g_product,
-	  NULL, gnumeric_g_product, NULL, NULL },
+	  NULL, gnumeric_g_product, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "quotient" , "ff", N_("numerator,denominator"), &help_quotient,
-	  gnumeric_quotient, NULL, NULL, NULL },
+	  gnumeric_quotient, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "radians", "f", N_("number"),    &help_radians,
-	  gnumeric_radians, NULL, NULL, NULL },
+	  gnumeric_radians, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "roman",      "f|f", N_("number,type"), &help_roman,
-	  gnumeric_roman, NULL, NULL, NULL },
+	  gnumeric_roman, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "round",      "f|f", N_("number,digits"), &help_round,
-	  gnumeric_round, NULL, NULL, NULL },
+	  gnumeric_round, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "rounddown",  "f|f", N_("number,digits"), &help_rounddown,
-	  gnumeric_rounddown, NULL, NULL, NULL },
+	  gnumeric_rounddown, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "roundup",    "f|f", N_("number,digits"), &help_roundup,
-	  gnumeric_roundup, NULL, NULL, NULL },
+	  gnumeric_roundup, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "seriessum", 0, N_("x,n,m,coefficients"), &help_seriessum,
-	  NULL, gnumeric_seriessum, NULL, NULL },
+	  NULL, gnumeric_seriessum, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "sign",    "f", N_("number"),    &help_sign,
-	  gnumeric_sign, NULL, NULL, NULL },
+	  gnumeric_sign, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "sin",     "f", N_("number"),    &help_sin,
-	  gnumeric_sin, NULL, NULL, NULL },
+	  gnumeric_sin, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "sinh",    "f", N_("number"),    &help_sinh,
-	  gnumeric_sinh, NULL, NULL, NULL },
+	  gnumeric_sinh, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "sqrt",    "f", N_("number"),    &help_sqrt,
-	  gnumeric_sqrt, NULL, NULL, NULL },
+	  gnumeric_sqrt, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "sqrtpi",  "f", N_("number"),    &help_sqrtpi,
-	  gnumeric_sqrtpi, NULL, NULL, NULL },
+	  gnumeric_sqrtpi, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "suma",    0, N_("number,number,"), &help_suma,
-	  NULL, gnumeric_suma, NULL, NULL },
-	{ "sumif",   "r?|r", N_("range,criteria,actual_range"), &help_sumif,
-	  gnumeric_sumif, NULL, NULL, NULL },
+	  NULL, gnumeric_suma, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+	{ "sumif",   "rS|r", N_("range,criteria,actual_range"), &help_sumif,
+	  gnumeric_sumif, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "sumproduct", 0, N_("range,range,"), &help_sumproduct,
-	  NULL, gnumeric_sumproduct, NULL, NULL },
+	  NULL, gnumeric_sumproduct, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "sumsq",   0, N_("number"),      &help_sumsq,
-	  NULL, gnumeric_sumsq, NULL, NULL },
+	  NULL, gnumeric_sumsq, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "sumx2my2", "AA", N_("array1,array2"), &help_sumx2my2,
-	  gnumeric_sumx2my2, NULL, NULL, NULL },
+	  gnumeric_sumx2my2, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "sumx2py2", "AA", N_("array1,array2"), &help_sumx2py2,
-	  gnumeric_sumx2py2, NULL, NULL, NULL },
+	  gnumeric_sumx2py2, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "sumxmy2",  "AA", N_("array1,array2"), &help_sumxmy2,
-	  gnumeric_sumxmy2, NULL, NULL, NULL },
+	  gnumeric_sumxmy2, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "tan",     "f", N_("number"),    &help_tan,
-	  gnumeric_tan, NULL, NULL, NULL },
+	  gnumeric_tan, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "tanh",    "f", N_("number"),    &help_tanh,
-	  gnumeric_tanh, NULL, NULL, NULL },
+	  gnumeric_tanh, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "trunc",   "f|f", N_("number,digits"), &help_trunc,
-	  gnumeric_trunc, NULL, NULL, NULL },
+	  gnumeric_trunc, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "pi",      "", "",           &help_pi,
-	  gnumeric_pi, NULL, NULL, NULL },
+	  gnumeric_pi, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "mmult",   "AA", N_("array1,array2"), &help_mmult,
-	  gnumeric_mmult, NULL, NULL, NULL },
+	  gnumeric_mmult, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_RETURNS_NON_SCALAR, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "minverse","A", N_("array"),     &help_minverse,
-	  gnumeric_minverse, NULL, NULL, NULL },
+	  gnumeric_minverse, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_RETURNS_NON_SCALAR, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "mdeterm", "A", N_("array,matrix_type,bandsize"), &help_mdeterm,
-	  gnumeric_mdeterm, NULL, NULL, NULL },
+	  gnumeric_mdeterm, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_RETURNS_NON_SCALAR, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 #if 0
 	{ "logmdeterm", "A|si", N_("array,matrix_type,bandsize"),
 	  &help_logmdeterm, gnumeric_logmdeterm, NULL, NULL, NULL },
