@@ -23,6 +23,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #include <gnumeric-config.h>
+#include <gnumeric-i18n.h>
 #include <gnumeric.h>
 #include <func.h>
 
@@ -42,7 +43,6 @@
 #include <math.h>
 #include <limits.h>
 #include <string.h>
-#include <libgnome/gnome-i18n.h>
 
 #include "plugin.h"
 #include "plugin-util.h"
@@ -669,7 +669,7 @@ gnumeric_t_ (FunctionEvalInfo *ei, Value **argv)
 	if (argv[0]->type == VALUE_STRING)
 		return value_duplicate (argv[0]);
 	else
-		return value_new_string ("");
+		return value_new_empty ();
 }
 
 /***************************************************************************/
@@ -709,8 +709,7 @@ gnumeric_text (FunctionEvalInfo *ei, Value **args)
 		char *str = format_value (format,
 					  (tmp != NULL) ? tmp : arg,
 					  NULL, -1);
-		res = value_new_string (str);
-		g_free (str);
+		res = value_new_string_nocopy (str);
 	} else
 		res = value_new_error (ei->pos, _("Type mismatch"));
 
@@ -1167,6 +1166,7 @@ free_all_after_search (GSList *conditions, gchar *text, gchar *within)
 	g_free (within);
 }
 
+/* FIXME:  NOT UTF8 safe! */
 static Value *
 gnumeric_search (FunctionEvalInfo *ei, Value **argv)
 {
@@ -1181,10 +1181,8 @@ gnumeric_search (FunctionEvalInfo *ei, Value **argv)
 	else
 		start_num = value_get_as_int (argv[2]) - 1;
 
-	text = value_get_as_string (argv[0]);
-	within = value_get_as_string (argv[1]);
-	g_strdown (text);
-	g_strdown (within);
+	text = g_utf8_strdown (value_peek_string (argv[0]), -1);
+	within = g_utf8_strdown (value_peek_string (argv[1]), -1);
 
 	within_len = strlen (within);
 
@@ -1236,52 +1234,76 @@ match_again:
 
 /***************************************************************************/
 
-const ModulePluginFunctionInfo string_functions[] = {
+const GnmFuncDescriptor string_functions[] = {
         { "char",       "f",     N_("number"),                  &help_char,
-	  gnumeric_char, NULL, NULL, NULL },
+	  gnumeric_char, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "clean",      "S",     N_("text"),                    &help_clean,
-       gnumeric_clean, NULL, NULL, NULL },
+          gnumeric_clean, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "code",       "S",     N_("text"),                    &help_code,
-	  gnumeric_code, NULL, NULL, NULL },
+	  gnumeric_code, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "concatenate", 0,      N_("text,text,"),            &help_concatenate,
-	  NULL, gnumeric_concatenate, NULL, NULL },
+	  NULL, gnumeric_concatenate, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "dollar",     "f|f",   N_("num,decimals"),            &help_dollar,
-	  gnumeric_dollar, NULL, NULL, NULL },
+	  gnumeric_dollar, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "exact",      "SS",    N_("text1,text2"),             &help_exact,
-	  gnumeric_exact, NULL, NULL, NULL },
+	  gnumeric_exact, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "find",       "SS|f",  N_("text1,text2,num"),         &help_find,
-	  gnumeric_find, NULL, NULL, NULL },
+	  gnumeric_find, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "fixed",      "f|fb",  N_("num,decs,no_commas"),      &help_fixed,
-	  gnumeric_fixed, NULL, NULL, NULL },
+	  gnumeric_fixed, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "left",       "S|f",   N_("text,num_chars"),          &help_left,
-	  gnumeric_left, NULL, NULL, NULL },
+	  gnumeric_left, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "len",        "S",     N_("text"),                    &help_len,
-	  gnumeric_len, NULL, NULL, NULL },
+	  gnumeric_len, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "lower",      "S",     N_("text"),                    &help_lower,
-	  gnumeric_lower, NULL, NULL, NULL },
-        { "proper",     "S",     N_("text"),                    &help_proper,
-	  gnumeric_proper, NULL, NULL, NULL },
+	  gnumeric_lower, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "mid",        "Sff",   N_("text,pos,num"),            &help_mid,
-	  gnumeric_mid, NULL, NULL, NULL },
+	  gnumeric_mid, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+        { "proper",     "S",     N_("text"),                    &help_proper,
+	  gnumeric_proper, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "replace",    "SffS",  N_("old,start,num,new"),       &help_replace,
-	  gnumeric_replace, NULL, NULL, NULL },
+	  gnumeric_replace, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "rept",       "Sf",    N_("text,num"),                &help_rept,
-	  gnumeric_rept, NULL, NULL, NULL },
+	  gnumeric_rept, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "right",      "S|f",   N_("text,num_chars"),          &help_right,
-	  gnumeric_right, NULL, NULL, NULL },
+	  gnumeric_right, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "search",     "SS|f",  N_("find,within,start_num"),   &help_search,
-	  gnumeric_search, NULL, NULL, NULL },
+	  gnumeric_search, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "substitute", "SSS|f", N_("text,old,new,num"),       &help_substitute,
-	  gnumeric_substitute, NULL, NULL, NULL },
-        { "t",          "?",     N_("value"),                   &help_t_,
-          gnumeric_t_, NULL, NULL, NULL },
+	  gnumeric_substitute, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+        { "t",          "S",     N_("value"),                   &help_t_,
+          gnumeric_t_, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "text",       "Ss",    N_("value,format_text"),       &help_text,
-	  gnumeric_text, NULL, NULL, NULL },
+	  gnumeric_text, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "trim",       "S",     N_("text"),                    &help_trim,
-	  gnumeric_trim, NULL, NULL, NULL },
+	  gnumeric_trim, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "upper",      "S",     N_("text"),                    &help_upper,
-	  gnumeric_upper, NULL, NULL, NULL },
-        { "value",      "?",     N_("text"),                    &help_value,
-	  gnumeric_value, NULL, NULL, NULL },
+	  gnumeric_upper, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+        { "value",      "S",     N_("text"),                    &help_value,
+	  gnumeric_value, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+
         {NULL}
 };
