@@ -59,6 +59,7 @@
 
 #include <errno.h>
 #include <stdio.h>
+#include <locale.h>
 
 typedef enum {
 	LATEX_NO_BORDER = 0,
@@ -69,12 +70,12 @@ typedef enum {
 
 typedef struct {
 	latex_border_t latex;
-	const char     *vertical;
-	const char     *horizontal;
+	char const     *vertical;
+	char const     *horizontal;
 } latex_border_translator_t;
 
 /* the index into the following array is StyleBorderType */
-static const latex_border_translator_t border_styles[] = {
+static latex_border_translator_t const border_styles[] = {
 	{LATEX_NO_BORDER,     "",  "~"},
 	{LATEX_SINGLE_BORDER, "|", "-"},
 	{LATEX_SINGLE_BORDER, "|", "-"},
@@ -92,12 +93,12 @@ static const latex_border_translator_t border_styles[] = {
 };
 
 typedef struct {
-	const char     *p_1;
-	const char     *p_2;
+	char const     *p_1;
+	char const     *p_2;
 } latex_border_connectors_t;
 
 
-static const latex_border_connectors_t conn_styles[LATEX_MAX_BORDER]
+static latex_border_connectors_t const conn_styles[LATEX_MAX_BORDER]
 [LATEX_MAX_BORDER][LATEX_MAX_BORDER][LATEX_MAX_BORDER] = {
         /*  FIXME: once we are sure that none of the numbered */
         /*entries are in fact needed we should removed the digits */
@@ -109,8 +110,8 @@ static const latex_border_connectors_t conn_styles[LATEX_MAX_BORDER]
 	  {{"1",""}, { "13",""}, { "34",""}}},
 	 {{{"",""}, { "",""}, { "|","|"}},
 	  {{"","|b|"}, { "14",""}, { "|","|"}},
-	  {{"","|b:"}, { "15",""}, { "|",":"}}}},
-	{{{{"",""}, { "","|"}, { "",""}},
+	  {{"","|b:"}, { "|b:",""}, { "|",":"}}}},
+	{{{{"",""}, { "",""}, { "",""}},
 	  {{"",""}, { "|",""}, { "35",""}},
 	  {{"","|"}, { "17",""}, { "36",""}}},
 	 {{{"","|"}, { "|",""}, { "37",""}},
@@ -118,16 +119,16 @@ static const latex_border_connectors_t conn_styles[LATEX_MAX_BORDER]
 	  {{"4",""}, { "19",""}, { "39",""}}},
 	 {{{"","|b|"}, { "20",""}, { "|","|"}},
 	  {{"5",""}, { "21",""}, { "|","|"}},
-	  {{"6",""}, { "22",""}, { "40",""}}}},
+	  {{"|b:",""}, { "22",""}, { "40",""}}}},
 	{{{{"",""}, { "23",""}, { ":t|",""}},
 	  {{"|",""}, { "24",""}, { ":t|",""}},
-	  {{"",""}, { "",""}, { "41",""}}},
-	 {{{"7",""}, { "26",""}, { "42",""}},
+	  {{"",""}, { "",""}, { ":t:",""}}},
+	 {{{"7",""}, { "26",""}, { ":t|",""}},
 	  {{"8",""}, { "27",""}, { "43",""}},
-	  {{"",""}, { "28",""}, { "44",""}}},
+	  {{"",""}, { "",""}, { ":t:",""}}},
 	 {{{":b|",""}, { "29",""}, { ":","|"}},
 	  {{":b|",""}, { "30",""}, { "45",""}},
-	  {{"10",""}, { "31",""}, { ":",":"}}}}
+	  {{":b:",""}, { "31",""}, { ":",":"}}}}
 };
 
 
@@ -141,7 +142,7 @@ static const latex_border_connectors_t conn_styles[LATEX_MAX_BORDER]
  * from Rasca's code to have most common first.
  */
 static void
-latex_fputs (const char *p, FILE *fp)
+latex_fputs (char const *p, FILE *fp)
 {
 	for (; *p; p++) {
 		switch (*p) {
@@ -214,8 +215,6 @@ latex2e_write_file_header(FILE *fp)
 "%%  In addition the following commands need to be executed in the   %%\n"
 "%%  preamble:                                                       %%\n"
 "%%                                                                  %%\n"
-"%%  \\newlength{\\gnumericTableWidth}                                 %%\n"
-"%%  \\newlength{\\gnumericTableWidthComplete}                         %%\n"
 "%%                                                                  %%\n"
 "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n"
 "\n"
@@ -258,8 +257,6 @@ latex2e_write_file_header(FILE *fp)
 "\n"
 "	\\begin{document}\n"
 "\n"
-"        \\newlength{\\gnumericTableWidth}\n"
-"        \\newlength{\\gnumericTableWidthComplete}\n"
 "\n"
 "%%  End of the preamble for the standalone. The next section is for %%\n"
 "%%  documents which are included into other LaTeX2e files.          %%\n"
@@ -286,8 +283,17 @@ latex2e_write_file_header(FILE *fp)
 "%%                                                                  %%\n"
 "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n"
 "\n"
+"\\providecommand{\\gnumericmathit}[1]{#1} \n" 
+"%%  Uncomment the next line if you would like your numbers to be in %%\n"
+"%%  italics if they are italizised in the gnumeric table.           %%\n"
+"%\\renewcommand{\\gnumericmathit}[1]{\\mathit{#1}}\n"
 "\\providecommand{\\gnumericPB}[1]%\n"
 "{\\let\\gnumericTemp=\\\\#1\\let\\\\=\\gnumericTemp\\hspace{0pt}}\n"
+" \\ifundefined{gnumericTableWidthDefined}"
+"        \\newlength{\\gnumericTableWidth}\n"
+"        \\newlength{\\gnumericTableWidthComplete}\n"
+"        \\def\\gnumericTableWidthDefined{}\n"
+" \\fi\n"
 "\n"
 "%%  The default table format retains the relative column widths of  %%\n"
 "%%  gnumeric. They can easily be changed to c, r or l. In that case %%\n"
@@ -487,8 +493,8 @@ latex2e_write_blank_cell (FILE *fp, gint col, gint row, gint index,
  * makes it much more difficult to change column widths later on.
  */
 static void
-latex2e_write_multicolumn_cell (FILE *fp, const Cell *cell, const int num_merged_cols,
-				const int num_merged_rows,  gint index,
+latex2e_write_multicolumn_cell (FILE *fp, Cell const *cell, int num_merged_cols,
+				int num_merged_rows,  gint index,
 				StyleBorderType *borders, Sheet *sheet)
 {
 	char * rendered_string;
@@ -511,7 +517,7 @@ latex2e_write_multicolumn_cell (FILE *fp, const Cell *cell, const int num_merged
 		int i;
 
 		for (i = 0; i < num_merged_cols; i++) {
-			ci = sheet_col_get_info (sheet, cell->col_info->pos + i);
+			ci = sheet_col_get_info (sheet, cell->pos.col + i);
 			merge_width += ci->size_pixels;
 		}
 	}
@@ -537,7 +543,7 @@ latex2e_write_multicolumn_cell (FILE *fp, const Cell *cell, const int num_merged
 			fprintf (fp, "p{");
 			for (i = 0; i < num_merged_cols; i++) {
 				fprintf (fp, "\t\\gnumericCol%s+%%\n",
-					 col_name (cell->col_info->pos + i));
+					 col_name (cell->pos.col + i));
 			}
 			fprintf (fp, "\t\\tabcolsep*2*%i}", num_merged_cols - 1);
 		}
@@ -574,7 +580,7 @@ latex2e_write_multicolumn_cell (FILE *fp, const Cell *cell, const int num_merged
 		fprintf (fp, "\\multirow{%d}[%i]*{\\begin{tabular}{p{",
 			 num_merged_rows, num_merged_rows/2);
 		for (i = 0; i < num_merged_cols; i++) {
-			fprintf (fp, "\t\\gnumericCol%s+%%\n", col_name (cell->col_info->pos + i));
+			fprintf (fp, "\t\\gnumericCol%s+%%\n", col_name (cell->pos.col + i));
 		}
 		if (num_merged_cols > 2)
 			fprintf (fp, "\t\\tabcolsep*2*%i}}", num_merged_cols - 2);
@@ -630,14 +636,22 @@ latex2e_write_multicolumn_cell (FILE *fp, const Cell *cell, const int num_merged
 	if (!cell_is_blank (cell)) {
                 /* Check the foreground (text) colour. */
 		textColor = cell_get_render_color (cell);
-		if (textColor == NULL)
+		if (textColor == NULL && mstyle_is_element_set (mstyle, MSTYLE_COLOR_FORE))
 			textColor = mstyle_get_color (mstyle, MSTYLE_COLOR_FORE);
-		r = textColor->red;
-		g = textColor->green;
-		b = textColor->blue;
-		if (r != 0 || g != 0 || b != 0)
+		if (textColor == NULL)
+			r = g = b = 0;
+		else {
+			r = textColor->red;
+			g = textColor->green;
+			b = textColor->blue;
+		}
+		if (r != 0 || g != 0 || b != 0) {
+			char *locale;
+			locale = setlocale (LC_NUMERIC, "C");
 			fprintf (fp, "{\\color[rgb]{%.2f,%.2f,%.2f} ",
 				 (double)r/65535, (double)g/65535, (double)b/65535);
+			locale = setlocale (LC_NUMERIC, locale);
+		}
 
 		/* Establish the font's style for the styles that can be addressed by LaTeX.
 		 * More complicated efforts (like changing fonts) are left to the user.
@@ -656,9 +670,11 @@ latex2e_write_multicolumn_cell (FILE *fp, const Cell *cell, const int num_merged
 							   &cell_format_characteristic);
 		if (cell_format_family == FMT_NUMBER || cell_format_family == FMT_CURRENCY ||
 		    cell_format_family == FMT_PERCENT || cell_format_family == FMT_FRACTION ||
-		    cell_format_family == FMT_SCIENCE)
+		    cell_format_family == FMT_SCIENCE){
 			fprintf (fp, "$");
-
+		        if (mstyle_get_font_italic(mstyle))
+			    fprintf (fp, "\\gnumericmathit{");
+		}
 		/* Print the cell contents. */
 		rendered_string = cell_get_rendered_text (cell);
 		latex_fputs (rendered_string, fp);
@@ -667,7 +683,7 @@ latex2e_write_multicolumn_cell (FILE *fp, const Cell *cell, const int num_merged
 		if (cell_format_family == FMT_NUMBER || cell_format_family == FMT_CURRENCY ||
 		    cell_format_family == FMT_PERCENT || cell_format_family == FMT_FRACTION ||
 		    cell_format_family == FMT_SCIENCE)
-			fprintf (fp, "$");
+			fprintf (fp, "}$");
 
 		/* Close the styles for the cell. */
 		if (mstyle_get_font_italic (mstyle))
@@ -810,7 +826,7 @@ latex2e_print_hhline (FILE *fp, StyleBorderType *clines, int n, StyleBorderType
  */
 void
 latex_file_save (GnumFileSaver const *fs, IOContext *io_context,
-                   WorkbookView *wb_view, const gchar *file_name)
+                   WorkbookView *wb_view, gchar const *file_name)
 {
 	FILE *fp;
 	Cell *cell;
