@@ -6,6 +6,7 @@
  *    Sean Atkinson <sca20@cam.ac.uk>
  **/
 #include <gnumeric-config.h>
+#include <gnumeric-i18n.h>
 #include <gnumeric.h>
 #include "xbase.h"
 
@@ -21,14 +22,12 @@
 #include <ranges.h>
 #include <mstyle.h>
 #include <sheet-style.h>
-
-#include <libgnome/gnome-i18n.h>
-#include <libgnome/gnome-util.h>
+#include <io-context.h>
 
 GNUMERIC_MODULE_PLUGIN_INFO_DECL;
 
 void xbase_file_open (GnumFileOpener const *fo, IOContext *io_context,
-                      WorkbookView *wb_view, char const *filename);
+                      WorkbookView *wb_view, GsfInput *input);
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
 
@@ -74,16 +73,19 @@ xb_setdouble (guint8 *p, double d)
 #endif
 
 static Value *
-xbase_field_as_value (gchar *content, XBfield *field)
+xbase_field_as_value (gchar *content, XBfield *field, XBfile *file)
 {
 	gchar *s = g_strndup (content, field->len);
 	Value *val;
 
 	switch (field->type) {
-	case 'C':
-		val = value_new_string (g_strchomp (s));
+	case 'C': {
+		val = value_new_string_nocopy (
+			g_convert_with_iconv (g_strchomp (s), -1,
+			      file->char_map, NULL, NULL, NULL));
 		g_free (s);
 		return val;
+	}
 	case 'N':
 		val = value_new_int (atoi (s));
 		g_free (s);
@@ -146,7 +148,7 @@ xbase_field_as_value (gchar *content, XBfield *field)
 
 void
 xbase_file_open (GnumFileOpener const *fo, IOContext *io_context,
-                 WorkbookView *wb_view, char const *filename)
+                 WorkbookView *wb_view, GsfInput *input)
 {
 	Workbook  *wb;
 	XBfile	  *file;
@@ -159,19 +161,16 @@ xbase_file_open (GnumFileOpener const *fo, IOContext *io_context,
 	ErrorInfo *open_error;
 	guint row, i;
 
-	if ((file = xbase_open (filename, &open_error)) == NULL) {
+	if ((file = xbase_open (input, &open_error)) == NULL) {
 		gnumeric_io_error_info_set (io_context, error_info_new_str_with_details (
 		                            _("Error while opening xbase file."),
 		                            open_error));
 		return;
 	}
 
-	name = g_strdup(filename);
-
-	*((gchar *) g_extension_pointer (name)) = '\0'; /* remove "dbf" */
-
 	wb = wb_view_workbook (wb_view);
-	sheet = sheet_new (wb, g_basename (name));
+	name = workbook_sheet_get_free_name (wb, _("Sheet"), FALSE, TRUE);
+	sheet = sheet_new (wb, name);
 	g_free (name);
 	workbook_sheet_attach (wb, sheet, NULL);
 
@@ -194,7 +193,7 @@ xbase_file_open (GnumFileOpener const *fo, IOContext *io_context,
 		for (i = 0; i < file->fields ; i++) {
 			field = record->file->format [i];
 			val = xbase_field_as_value (
-				record_get_field (record, i), field);
+				record_get_field (record, i), field, file);
 			cell = sheet_cell_fetch (sheet, i, row);
 			value_set_fmt (val, field->fmt);
 			cell_set_value (cell, val);
