@@ -14,8 +14,9 @@
 #include "selection.h"
 #include "workbook-control.h"
 #include "workbook-view.h"
-#include "workbook.h"
+#include "workbook-priv.h" /* For Workbook::name */
 #include "sheet.h"
+#include "sheet-view.h"
 #include "sheet-private.h"
 #include "auto-correct.h"
 #include "pixmaps/gnumeric-stock-pixbufs.h"
@@ -23,7 +24,6 @@
 
 #include <gnumeric-gconf.h>
 #include <gsf/gsf-impl-utils.h>
-#include <gal/util/e-util.h> /* e_free_string_slist */
 #include <gtk/gtk.h>
 
 /* Signals */
@@ -279,10 +279,9 @@ application_clipboard_area_get (void)
 	return NULL;
 }
 
-struct wb_name_closure
-{
+struct wb_name_closure {
 	Workbook *wb;
-	char const * name;
+	char const *name;
 };
 static gboolean
 cb_workbook_name (Workbook * wb, gpointer closure)
@@ -479,7 +478,8 @@ application_history_write_config (void)
 	gnm_gconf_set_file_history_files (app->history_list);
 	gnm_conf_sync ();
 
-	e_free_string_slist (app->history_list);
+	g_slist_foreach (app->history_list, (GFunc)g_free, NULL);
+	g_slist_free (app->history_list);
 	app->history_list = NULL;
 }
 
@@ -593,14 +593,16 @@ gnumeric_application_setup_icons (void)
 		{ gnm_object_radiobutton_24,		NULL,				"Gnumeric_ObjectRadiobutton" },
 		{ gnm_object_scrollbar_24,		NULL,				"Gnumeric_ObjectScrollbar" },
 
-		{ gnm_padlock,		                NULL,				"Gnumeric_Padlock" },
-		{ gnm_padlock_no,       		NULL,				"Gnumeric_PadlockNo" },
+		{ gnm_protection_yes,	                NULL,				"Gnumeric_Protection_Yes" },
+		{ gnm_protection_no,       		NULL,				"Gnumeric_Protection_No" },
 
 		{ gnm_link_add_24,			gnm_link_add_16,		"Gnumeric_Link_Add" },
 		{ NULL,					gnm_link_delete_16,		"Gnumeric_Link_Delete" },
 		{ NULL,					gnm_link_edit_16,		"Gnumeric_Link_Edit" },
-		{ gnm_link_external_24,			NULL,				"Gnumeric_Link_External" },
-		{ gnm_link_internal_24,			NULL,				"Gnumeric_Link_Internal" },
+		{ gnm_link_external_24,			gnm_link_external_16,		"Gnumeric_Link_External" },
+		{ gnm_link_internal_24,			gnm_link_internal_16,		"Gnumeric_Link_Internal" },
+		{ gnm_link_email_24,			gnm_link_email_16,		"Gnumeric_Link_EMail" },
+		{ gnm_link_url_24,			gnm_link_url_16,		"Gnumeric_Link_URL" },
 	};
 	unsigned i = 0;
 	GtkIconFactory *factory = gtk_icon_factory_new ();
