@@ -55,7 +55,9 @@ typedef enum {
 void auto_format_init (void);
 void auto_format_shutdown (void);
 
-void auto_format_function_result (FunctionDefinition *fd, AutoFormatTypes res);
-StyleFormat *auto_style_format_suggest (const ExprTree *expr, const EvalPos *epos);
+void auto_format_function_result (GnmFunc *fd, AutoFormatTypes res);
+void auto_format_function_result_by_name (const char *func, AutoFormatTypes res);
+void auto_format_function_result_remove (const char *func);
+StyleFormat *auto_style_format_suggest (const GnmExpr *expr, const EvalPos *epos);
 
 #endif
