@@ -20,13 +20,13 @@
  */
 
 #include <gnumeric-config.h>
+#include <gnumeric-i18n.h>
 #include <gnumeric.h>
 #include "dialog-stf-export.h"
 #include "dialog-stf-export-private.h"
 
 #include <command-context.h>
-
-#include <libgnome/gnome-i18n.h>
+#include <gdk/gdkkeysyms.h>
 
 #define GLADE_FILE "dialog-stf-export.glade"
 
@@ -41,25 +41,11 @@
  * returns : TRUE if the user actually wants to cancel, FALSE otherwise.
  **/
 static gboolean
-stf_export_dialog_druid_page_cancel (GnomeDruidPage *page, GnomeDruid *druid, StfE_DruidData_t *druid_data)
+stf_export_dialog_druid_page_cancel (GnomeDruidPage *page, GnomeDruid *druid,
+				     StfE_DruidData_t *druid_data)
 {
-	GtkWidget *dialog, *no_button;
-	int ret;
-
-	g_return_val_if_fail (page != NULL, TRUE);
-	g_return_val_if_fail (druid != NULL, TRUE);
-	g_return_val_if_fail (druid_data != NULL, TRUE);
-
-	dialog = gnome_question_dialog_parented (_("Are you sure you want to cancel?"),
-						 NULL,
-						 NULL,
-						 druid_data->window);
-
-	no_button = g_list_last (GNOME_DIALOG (dialog)->buttons)->data;
-	gtk_widget_grab_focus (no_button);
-	ret = gnome_dialog_run (GNOME_DIALOG (dialog));
-
-	return (ret==1);
+	return !gnumeric_dialog_question_yes_no (druid_data->wbcg,
+		_("Are you sure you want to cancel?"), FALSE);
 }
 
 /**
@@ -224,7 +210,7 @@ stf_export_dialog_druid_page_previous (GnomeDruidPage *page, GnomeDruid *druid,
 	stf_export_dialog_set_initial_keyboard_focus (druid_data);
 
 	if (newpos == DPG_SHEET)
-		gnome_druid_set_buttons_sensitive (druid, FALSE, TRUE, TRUE);
+		gnome_druid_set_buttons_sensitive (druid, FALSE, TRUE, TRUE, TRUE);
 	else
 		gtk_widget_grab_default (druid_data->druid->next);
 
@@ -298,46 +284,39 @@ stf_export_dialog_attach_page_signals (GladeXML *gui, StfE_DruidData_t *druid_da
 
 	druid_data->active_page  = DPG_SHEET;
 
-	gnome_druid_set_buttons_sensitive (druid_data->druid, FALSE, TRUE, TRUE);
+	gnome_druid_set_buttons_sensitive (druid_data->druid, FALSE, TRUE, TRUE, TRUE);
 
 	/* Signals for individual pages */
 
-	gtk_signal_connect (GTK_OBJECT (druid_data->sheet_page),
-			    "next",
-			    GTK_SIGNAL_FUNC (stf_export_dialog_druid_page_next),
-			    druid_data);
+	g_signal_connect (G_OBJECT (druid_data->sheet_page),
+		"next",
+		G_CALLBACK (stf_export_dialog_druid_page_next), druid_data);
 
-        gtk_signal_connect (GTK_OBJECT (druid_data->format_page),
-			    "back",
-			    GTK_SIGNAL_FUNC (stf_export_dialog_druid_page_previous),
-			    druid_data);
-	gtk_signal_connect (GTK_OBJECT (druid_data->format_page),
-			    "finish",
-			    GTK_SIGNAL_FUNC (stf_export_dialog_druid_format_page_finish),
-			    druid_data);
+        g_signal_connect (G_OBJECT (druid_data->format_page),
+		"back",
+		G_CALLBACK (stf_export_dialog_druid_page_previous), druid_data);
+	g_signal_connect (G_OBJECT (druid_data->format_page),
+		"finish",
+		G_CALLBACK (stf_export_dialog_druid_format_page_finish), druid_data);
 
-	gtk_signal_connect (GTK_OBJECT (druid_data->sheet_page),
-			    "cancel",
-			    GTK_SIGNAL_FUNC (stf_export_dialog_druid_page_cancel),
-			    druid_data);
-	gtk_signal_connect (GTK_OBJECT (druid_data->format_page),
-			    "cancel",
-			    GTK_SIGNAL_FUNC (stf_export_dialog_druid_page_cancel),
-			    druid_data);
+	g_signal_connect (G_OBJECT (druid_data->sheet_page),
+		"cancel",
+		G_CALLBACK (stf_export_dialog_druid_page_cancel), druid_data);
+	g_signal_connect (G_OBJECT (druid_data->format_page),
+		"cancel",
+		G_CALLBACK (stf_export_dialog_druid_page_cancel), druid_data);
 
 	/* Signals for the druid itself */
 
-	gtk_signal_connect (GTK_OBJECT (druid_data->druid),
-			    "cancel",
-			    GTK_SIGNAL_FUNC (stf_export_dialog_druid_cancel),
-			    druid_data);
+	g_signal_connect (G_OBJECT (druid_data->druid),
+		"cancel",
+		G_CALLBACK (stf_export_dialog_druid_cancel), druid_data);
 
 	/* And for the surrounding window */
 
-	gtk_signal_connect (GTK_OBJECT (druid_data->window),
-			    "key_press_event",
-			    GTK_SIGNAL_FUNC (stf_export_dialog_check_escape),
-			    druid_data);
+	g_signal_connect (G_OBJECT (druid_data->window),
+		"key_press_event",
+		G_CALLBACK (stf_export_dialog_check_escape), druid_data);
 }
 
 /**
@@ -352,7 +331,7 @@ static void
 stf_export_dialog_editables_enter (StfE_DruidData_t *druid_data)
 {
 	gnumeric_editable_enters (druid_data->window,
-				  GTK_EDITABLE (druid_data->format_page_data->format_custom));
+				  GTK_WIDGET (druid_data->format_page_data->format_custom));
 	gnumeric_combo_enters (druid_data->window,
 			       druid_data->format_page_data->format_quotechar);
 }
@@ -393,6 +372,7 @@ stf_export_dialog (WorkbookControlGUI *wbcg, Workbook *wb)
 		return NULL;
 	}
 
+	druid_data.wbcg             = wbcg;
 	druid_data.canceled         = FALSE;
 	druid_data.sheet_page_data  = stf_export_dialog_sheet_page_init (gui, wb);
 	druid_data.format_page_data = stf_export_dialog_format_page_init (gui);
@@ -429,7 +409,7 @@ stf_export_dialog (WorkbookControlGUI *wbcg, Workbook *wb)
 	stf_export_dialog_format_page_cleanup (druid_data.format_page_data);
 
 	gtk_widget_destroy (GTK_WIDGET (druid_data.window));
-	gtk_object_unref (GTK_OBJECT (gui));
+	g_object_unref (G_OBJECT (gui));
 
 	return dialogresult;
 }
