@@ -25,7 +25,7 @@
 #include "dialogs.h"
 
 #include <gui-util.h>
-#include <workbook.h>
+#include <workbook-priv.h> /* for Workbook::summary_info */
 #include <workbook-edit.h>
 #include <commands.h>
 
@@ -105,7 +105,7 @@ dialog_summary_get (SummaryState *state)
 	}
 
 	if (changes)
-		return cmd_change_summary (state->wbcg, changes);
+		return cmd_change_summary (WORKBOOK_CONTROL (state->wbcg), changes);
 	return FALSE;
 }
 
