@@ -39,6 +39,7 @@
 #include <workbook-control.h>
 #include <workbook-view.h>
 #include <workbook-edit.h>
+#include <workbook-priv.h>
 #include <workbook.h>
 #include <commands.h>
 
@@ -46,15 +47,16 @@
 
 #define WORKBOOK_ATTRIBUTE_KEY "workbook-attribute-dialog"
 
-typedef struct _AttrState
-{
-	GladeXML	 *gui;
-	GtkWidget        *dialog;
-	GtkWidget        *notebook;
-	GtkWidget        *ok_button;
-	GtkWidget        *apply_button;
+typedef struct {
+	GladeXML	*gui;
+	GtkWidget	*dialog;
+	GtkWidget	*notebook;
+	GtkWidget	*ok_button;
+	GtkWidget	*apply_button;
+	GtkWidget	*iteration_table;
 	gboolean         destroying;
 
+	Workbook	 *wb;
 	WorkbookView     *wbv;
 	WorkbookControlGUI	 *wbcg;
 
@@ -64,6 +66,10 @@ typedef struct _AttrState
 		GtkToggleButton	*show_tabs;
 		GtkToggleButton	*autocomplete;
 		GtkToggleButton	*is_protected;
+		GtkToggleButton	*recalc_auto;
+		GtkToggleButton *iteration_enabled;
+		GtkEntry	*max_iterations;
+		GtkEntry	*iteration_tolerance;
 	} view;
 	struct {
 		gboolean	show_hsb;
@@ -71,6 +77,10 @@ typedef struct _AttrState
 		gboolean	show_tabs;
 		gboolean	autocomplete;
 		gboolean	is_protected;
+		gboolean	recalc_auto;
+		gboolean	iteration_enabled;
+		int		max_iterations;
+		double		iteration_tolerance;
 	} old;
 } AttrState;
 
@@ -95,22 +105,70 @@ cb_page_select (GtkNotebook *notebook, GtkNotebookPage *page,
 
 /*****************************************************************************/
 
+static void
+get_entry_values (AttrState *state, int *max_iterations, double *iteration_tolerance)
+{
+	if (!entry_to_int (state->view.max_iterations, max_iterations, TRUE))
+		*max_iterations = state->old.max_iterations;
+	if (!entry_to_float (state->view.iteration_tolerance, iteration_tolerance, TRUE))
+		*iteration_tolerance = state->old.iteration_tolerance;
+}
+
+static void
+cb_widget_changed (GtkWidget *widget, AttrState *state)
+{
+	gboolean changed;
+	int max_iterations;
+	double iteration_tolerance;
+	
+	get_entry_values (state, &max_iterations, &iteration_tolerance);
+	changed =
+		!((gtk_toggle_button_get_active (state->view.show_hsb) == state->old.show_hsb) &&
+		  (gtk_toggle_button_get_active (state->view.show_vsb) == state->old.show_vsb) &&
+		  (gtk_toggle_button_get_active (state->view.show_tabs) == state->old.show_tabs) &&
+		  (gtk_toggle_button_get_active (state->view.autocomplete) == state->old.autocomplete) &&
+		  (gtk_toggle_button_get_active (state->view.is_protected) == state->old.is_protected) &&
+		  (gtk_toggle_button_get_active (state->view.recalc_auto) == state->old.recalc_auto) &&
+		  (gtk_toggle_button_get_active (state->view.iteration_enabled) == state->old.iteration_enabled) &&
+		  (max_iterations == state->old.max_iterations) &&
+		  (iteration_tolerance == state->old.iteration_tolerance));
+
+	gtk_widget_set_sensitive (state->ok_button, changed);
+	gtk_widget_set_sensitive (state->apply_button, changed);
+
+	gtk_widget_set_sensitive (state->iteration_table,
+		gtk_toggle_button_get_active (state->view.iteration_enabled));
+}
+
 /* Handler for the apply button */
 static void
 cb_attr_dialog_dialog_apply (GtkWidget *button, AttrState *state)
 {
-	state->wbv->show_horizontal_scrollbar =
+	state->wbv->show_horizontal_scrollbar = state->old.show_hsb =
 		gtk_toggle_button_get_active (state->view.show_hsb);
-	state->wbv->show_vertical_scrollbar =
+	state->wbv->show_vertical_scrollbar = state->old.show_vsb =
 		gtk_toggle_button_get_active (state->view.show_vsb);
-	state->wbv->show_notebook_tabs =
+	state->wbv->show_notebook_tabs = state->old.show_tabs =
 		gtk_toggle_button_get_active (state->view.show_tabs);
-	state->wbv->do_auto_completion =
+	state->wbv->do_auto_completion = state->old.autocomplete =
 		gtk_toggle_button_get_active (state->view.autocomplete);
-	state->wbv->is_protected =
+	state->wbv->is_protected = state->old.is_protected =
 		gtk_toggle_button_get_active (state->view.is_protected);
 
+	state->old.recalc_auto =
+		gtk_toggle_button_get_active (state->view.recalc_auto);
+	state->old.iteration_enabled =
+		gtk_toggle_button_get_active (state->view.iteration_enabled);
+
+	get_entry_values (state, &state->old.max_iterations,
+			  &state->old.iteration_tolerance);
+	workbook_autorecalc_enable	(state->wb, state->old.recalc_auto);
+	workbook_iteration_enabled	(state->wb, state->old.iteration_enabled);
+	workbook_iteration_max_number	(state->wb, state->old.max_iterations);
+	workbook_iteration_tolerance	(state->wb, state->old.iteration_tolerance);
+
 	wb_view_prefs_update (state->wbv);
+	cb_widget_changed (NULL, state);
 }
 
 static void
@@ -127,13 +185,9 @@ cb_attr_dialog_dialog_ok (GtkWidget *button, AttrState *state)
 	cb_attr_dialog_dialog_close (button, state);	
 }
 
-/* Handler for destroy */
-static gboolean
-cb_attr_dialog_dialog_destroy (GtkObject *w, AttrState *state)
+static void
+cb_attr_dialog_dialog_destroy (AttrState *state)
 {
-	g_return_val_if_fail (w != NULL, FALSE);
-	g_return_val_if_fail (state != NULL, FALSE);
-
 	wbcg_edit_detach_guru (state->wbcg);
 
 	if (state->gui != NULL) {
@@ -143,29 +197,10 @@ cb_attr_dialog_dialog_destroy (GtkObject *w, AttrState *state)
 
 	state->dialog = NULL;
 	g_free (state);
-
-	return FALSE;
 }
 
 /*****************************************************************************/
 
-static void
-cb_toggled (GtkWidget *widget, AttrState *state)
-{
-	gboolean changed = !((gtk_toggle_button_get_active (state->view.show_hsb) 
-			    == state->old.show_hsb) &&
-		(gtk_toggle_button_get_active (state->view.show_vsb) 
-		 == state->old.show_vsb) &&
-		(gtk_toggle_button_get_active (state->view.show_tabs) 
-		 == state->old.show_tabs) &&
-		(gtk_toggle_button_get_active (state->view.autocomplete) 
-					       == state->old.autocomplete) &&
-		(gtk_toggle_button_get_active (state->view.is_protected) 
-		 == state->old.is_protected));
-	gtk_widget_set_sensitive (state->ok_button, changed);
-	gtk_widget_set_sensitive (state->apply_button, changed);
-}
-
 static GtkToggleButton *
 attr_dialog_init_toggle (AttrState *state, char const *name, gboolean val,
 			 gboolean *storage)
@@ -174,15 +209,29 @@ attr_dialog_init_toggle (AttrState *state, char const *name, gboolean val,
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), val);
 	g_signal_connect (G_OBJECT (w),
 		"toggled",
-		G_CALLBACK (cb_toggled), state);
+		G_CALLBACK (cb_widget_changed), state);
 	*storage = val;
 
 	return GTK_TOGGLE_BUTTON (w);
 }
 
+static GtkEntry *
+attr_dialog_init_entry (AttrState *state, char const *name, char const *val)
+{
+	GtkWidget *w = glade_xml_get_widget (state->gui, name);
+	gtk_entry_set_text (GTK_ENTRY (w), val);
+	g_signal_connect (G_OBJECT (w),
+		"changed",
+		G_CALLBACK (cb_widget_changed), state);
+	gnumeric_editable_enters (GTK_WINDOW (state->dialog), w);
+	return GTK_ENTRY (w);
+}
+
 static void
 attr_dialog_init_view_page (AttrState *state)
 {
+	char buf[128];
+
 	state->view.show_hsb     = attr_dialog_init_toggle (state,
 		"WorkbookView::show_horizontal_scrollbar",
 		state->wbv->show_horizontal_scrollbar,
@@ -203,6 +252,27 @@ attr_dialog_init_view_page (AttrState *state)
 		"WorkbookView::workbook_protected",
 		state->wbv->is_protected,
 		&state->old.is_protected);
+	if (!state->wb->recalc_auto) {
+		GtkWidget *w = glade_xml_get_widget (state->gui, "recalc_manual");
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), TRUE);
+	}
+	state->view.recalc_auto = attr_dialog_init_toggle (state,
+		"recalc_auto",
+		state->wb->recalc_auto,
+		&state->old.recalc_auto);
+	state->view.iteration_enabled = attr_dialog_init_toggle (state,
+		"iteration_enabled",
+		state->wb->iteration.enabled,
+		&state->old.iteration_enabled);
+
+	g_snprintf (buf, sizeof(buf), "%d", state->wb->iteration.max_number);
+	state->old.max_iterations = state->wb->iteration.max_number;
+	state->view.max_iterations =
+		attr_dialog_init_entry (state, "max_iterations", buf);
+	g_snprintf (buf, sizeof(buf), "%g", state->wb->iteration.tolerance);
+	state->old.iteration_tolerance = state->wb->iteration.tolerance;
+	state->view.iteration_tolerance =
+		attr_dialog_init_entry (state, "iteration_tolerance", buf);
 }
 
 /*****************************************************************************/
@@ -229,7 +299,7 @@ attr_dialog_impl (AttrState *state)
 		"switch_page",
 		G_CALLBACK (cb_page_select), state);
 
-
+	state->iteration_table = glade_xml_get_widget (state->gui, "iteration_table");
 	state->ok_button = glade_xml_get_widget (state->gui, "ok_button");
 	g_signal_connect (G_OBJECT (state->ok_button),
 			  "clicked",
@@ -241,17 +311,16 @@ attr_dialog_impl (AttrState *state)
 	g_signal_connect (G_OBJECT (glade_xml_get_widget (state->gui, "close_button")),
 			  "clicked",
 			  G_CALLBACK (cb_attr_dialog_dialog_close), state);
+	cb_widget_changed (NULL, state);
+
 /* FIXME: Add correct helpfile address */
 	gnumeric_init_help_button (
 		glade_xml_get_widget (state->gui, "help_button"),
 		"workbook-attributes.html");
 
-	g_signal_connect (G_OBJECT (dialog),
-		"destroy",
-		G_CALLBACK (cb_attr_dialog_dialog_destroy), state);
-
-	cb_toggled (NULL, state);
-
+	/* a candidate for merging into attach guru */
+	g_object_set_data_full (G_OBJECT (dialog),
+		"state", state, (GDestroyNotify) cb_attr_dialog_dialog_destroy);
 	wbcg_edit_attach_guru (state->wbcg, state->dialog);
 	gnumeric_keyed_dialog (state->wbcg, GTK_WINDOW (state->dialog),
 			       WORKBOOK_ATTRIBUTE_KEY);
@@ -278,6 +347,7 @@ dialog_workbook_attr (WorkbookControlGUI *wbcg)
 	state->gui = gui;
 	state->wbcg = wbcg;
 	state->wbv  = wb_control_view (WORKBOOK_CONTROL (wbcg));
+	state->wb   = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
 
 	attr_dialog_impl (state);
 }
