@@ -36,7 +36,7 @@
 #include "parse-util.h"
 #include "ranges.h"
 #include "number-match.h"
-#include "workbook.h"
+#include "workbook-priv.h"
 #include "gutils.h"
 #include "parse-util.h"
 #include "mathfunc.h"
@@ -77,6 +77,12 @@ gnm_expr_constant_eq (GnmExprConstant const *a,
 {
 }
 #endif
+/**
+ * gnm_expr_new_constant :
+ * @v :
+ *
+ * Absorbs the value.
+ **/
 GnmExpr const *
 gnm_expr_new_constant (Value *v)
 {
@@ -110,7 +116,7 @@ gnm_expr_function_eq (GnmExprFunction const *a,
 #endif
 
 GnmExpr const *
-gnm_expr_new_funcall (FunctionDefinition *func, GnmExprList *args)
+gnm_expr_new_funcall (GnmFunc *func, GnmExprList *args)
 {
 	GnmExprFunction *ans;
 	g_return_val_if_fail (func, NULL);
@@ -121,7 +127,7 @@ gnm_expr_new_funcall (FunctionDefinition *func, GnmExprList *args)
 
 	ans->ref_count = 1;
 	ans->oper = GNM_EXPR_OP_FUNCALL;
-	func_ref (func);
+	gnm_func_ref (func);
 	ans->func = func;;
 	ans->arg_list = args;
 
@@ -346,7 +352,7 @@ do_gnm_expr_unref (GnmExpr const *expr)
 
 	case GNM_EXPR_OP_FUNCALL:
 		gnm_expr_list_unref (expr->func.arg_list);
-		func_unref (expr->func.func);
+		gnm_func_unref (expr->func.func);
 		break;
 
 	case GNM_EXPR_OP_NAME:
@@ -375,9 +381,12 @@ do_gnm_expr_unref (GnmExpr const *expr)
 	case GNM_EXPR_OP_SET:
 		gnm_expr_list_unref (expr->set.set);
 		break;
+
+#ifndef DEBUG_SWITCH_ENUM
 	default:
-		g_warning ("do_gnm_expr_unref error.");
+		g_assert_not_reached ();
 		break;
+#endif
 	}
 
 	CHUNK_FREE (expression_pool, (gpointer)expr);
@@ -446,7 +455,9 @@ gnm_expr_equal (GnmExpr const *a, GnmExpr const *b)
 			gnm_expr_list_equal (a->func.arg_list, b->func.arg_list);
 
 	case GNM_EXPR_OP_NAME:
-		return a->name.name == b->name.name;
+		return	a->name.name == b->name.name &&
+			a->name.optional_scope == b->name.optional_scope &&
+			a->name.optional_wb_scope == b->name.optional_wb_scope;
 
 	case GNM_EXPR_OP_CELLREF:
 		return cellref_equal (&a->cellref.ref, &b->cellref.ref);
@@ -483,13 +494,6 @@ gnm_expr_equal (GnmExpr const *a, GnmExpr const *b)
 	return FALSE;
 }
 
-static Value *
-cb_range_eval (Sheet *sheet, int col, int row, Cell *cell, void *ignore)
-{
-	cell_eval (cell);
-	return NULL;
-}
-
 static Cell *
 expr_array_corner (GnmExpr const *expr,
 			Sheet const *sheet, CellPos const *pos)
@@ -547,16 +551,16 @@ gnm_expr_extract_ref (CellRef *res, GnmExpr const *expr,
 	}
 
 	case GNM_EXPR_OP_NAME:
-		if (!expr->name.name->active || expr->name.name->builtin)
+		if (!expr->name.name->active)
 			return TRUE;
-		return gnm_expr_extract_ref (res, expr->name.name->t.expr_tree, pos, flags);
+		return gnm_expr_extract_ref (res, expr->name.name->expr_tree, pos, flags);
 	default :
 		break;
 	}
 	return TRUE;
 }
 
-static Value *
+static inline Value *
 handle_empty (Value *res, GnmExprEvalFlags flags)
 {
 	if (res == NULL)
@@ -571,6 +575,91 @@ handle_empty (Value *res, GnmExprEvalFlags flags)
 	return res;
 }
 
+/**
+ * value_intersection :
+ * @v   : a VALUE_CELLRANGE
+ * @pos : 
+ *
+ * Handle the implicit union of a single row or column with the eval position.
+ *
+ * NOTE : We do not need to know if this is expression is being evaluated as an
+ * array or not because we can differentiate based on the required type for the
+ * argument.
+ *
+ * Always release the value passed in.
+ *
+ * Return value:
+ *     If the intersection succeeded return a duplicate of the value
+ *     at the intersection point.  This value needs to be freed.
+ *     NULL if there is no intersection
+ * Returns the upper left corner of an array.
+ **/
+static Value *
+value_intersection (Value *v, EvalPos const *pos)
+{
+	Value *res = NULL;
+	Range r;
+	Sheet *start_sheet, *end_sheet;
+	gboolean found = FALSE;
+
+	if (v->type == VALUE_ARRAY) {
+		res = value_duplicate (v->v_array.vals[0][0]);
+		value_release (v);
+		return res;
+	}
+
+	/* Handle the implicit union of a single row or
+	 * column with the eval position.
+	 * NOTE : We do not need to know if this is expression is
+	 * being evaluated as an array or not because we can differentiate
+	 * based on the required type for the argument.
+	 */
+
+	/* inverted ranges */
+	rangeref_normalize (&v->v_range.cell, pos, &start_sheet, &end_sheet, &r);
+	value_release (v);
+
+	if (start_sheet == end_sheet || end_sheet == NULL) {
+		int col = pos->eval.col;
+		int row = pos->eval.row;
+
+		if (r.start.row == r.end.row) {
+			if (r.start.col <= col && col <= r.end.col) {
+				row = r.start.row;
+				found = TRUE;
+			} else if (r.start.col == r.end.col) {
+				col = r.start.col;
+				row = r.start.row;
+				found = TRUE;
+			}
+		} else if (r.start.col == r.end.col) {
+			if (r.start.row <= row && row <= r.end.row) {
+				col = r.start.col;
+				found = TRUE;
+			}
+		}
+		if (found) {
+			Cell *cell = sheet_cell_get (
+				eval_sheet (start_sheet, pos->sheet),
+				col, row);
+			if (cell == NULL)
+				return value_new_empty ();
+			cell_eval (cell);
+			return value_duplicate (cell->value);
+		}
+	}
+
+	return value_new_error (pos, gnumeric_err_VALUE);
+}
+#if 0
+static Value *
+cb_range_eval (Sheet *sheet, int col, int row, Cell *cell, void *ignore)
+{
+	cell_eval (cell);
+	return NULL;
+}
+#endif
+
 /**
  * gnm_expr_eval :
  * @expr :
@@ -598,33 +687,17 @@ gnm_expr_eval (GnmExpr const *expr, EvalPos const *pos,
 	case GNM_EXPR_OP_LTE: {
 		ValueCompare comp;
 
-		a = gnm_expr_eval (expr->binary.value_a, pos,
-				   flags | GNM_EXPR_EVAL_PERMIT_EMPTY);
-		if (a != NULL) {
-			if (a->type == VALUE_CELLRANGE || a->type == VALUE_ARRAY) {
-				a = value_intersection (a, pos);
-				if (a == NULL)
-					return value_new_error (pos, gnumeric_err_VALUE);
-			} else if (a->type == VALUE_ERROR)
-				return a;
-		}
+		flags = (flags | GNM_EXPR_EVAL_PERMIT_EMPTY) & ~GNM_EXPR_EVAL_PERMIT_NON_SCALAR;
 
-		b = gnm_expr_eval (expr->binary.value_b, pos,
-				   flags | GNM_EXPR_EVAL_PERMIT_EMPTY);
-		if (b != NULL) {
-			Value *res = NULL;
-			if (b->type == VALUE_CELLRANGE || b->type == VALUE_ARRAY) {
-				b = value_intersection (b, pos);
-				if (b == NULL)
-					res = value_new_error (pos, gnumeric_err_VALUE);
-			} else if (b->type == VALUE_ERROR)
-				res = b;
+		a = gnm_expr_eval (expr->binary.value_a, pos, flags);
+		if (a != NULL && a->type == VALUE_ERROR)
+			return a;
 
-			if (res != NULL) {
-				if (a != NULL)
-					value_release (a);
-				return handle_empty (res, flags);
-			}
+		b = gnm_expr_eval (expr->binary.value_b, pos, flags);
+		if (b != NULL && b->type == VALUE_ERROR) {
+			if (a != NULL)
+				value_release (a);
+			return b;
 		}
 
 		comp = value_compare (a, b, FALSE);
@@ -672,10 +745,12 @@ gnm_expr_eval (GnmExpr const *expr, EvalPos const *pos,
 			res = value_new_bool (comp != IS_LESS);
 			break;
 
+#ifndef DEBUG_SWITCH_ENUM
 		default:
 			g_assert_not_reached ();
 			res = value_new_error (pos,
 						_("Internal type error"));
+#endif
 		}
 		return res;
 	}
@@ -694,18 +769,11 @@ gnm_expr_eval (GnmExpr const *expr, EvalPos const *pos,
 		 * 5) result of operation, or error specific to the operation
 		 */
 
-	        /* Guarantees a != NULL */
-		a = gnm_expr_eval (expr->binary.value_a, pos,
-				   flags & (~GNM_EXPR_EVAL_PERMIT_EMPTY));
-
-		/* Handle implicit intersection */
-		if (a->type == VALUE_CELLRANGE || a->type == VALUE_ARRAY) {
-			a = value_intersection (a, pos);
-			if (a == NULL)
-				return value_new_error (pos, gnumeric_err_VALUE);
-		}
+	        /* Guarantees value != NULL */
+		flags &= ~(GNM_EXPR_EVAL_PERMIT_EMPTY | GNM_EXPR_EVAL_PERMIT_NON_SCALAR);
 
 		/* 1) Error from A */
+		a = gnm_expr_eval (expr->binary.value_a, pos, flags);
 		if (a->type == VALUE_ERROR)
 			return value_error_set_pos (&a->v_err, pos);
 
@@ -722,18 +790,8 @@ gnm_expr_eval (GnmExpr const *expr, EvalPos const *pos,
 			return value_new_error (pos, gnumeric_err_VALUE);
 		}
 
-	        /* Guarantees that b != NULL */
-		b = gnm_expr_eval (expr->binary.value_b, pos,
-				   flags & (~GNM_EXPR_EVAL_PERMIT_EMPTY));
-
-		/* Handle implicit intersection */
-		if (b->type == VALUE_CELLRANGE || b->type == VALUE_ARRAY) {
-			b = value_intersection (a, pos);
-			if (b == NULL)
-				return value_new_error (pos, gnumeric_err_VALUE);
-		}
-
 		/* 3) Error from B */
+		b = gnm_expr_eval (expr->binary.value_b, pos, flags);
 		if (b->type == VALUE_ERROR) {
 			value_release (a);
 			return value_error_set_pos (&b->v_err, pos);
@@ -845,17 +903,10 @@ gnm_expr_eval (GnmExpr const *expr, EvalPos const *pos,
 	case GNM_EXPR_OP_PERCENTAGE:
 	case GNM_EXPR_OP_UNARY_NEG:
 	case GNM_EXPR_OP_UNARY_PLUS:
-	        /* Garantees that a != NULL */
-		a = gnm_expr_eval (expr->unary.value, pos,
-				   flags & (~GNM_EXPR_EVAL_PERMIT_EMPTY));
-
-		/* Handle implicit intersection */
-		if (a->type == VALUE_CELLRANGE || a->type == VALUE_ARRAY) {
-			a = value_intersection (a, pos);
-			if (a == NULL)
-				return value_new_error (pos, gnumeric_err_VALUE);
-		}
+	        /* Guarantees value != NULL */
+		flags &= ~(GNM_EXPR_EVAL_PERMIT_EMPTY | GNM_EXPR_EVAL_PERMIT_NON_SCALAR);
 
+		a = gnm_expr_eval (expr->unary.value, pos, flags);
 		if (a->type == VALUE_ERROR)
 			return a;
 
@@ -886,13 +937,11 @@ gnm_expr_eval (GnmExpr const *expr, EvalPos const *pos,
 		return res;
 
 	case GNM_EXPR_OP_CAT:
-#warning BUG does not support intersection
-		a = gnm_expr_eval (expr->binary.value_a, pos,
-				   flags | GNM_EXPR_EVAL_PERMIT_EMPTY);
+		flags = (flags | GNM_EXPR_EVAL_PERMIT_EMPTY) & ~GNM_EXPR_EVAL_PERMIT_NON_SCALAR;
+		a = gnm_expr_eval (expr->binary.value_a, pos, flags);
 		if (a != NULL && a->type == VALUE_ERROR)
 			return a;
-		b = gnm_expr_eval (expr->binary.value_b, pos,
-				   flags | GNM_EXPR_EVAL_PERMIT_EMPTY);
+		b = gnm_expr_eval (expr->binary.value_b, pos, flags);
 		if (b != NULL && b->type == VALUE_ERROR) {
 			if (a != NULL)
 				value_release (a);
@@ -922,7 +971,18 @@ gnm_expr_eval (GnmExpr const *expr, EvalPos const *pos,
 		FunctionEvalInfo ei;
 		ei.pos = pos;
 		ei.func_call = (GnmExprFunction const *)expr;
-		return handle_empty (function_call_with_list (&ei, expr->func.arg_list, flags), flags);
+		res = function_call_with_list (&ei, expr->func.arg_list, flags);
+		if (res != NULL && res->type == VALUE_CELLRANGE) {
+			dependent_add_dynamic_dep (pos->dep, &res->v_range);
+			if (!(flags & GNM_EXPR_EVAL_PERMIT_NON_SCALAR)) {
+				res = value_intersection (res, pos);
+				return (res != NULL)
+					? handle_empty (res, flags)
+					: value_new_error (pos, gnumeric_err_VALUE);
+			}
+			return res;
+		}
+		return handle_empty (res, flags);
 	}
 
 	case GNM_EXPR_OP_NAME:
@@ -947,59 +1007,24 @@ gnm_expr_eval (GnmExpr const *expr, EvalPos const *pos,
 		return handle_empty (value_duplicate (cell->value), flags);
 	}
 
-	case GNM_EXPR_OP_CONSTANT: {
-		Value const *res = expr->constant.value;
+	case GNM_EXPR_OP_CONSTANT:
+		res = value_duplicate (expr->constant.value);
 		if (res->type != VALUE_CELLRANGE)
-			return handle_empty (value_duplicate (res), flags);
+			return handle_empty (res, flags);
+
 		if (flags & GNM_EXPR_EVAL_PERMIT_NON_SCALAR) {
+#if 0
 			workbook_foreach_cell_in_range (pos, res,
 				CELL_ITER_IGNORE_BLANK,
 				cb_range_eval, NULL);
-			return value_duplicate (res);
+#endif
+			return res;
 		} else {
-			/* Handle the implicit union of a single row or
-			 * column with the eval position.
-			 * NOTE : We do not need to know if this is expression is
-			 * being evaluated as an array or not because we can differentiate
-			 * based on the required type for the argument.
-			 */
-			CellRef const * const ref_a = & res->v_range.cell.a;
-			CellRef const * const ref_b = & res->v_range.cell.b;
-			gboolean found = FALSE;
-
-			if (ref_a->sheet == ref_b->sheet) {
-				CellPos a, b;
-				int c = pos->eval.col;
-				int r = pos->eval.row;
-
-				cellref_get_abs_pos (ref_a, &pos->eval, &a);
-				cellref_get_abs_pos (ref_b, &pos->eval, &b);
-				if (a.row == b.row) {
-					if (a.col <= c && c <= b.col) {
-						r = a.row;
-						found = TRUE;
-					}
-				} else if (a.col == b.col) {
-					if (a.row <= r && r <= b.row) {
-						c = a.col;
-						found = TRUE;
-					}
-				}
-				if (found) {
-					Cell * cell = sheet_cell_get (
-						eval_sheet (ref_a->sheet, pos->sheet),
-						c, r);
-					if (cell == NULL)
-						return handle_empty (NULL, flags);
-
-					cell_eval (cell);
-
-					return handle_empty (value_duplicate (cell->value), flags);
-				}
-			}
-			return value_new_error (pos, gnumeric_err_VALUE);
+			res = value_intersection (res, pos);
+			return (res != NULL)
+				? handle_empty (res, flags)
+				: value_new_error (pos, gnumeric_err_VALUE);
 		}
-	}
 
 	case GNM_EXPR_OP_ARRAY: {
 		/* The upper left corner manages the recalc of the expr */
@@ -1012,7 +1037,7 @@ gnm_expr_eval (GnmExpr const *expr, EvalPos const *pos,
 				value_release (a);
 
 			a = gnm_expr_eval (expr->array.corner.expr, pos,
-					   GNM_EXPR_EVAL_PERMIT_NON_SCALAR);
+				flags | GNM_EXPR_EVAL_PERMIT_NON_SCALAR);
 
 			/* Store real result (cast away const)*/
 			*((Value **)&(expr->array.corner.value)) = a;
@@ -1028,8 +1053,8 @@ gnm_expr_eval (GnmExpr const *expr, EvalPos const *pos,
 
 		if (a != NULL &&
 		    (a->type == VALUE_CELLRANGE || a->type == VALUE_ARRAY)) {
-			int const num_x = value_area_get_width (pos, a);
-			int const num_y = value_area_get_height (pos, a);
+			int const num_x = value_area_get_width (a, pos);
+			int const num_y = value_area_get_height (a, pos);
 
 			/* Evaluate relative to the upper left corner */
 			EvalPos tmp_ep = *pos;
@@ -1044,7 +1069,7 @@ gnm_expr_eval (GnmExpr const *expr, EvalPos const *pos,
 			if (x >= num_x || y >= num_y)
 				return value_new_error (pos, gnumeric_err_NA);
 
-			a = (Value *)value_area_get_x_y (&tmp_ep, a, x, y);
+			a = (Value *)value_area_get_x_y (a, x, y, &tmp_ep);
 		}
 
 		return handle_empty ((a != NULL) ? value_duplicate (a) : NULL, flags);
@@ -1057,8 +1082,18 @@ gnm_expr_eval (GnmExpr const *expr, EvalPos const *pos,
 		if (gnm_expr_extract_ref (&a, expr->binary.value_a, pos, flags) ||
 		    gnm_expr_extract_ref (&b, expr->binary.value_b, pos, flags))
 			return value_new_error (pos, gnumeric_err_REF);
-		return value_new_cellrange (&a, &b, pos->eval.col, pos->eval.row);
+
+		res = value_new_cellrange (&a, &b, pos->eval.col, pos->eval.row);
+		dependent_add_dynamic_dep (pos->dep, &res->v_range);
+		if (!(flags & GNM_EXPR_EVAL_PERMIT_NON_SCALAR)) {
+			res = value_intersection (res, pos);
+			return (res != NULL)
+				? handle_empty (res, flags)
+				: value_new_error (pos, gnumeric_err_VALUE);
+		}
+		return res;
 	}
+
 	case GNM_EXPR_OP_INTERSECT: {
 		CellRef a, b;
 		if (gnm_expr_extract_ref (&a, expr->binary.value_a, pos, flags) ||
@@ -1099,19 +1134,19 @@ do_expr_as_string (GnmExpr const *expr, ParsePos const *pp,
 		{ "-",  3, 1, 0 },
 		{ "*",  4, 1, 0 },
 		{ "/",  4, 1, 0 },
-		{ "^",  6, 0, 1 },
+		{ "^",  5, 0, 1 },
 		{ "&",  2, 1, 0 },
 		{ NULL, 0, 0, 0 }, /* Funcall  */
 		{ NULL, 0, 0, 0 }, /* Name     */
 		{ NULL, 0, 0, 0 }, /* Constant */
 		{ NULL, 0, 0, 0 }, /* Var      */
-		{ "-",  5, 0, 0 }, /* Unary -  */
-		{ "+",  5, 0, 0 }, /* Unary +  */
-		{ "%",  5, 0, 0 }, /* Percentage (NOT MODULO) */
+		{ "-",  7, 0, 0 }, /* Unary -  */
+		{ "+",  7, 0, 0 }, /* Unary +  */
+		{ "%",  6, 0, 0 }, /* Percentage (NOT MODULO) */
 		{ NULL, 0, 0, 0 }, /* Array    */
 		{ NULL, 0, 0, 0 }, /* Set      */
-		{ ":", 1, 1, 0 },  /* Range Ctor   */
-		{ " ", 1, 1, 0 }   /* Intersection */
+		{ ":",  9, 1, 0 }, /* Range Ctor   */
+		{ " ",  8, 1, 0 }  /* Intersection */
 	};
 	int const op = expr->any.oper;
 
@@ -1168,9 +1203,9 @@ do_expr_as_string (GnmExpr const *expr, ParsePos const *pp,
 
 		if (arg_list != NULL)
 			return gnm_expr_list_as_string (arg_list, pp,
-				function_def_get_name (expr->func.func));
+				gnm_func_get_name (expr->func.func));
 		else
-			return g_strconcat (function_def_get_name (expr->func.func),
+			return g_strconcat (gnm_func_get_name (expr->func.func),
 					    "()", NULL);
 	}
 
@@ -1184,7 +1219,11 @@ do_expr_as_string (GnmExpr const *expr, ParsePos const *pp,
 					expr->name.name->name->str, NULL);
 			return g_strconcat (expr->name.optional_scope->name_quoted, "!",
 					    expr->name.name->name->str, NULL);
-		}
+		} else if (pp->sheet != NULL &&
+			   expr->name.name->pos.sheet != NULL &&
+			   expr->name.name->pos.sheet != pp->sheet)
+			return g_strconcat (expr->name.name->pos.sheet->name_quoted, "!",
+					    expr->name.name->name->str, NULL);
 
 		return g_strdup (expr->name.name->name->str);
 
@@ -1541,9 +1580,6 @@ gnm_expr_rewrite (GnmExpr const *expr, GnmExprRewriteInfo const *rwinfo)
 		GnmNamedExpr *nexpr = expr->name.name;
 		GnmExpr const *tmp;
 
-		if (nexpr->builtin)
-			return NULL;
-
 		/* we can not invalidate references to the name that are
 		 * sitting in the undo queue, or the clipboard.  So we just
 		 * flag the name as inactive and remove the reference here.
@@ -1581,7 +1617,7 @@ gnm_expr_rewrite (GnmExpr const *expr, GnmExprRewriteInfo const *rwinfo)
 		}
 
 		/* Do NOT rewrite the name.  Just invalidate the use of the name */
-		tmp = gnm_expr_rewrite (expr->name.name->t.expr_tree, rwinfo);
+		tmp = gnm_expr_rewrite (expr->name.name->expr_tree, rwinfo);
 		if (tmp != NULL) {
 			gnm_expr_unref (tmp);
 			return gnm_expr_new_constant (
@@ -1672,7 +1708,7 @@ gnm_expr_rewrite (GnmExpr const *expr, GnmExprRewriteInfo const *rwinfo)
 	return NULL;
 }
 
-FunctionDefinition *
+GnmFunc *
 gnm_expr_get_func_def (GnmExpr const *expr)
 {
 	g_return_val_if_fail (expr != NULL, NULL);
@@ -1910,15 +1946,32 @@ gnm_expr_get_range (GnmExpr const *expr)
 		return NULL;
 
 	case GNM_EXPR_OP_NAME:
-		if (!expr->name.name->active || expr->name.name->builtin)
+		if (!expr->name.name->active)
 			return NULL;
-		return gnm_expr_get_range (expr->name.name->t.expr_tree);
+		return gnm_expr_get_range (expr->name.name->expr_tree);
 
 	default:
 		return NULL;
 	}
 }
 
+/**
+ * gnm_expr_get_constant:
+ * @expr :
+ *
+ * If this expression consists of just a constant, return it.
+ */
+Value const *
+gnm_expr_get_constant (GnmExpr const *expr)
+{
+	g_return_val_if_fail (expr != NULL, NULL);
+
+	if (expr->any.oper != GNM_EXPR_OP_CONSTANT)
+		return NULL;
+
+	return expr->constant.value;
+}
+
 /**
  * gnm_expr_is_rangeref :
  * @expr :
@@ -1950,8 +2003,8 @@ gnm_expr_is_rangeref (GnmExpr const *expr)
 		return FALSE;
 
 	case GNM_EXPR_OP_NAME:
-		if (expr->name.name->active && !expr->name.name->builtin)
-			return gnm_expr_is_rangeref (expr->name.name->t.expr_tree);
+		if (expr->name.name->active)
+			return gnm_expr_is_rangeref (expr->name.name->expr_tree);
 		return FALSE;
 
 	case GNM_EXPR_OP_ARRAY: /* I don't think this is possible */
@@ -1960,6 +2013,18 @@ gnm_expr_is_rangeref (GnmExpr const *expr)
 	};
 }
 
+gboolean
+gnm_expr_is_err (GnmExpr const *expr, char const *msg)
+{
+	g_return_val_if_fail (expr != NULL, FALSE);
+	g_return_val_if_fail (msg != NULL, FALSE);
+
+	return (expr->any.oper == GNM_EXPR_OP_CONSTANT &&
+		expr->constant.value != NULL &&
+		expr->constant.value->type == VALUE_ERROR &&
+		!strcmp (expr->constant.value->v_err.mesg->str, msg));
+}
+
 void
 gnm_expr_list_unref (GnmExprList *list)
 {
@@ -2072,8 +2137,11 @@ ets_hash (gconstpointer key)
 	case GNM_EXPR_OP_CONSTANT:
 		return value_hash (expr->constant.value);
 
-	case GNM_EXPR_OP_CELLREF:
 	case GNM_EXPR_OP_NAME:
+		/* all we need is a somewhat unique hash, ignore int != ptr */
+		return (guint)(expr->name.name);
+
+	case GNM_EXPR_OP_CELLREF:
 	case GNM_EXPR_OP_ARRAY:
 		break;
 	}
