@@ -19,9 +19,10 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #include <gnumeric-config.h>
+#include <gnumeric-i18n.h>
 #include <gnumeric.h>
 #include <func.h>
-#include <libgnome/gnome-i18n.h>
+#include <func-builtin.h>
 #include <rangefunc.h>
 #include <auto-format.h>
 #include <collect.h>
@@ -36,8 +37,8 @@ static const char *help_sum = {
 
 	   "@DESCRIPTION="
 	   "SUM computes the sum of all the values and cells referenced "
-	   "in the argument list.\n"
-	   "This function is Excel compatible."
+	   "in the argument list.\n\n"
+	   "* This function is Excel compatible."
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "Let us assume that the cells A1, A2, ..., A5 contain numbers "
@@ -66,9 +67,9 @@ static const char *help_product = {
 
 	   "@DESCRIPTION="
 	   "PRODUCT returns the product of all the values and cells "
-	   "referenced in the argument list.\n"
-	   "This function is Excel compatible.  In particular, this means "
-	   "that if all cells are empty, the result will be 0."
+	   "referenced in the argument list.\n\n"
+	   "* This function is Excel compatible.  In particular, this means "
+	   "that if all cells are empty, the result will be 0.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "PRODUCT(2,5,9) equals 90.\n"
@@ -121,139 +122,61 @@ gnumeric_version (FunctionEvalInfo *ei, Value *argv [])
 
 /***************************************************************************/
 
-static const char *help_selection = {
-	N_("@FUNCTION=SELECTION\n"
-	   "@SYNTAX=SELECTION(permit_intersection)\n"
-
-	   "@DESCRIPTION="
-	   "SELECTION function returns a list with the values in the current "
-	   "selection.  This is usually used to implement on-the-fly computation "
-	   "of values.\n"
-	   "If @permit_intersection is TRUE the user specifed selection "
-	   "ranges are returned, EVEN IF THEY OVERLAP.  "
-	   "If @permit_intersection is FALSE a distict set of regions is "
-	   "returned, however, there may be more of them than "
-	   "the user initially specified."
-
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "\n"
-	   "@SEEALSO=")
-};
-
-typedef struct
-{
-	GSList * res;
-	int	index;
-} selection_accumulator;
-
-static void
-accumulate_regions (Sheet *sheet,  Range const *r, gpointer closure)
-{
-	selection_accumulator *accum = closure;
-	CellRef a, b;
-
-	/* Fill it in */
-	/* start */
-	a.sheet = sheet;
-	a.col_relative = a.row_relative = FALSE;
-	a.col = r->start.col;
-	a.row = r->start.row;
-
-	/* end */
-	b.sheet = sheet;
-	b.col_relative = b.row_relative = FALSE;
-	b.col = r->end.col;
-	b.row = r->end.row;
-
-	/* Dummy up the eval pos it does not matter */
-	accum->res = g_slist_prepend (accum->res,
-				      value_new_cellrange(&a, &b, 0, 0));
-	accum->index++;
-}
-
-/* This routine is used to implement the auto_expr functionality.  It is called
- * to provide the selection to the defined functions.
- */
-static Value *
-gnumeric_selection (FunctionEvalInfo *ei, Value *argv [])
-{
-	Sheet * const sheet = ei->pos->sheet;
-	gboolean const permit_intersection = argv [0]->v_bool.val;
-	Value * res;
-	int i;
-
-	selection_accumulator accum;
-	accum.res = NULL;
-	accum.index = 0;
-	selection_apply (sheet, &accumulate_regions,
-			 permit_intersection, &accum);
-
-	i = accum.index;
-	res = value_new_array_empty (i, 1);
-	while (i-- > 0) {
-		/* pop the 1st element off the list */
-		Value *range = accum.res->data;
-		accum.res = g_slist_remove (accum.res, range);
-
-		value_array_set (res, i, 0, range);
-	}
-	return res;
-}
-
-/***************************************************************************/
-
 static const char *mathcatname = N_("Maths / Trig.");
 static GSList *mathfuncs = NULL;
 
 static const char *gnumericcatname = N_("Gnumeric");
 static GSList *gnumericfuncs = NULL;
 
-static const char *sheetcatname = N_("Sheet");
-static GSList *sheetfuncs = NULL;
-
 void
 func_builtin_init (void)
 {
-	FunctionDefinition *def;
-	FunctionCategory *mathcat = function_get_category_with_translation
-		(mathcatname, _(mathcatname));
-	FunctionCategory *gnumericcat = function_get_category_with_translation
-		(gnumericcatname, _(gnumericcatname));
-	FunctionCategory *sheetcat = function_get_category_with_translation
-		(sheetcatname, _(sheetcatname));
+	static GnmFuncDescriptor const builtins [] = {
+		{	"sum",		NULL,	N_("number,number,"),
+			&help_sum,	NULL,	gnumeric_sum,
+			NULL, NULL, NULL, GNM_FUNC_SIMPLE,
+			GNM_FUNC_IMPL_STATUS_COMPLETE,
+			GNM_FUNC_TEST_STATUS_BASIC
+		},
+		{	"product",		NULL,	N_("number,number,"),
+			&help_product,	NULL,	gnumeric_product,
+			NULL, NULL, NULL, GNM_FUNC_SIMPLE,
+			GNM_FUNC_IMPL_STATUS_COMPLETE,
+			GNM_FUNC_TEST_STATUS_BASIC
+		},
+		{	"gnumeric_version",	"",	"",
+			&help_gnumeric_version,	gnumeric_version,
+			NULL, NULL, NULL, NULL, GNM_FUNC_SIMPLE,
+			GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC,
+			GNM_FUNC_TEST_STATUS_EXHAUSTIVE
+		},
+		{ NULL }
+	};
 
-	def = function_add_nodes (mathcat, "sum",     0,
-				  "number1,number2,...",
-				  &help_sum, gnumeric_sum);
-	auto_format_function_result (def, AF_FIRST_ARG_FORMAT);
-	mathfuncs = g_slist_prepend (mathfuncs, def);
+	GnmFunc *func;
+	GnmFuncGroup *mathcat = gnm_func_group_fetch (mathcatname);
+	GnmFuncGroup *gnumericcat = gnm_func_group_fetch (gnumericcatname);
 
-	def = function_add_nodes (mathcat, "product", 0,
-				  "number",    &help_product,  gnumeric_product);
-	mathfuncs = g_slist_prepend (mathfuncs, def);
+	func = gnm_func_add (mathcat, builtins + 0);
+	auto_format_function_result (func, AF_FIRST_ARG_FORMAT);
+	mathfuncs = g_slist_prepend (mathfuncs, func);
 
-	def = function_add_args (gnumericcat, "gnumeric_version", "",  "",
-				 &help_gnumeric_version, gnumeric_version);
-	gnumericfuncs = g_slist_prepend (gnumericfuncs, def);
+	func = gnm_func_add (mathcat, builtins + 1);
+	mathfuncs = g_slist_prepend (mathfuncs, func);
 
-	def = function_add_args (sheetcat, "selection", "b",  "permit_intersection",
-				 &help_selection, gnumeric_selection);
-	sheetfuncs = g_slist_prepend (sheetfuncs, def);
+	func = gnm_func_add (gnumericcat, builtins + 2);
+	gnumericfuncs = g_slist_prepend (gnumericfuncs, func);
 }
 
 static void
 shutdown_cat (const char *catname, GSList **funcs)
 {
 	GSList *tmp;
-
-	FunctionCategory *cat =
-		function_get_category_with_translation (catname, _(catname));
+	GnmFuncGroup *cat = gnm_func_group_fetch (catname);
 
 	for (tmp = *funcs; tmp; tmp = tmp->next) {
-		FunctionDefinition *def = tmp->data;
-		const char *name = function_def_get_name (def);
-
+		GnmFunc *def = tmp->data;
+		char const *name = gnm_func_get_name (def);
 		auto_format_function_result_remove (name);
 		function_remove (cat, name);
 	}
@@ -267,5 +190,4 @@ func_builtin_shutdown (void)
 {
 	shutdown_cat (mathcatname, &mathfuncs);
 	shutdown_cat (gnumericcatname, &gnumericfuncs);
-	shutdown_cat (sheetcatname, &sheetfuncs);
 }
