@@ -36,7 +36,7 @@
 static GList *categories;
 static SymbolTable *global_symbol_table;
 
-static FunctionCategory *unknown_cat;
+static GnmFuncGroup *unknown_cat;
 static GSList *unknown_functions;
 
 void
@@ -52,22 +52,25 @@ functions_shutdown (void)
 	g_assert ((unknown_cat == NULL) == (unknown_functions == NULL));
 
 	while (unknown_functions) {
-		FunctionDefinition *func = unknown_functions->data;
-		function_remove (unknown_cat, function_def_get_name (func));
+		GnmFunc *func = unknown_functions->data;
+		function_remove (unknown_cat, gnm_func_get_name (func));
 		unknown_functions = g_slist_remove (unknown_functions, func);
 	}
 
 	func_builtin_shutdown ();
+
+	symbol_table_destroy (global_symbol_table);
+	global_symbol_table = NULL;
 }
 
 static void
 copy_hash_table_to_ptr_array (gpointer key, gpointer value, gpointer array)
 {
 	Symbol *sym = value;
-	FunctionDefinition *fd = sym->data;
+	GnmFunc *fd = sym->data;
 	if (sym->type == SYMBOL_FUNCTION && fd->name != NULL) {
-		if (fd->fn_type == FUNCTION_NAMEONLY)
-			func_def_load ((FunctionDefinition *) fd);
+		if (fd->fn_type == GNM_FUNC_TYPE_STUB)
+			gnm_func_load_stub ((GnmFunc *) fd);
 		if (fd->help != NULL)
 			g_ptr_array_add (array, fd);
 	}
@@ -76,18 +79,28 @@ copy_hash_table_to_ptr_array (gpointer key, gpointer value, gpointer array)
 static int
 func_def_cmp (gconstpointer a, gconstpointer b)
 {
-	FunctionDefinition const *fda = *(FunctionDefinition const **)a ;
-	FunctionDefinition const *fdb = *(FunctionDefinition const **)b ;
+	GnmFunc const *fda = *(GnmFunc const **)a ;
+	GnmFunc const *fdb = *(GnmFunc const **)b ;
+
+	g_return_val_if_fail (fda->name != NULL, 0);
+	g_return_val_if_fail (fdb->name != NULL, 0);
+
+	if (fda->fn_group != NULL && fdb->fn_group != NULL) {
+		int res = strcmp (fda->fn_group->display_name->str, fdb->fn_group->display_name->str);
+		if (res != 0)
+			return res;
+	}
 
 	return g_ascii_strcasecmp (fda->name, fdb->name);
 }
 
 void
-function_dump_defs (char const *filename)
+function_dump_defs (char const *filename, gboolean as_def)
 {
 	FILE *output_file;
 	unsigned i;
 	GPtrArray *ordered;
+	GnmFuncGroup const *group = NULL;
 
 	g_return_if_fail (filename != NULL);
 
@@ -96,7 +109,7 @@ function_dump_defs (char const *filename)
 		exit (1);
 	}
 
-	/* TODO : Use the translated names and split by category. */
+	/* TODO : Use the translated names and split by fn_group. */
 	ordered = g_ptr_array_new ();
 	g_hash_table_foreach (global_symbol_table->hash,
 		copy_hash_table_to_ptr_array, ordered);
@@ -106,68 +119,108 @@ function_dump_defs (char const *filename)
 		       ordered->len, sizeof (gpointer),
 		       func_def_cmp);
 
-	for (i = 0; i < ordered->len; i++) {
-		FunctionDefinition const *fd = g_ptr_array_index (ordered, i);
-		fprintf (output_file, "%s\n\n", _( *(fd->help) ) );
-	}
+	if (!as_def)
+		fprintf (output_file, "<table border=1>\n");
 
-	g_ptr_array_free (ordered,TRUE);
+	for (i = 0; i < ordered->len; i++) {
+		GnmFunc const *fd = g_ptr_array_index (ordered, i);
+		if (as_def) {
+			fprintf (output_file, "%s\n\n", _(fd->help));
+		} else {
+			static struct {
+				char const *name;
+				char const *colour_str;
+			} const testing [] = {
+				{ "Unknown",		"FFFFFF" },
+				{ "No Testsuite",	"FF7662" },
+				{ "Basic",		"FFF79D" },
+				{ "Exhaustive",		"AEF8B5" },
+				{ "Under Development",	"FF6C00" }
+			};
+			static struct {
+				char const *name;
+				char const *colour_str;
+			} const implementation [] = {
+				{ "Exists",			"FFFFFF" },
+				{ "Unimplemented",		"FF7662" },
+				{ "Subset",			"FFF79D" },
+				{ "Complete",			"AEF8B5" },
+				{ "Superset",			"16E49E" },
+				{ "Subset with_extensions",	"59FFF2" },
+				{ "Under development",		"FF6C00" },
+				{ "Unique to Gnumeric",		"44BE18" },
+			};
+			if (group != fd->fn_group) {
+				group = fd->fn_group;
+				fprintf (output_file, "<tr><td><h1>%s</h1></td><td><b>Function</b></td><td><b>Implementation</b></td><td><b>Testing</b></td></tr>\n", group->display_name->str);
+			}
+			fprintf (output_file, "<tr><td></td>\n");
+			fprintf (output_file,
+"    <td><a href =\"doc/gnumeric-%s.html\">%s</a></td>\n", fd->name, fd->name);
+			fprintf (output_file,
+"    <td bgcolor=#%s><a href=\"mailto:gnumeric-list@gnome.org?subject=Re: %s implementation\">%s</a></td>\n",
+implementation[fd->impl_status].colour_str, fd->name, implementation[fd->impl_status].name);
+			fprintf (output_file,
+"    <td bgcolor=#%s><a href=\"mailto:gnumeric-list@gnome.org?subject=Re: %s testing\">%s</a></td>\n",
+testing[fd->test_status].colour_str, fd->name, testing[fd->test_status].name);
+			fprintf (output_file,"</tr>\n");
+		}
+	}
+	if (!as_def)
+		fprintf (output_file, "</table>\n");
+
+	g_ptr_array_free (ordered, TRUE);
 	fclose (output_file);
 }
 
 /* ------------------------------------------------------------------------- */
 
+static void
+gnm_func_group_free (GnmFuncGroup *fn_group)
+{
+	g_return_if_fail (fn_group != NULL);
+	g_return_if_fail (fn_group->functions == NULL);
+
+	string_unref (fn_group->internal_name);
+	string_unref (fn_group->display_name);
+	g_free (fn_group);
+}
+
 static gint
 function_category_compare (gconstpointer a, gconstpointer b)
 {
-	FunctionCategory const *cat_a = a;
-	FunctionCategory const *cat_b = b;
-	gchar *str_a, *str_b;
+	GnmFuncGroup const *cat_a = a;
+	GnmFuncGroup const *cat_b = b;
 
-	g_return_val_if_fail (cat_a->display_name != NULL, 0);
-	g_return_val_if_fail (cat_b->display_name != NULL, 0);
-
-#if 0
-	str_a = g_utf8_casefold (cat_a->display_name->str, -1);
-	str_b = g_utf8_casefold (cat_b->display_name->str, -1);
-#endif
-
-	str_a = g_alloca (strlen (cat_a->display_name->str) + 1);
-	str_b = g_alloca (strlen (cat_b->display_name->str) + 1);
-	g_strdown (strcpy (str_a, cat_a->display_name->str));
-	g_strdown (strcpy (str_b, cat_b->display_name->str));
-
-	return strcoll (str_a, str_b);
+	return g_utf8_collate (cat_a->display_name->str,
+			       cat_b->display_name->str);
 }
 
-FunctionCategory *
-function_get_category (gchar const *name)
+GnmFuncGroup *
+gnm_func_group_fetch (char const *name)
 {
-	return function_get_category_with_translation (name, _(name));
+	return gnm_func_group_fetch_with_translation (name, _(name));
 }
 
-FunctionCategory *
-function_get_category_with_translation (gchar const *name,
-                                        gchar const *translation)
+GnmFuncGroup *
+gnm_func_group_fetch_with_translation (char const *name,
+				       char const *translation)
 {
-	FunctionCategory *cat = NULL;
-	gchar *int_name;
+	GnmFuncGroup *cat = NULL;
 	GList *l;
 
 	g_return_val_if_fail (name != NULL, NULL);
 
-	int_name = g_alloca (strlen (name) + 1);
-	g_strdown (strcpy (int_name, name));
 	for (l = categories; l != NULL; l = l->next) {
 		cat = l->data;
-		if (strcmp (cat->internal_name->str, int_name) == 0) {
+		if (strcmp (cat->internal_name->str, name) == 0) {
 			break;
 		}
 	}
 
 	if (l == NULL) {
-		cat = g_new (FunctionCategory, 1);
-		cat->internal_name = string_get (int_name);
+		cat = g_new (GnmFuncGroup, 1);
+		cat->internal_name = string_get (name);
 		if (translation != NULL) {
 			cat->display_name = string_get (translation);
 			cat->has_translation = TRUE;
@@ -192,51 +245,39 @@ function_get_category_with_translation (gchar const *name,
 	return cat;
 }
 
-FunctionCategory *
-function_category_get_nth (int n)
+GnmFuncGroup *
+gnm_func_group_get_nth (int n)
 {
 	return g_list_nth_data (categories, n);
 }
 
-void
-function_category_add_func (FunctionCategory *category,
-			    FunctionDefinition *fn_def)
-{
-	g_return_if_fail (category != NULL);
-	g_return_if_fail (fn_def != NULL);
-
-	category->functions = g_list_append (category->functions, fn_def);
-}
-
 static void
-function_category_free (FunctionCategory *category)
+gnm_func_group_add_func (GnmFuncGroup *fn_group,
+			 GnmFunc *fn_def)
 {
-	g_return_if_fail (category != NULL);
-	g_return_if_fail (category->functions == NULL);
+	g_return_if_fail (fn_group != NULL);
+	g_return_if_fail (fn_def != NULL);
 
-	string_unref (category->internal_name);
-	string_unref (category->display_name);
-	g_free (category);
+	fn_group->functions = g_list_append (fn_group->functions, fn_def);
 }
 
 void
-function_category_remove_func (FunctionCategory *category,
-                               FunctionDefinition *fn_def)
+gnm_func_group_remove_func (GnmFuncGroup *fn_group, GnmFunc *func)
 {
-	g_return_if_fail (category != NULL);
-	g_return_if_fail (fn_def != NULL);
+	g_return_if_fail (fn_group != NULL);
+	g_return_if_fail (func != NULL);
 
-	category->functions = g_list_remove (category->functions, fn_def);
-	if (category->functions == NULL) {
-		categories = g_list_remove (categories, category);
-		function_category_free (category);
+	fn_group->functions = g_list_remove (fn_group->functions, func);
+	if (fn_group->functions == NULL) {
+		categories = g_list_remove (categories, fn_group);
+		gnm_func_group_free (fn_group);
 	}
 }
 
 /******************************************************************************/
 
 static void
-extract_arg_types (FunctionDefinition *def)
+extract_arg_types (GnmFunc *def)
 {
 	int i;
 
@@ -256,215 +297,164 @@ error_function_no_full_info (FunctionEvalInfo *ei, GnmExprList *expr_node_list)
 }
 
 void
-func_def_load (FunctionDefinition *fn_def)
+gnm_func_load_stub (GnmFunc *func)
 {
-	gchar const *args;
-	gchar const *arg_names;
-	gchar const **help;
-	FunctionArgs	 fn_args;
-	FunctionNodes	 fn_nodes;
-	FuncLinkHandle	 fn_link;
-	FuncUnlinkHandle fn_unlink;
-	gboolean success;
+	GnmFuncDescriptor desc;
 
-	g_return_if_fail (fn_def->fn_type == FUNCTION_NAMEONLY);
+	g_return_if_fail (func->fn_type == GNM_FUNC_TYPE_STUB);
 
-	success = fn_def->get_full_info_callback (
-		  fn_def, &args, &arg_names, &help,
-		  &fn_args, &fn_nodes, &fn_link, &fn_unlink);
+	/* default the content to 0 in case we add new fields
+	 * later and the services do not fill them in
+	 */
+	memset (&desc, 0, sizeof (GnmFuncDescriptor));
 
-	if (success) {
-		fn_def->named_arguments = arg_names;
-		fn_def->help = help;
-		if (fn_args != NULL) {
-			fn_def->fn_type = FUNCTION_ARGS;
-			fn_def->fn.args.func = fn_args;
-			fn_def->fn.args.arg_spec = args;
-			extract_arg_types (fn_def);
-		} else if (fn_nodes != NULL) {
-			fn_def->fn_type = FUNCTION_NODES;
-			fn_def->fn.fn_nodes = fn_nodes;
+	if (func->fn.load_desc (func, &desc)) {
+		func->arg_names	 = desc.arg_names;
+		func->help	 = desc.help ? *desc.help : NULL;
+		if (desc.fn_args != NULL) {
+			func->fn_type		= GNM_FUNC_TYPE_ARGS;
+			func->fn.args.func	= desc.fn_args;
+			func->fn.args.arg_spec	= desc.arg_spec;
+			extract_arg_types (func);
+		} else if (desc.fn_nodes != NULL) {
+			func->fn_type		= GNM_FUNC_TYPE_NODES;
+			func->fn.nodes		= desc.fn_nodes;
 		} else {
-			g_assert_not_reached ();
+			g_warning ("Invalid function descriptor with no function");
 		}
-		fn_def->link = fn_link;
-		fn_def->unlink = fn_unlink;
+		func->linker	  = desc.linker;
+		func->unlinker	  = desc.unlinker;
+		func->impl_status = desc.impl_status;
+		func->test_status = desc.test_status;
+		func->flags	  = desc.flags;
 	} else {
-		fn_def->named_arguments = "";
-		fn_def->fn_type = FUNCTION_NODES;
-		fn_def->fn.fn_nodes = &error_function_no_full_info;
-		fn_def->link = NULL;
-		fn_def->unlink = NULL;
+		func->arg_names = "";
+		func->fn_type = GNM_FUNC_TYPE_NODES;
+		func->fn.nodes = &error_function_no_full_info;
+		func->linker   = NULL;
+		func->unlinker = NULL;
 	}
 }
 
 void
-func_ref (FunctionDefinition *fn_def)
+gnm_func_ref (GnmFunc *func)
 {
-	g_return_if_fail (fn_def != NULL);
+	g_return_if_fail (func != NULL);
 
-	fn_def->ref_count++;
-	if (fn_def->ref_count == 1 && fn_def->ref_notify != NULL) {
-		fn_def->ref_notify (fn_def, 1);
-	}
+	func->ref_count++;
+	if (func->ref_count == 1 && func->ref_notify != NULL)
+		func->ref_notify (func, 1);
 }
 
 void
-func_unref (FunctionDefinition *fn_def)
+gnm_func_unref (GnmFunc *func)
 {
-	g_return_if_fail (fn_def != NULL);
-	g_return_if_fail (fn_def->ref_count > 0);
+	g_return_if_fail (func != NULL);
+	g_return_if_fail (func->ref_count > 0);
 
-	fn_def->ref_count--;
-	if (fn_def->ref_count == 0 && fn_def->ref_notify != NULL) {
-		fn_def->ref_notify (fn_def, 0);
-	}
+	func->ref_count--;
+	if (func->ref_count == 0 && func->ref_notify != NULL)
+		func->ref_notify (func, 0);
 }
 
-gint
-func_get_ref_count (FunctionDefinition *fn_def)
+GnmFunc *
+gnm_func_lookup (char const *name, Workbook const *optional_scope)
 {
-	g_return_val_if_fail (fn_def != NULL, 0);
-
-	return fn_def->ref_count;
-}
-
-FunctionDefinition *
-func_lookup_by_name (gchar const *fn_name, Workbook const *optional_scope)
-{
-	Symbol *sym;
-
-	sym = symbol_lookup (global_symbol_table, fn_name);
-	if (sym != NULL) {
+	Symbol *sym = symbol_lookup (global_symbol_table, name);
+	if (sym != NULL)
 		return sym->data;
-	}
-
 	return NULL;
 }
 
 void
-function_remove (FunctionCategory *category, gchar const *name)
+function_remove (GnmFuncGroup *fn_group, char const *name)
 {
-	FunctionDefinition *fn_def;
+	GnmFunc *func;
 	Symbol *sym;
 
 	g_return_if_fail (name != NULL);
 
-	fn_def = func_lookup_by_name (name, NULL);
-	g_return_if_fail (fn_def->ref_count == 0);
-	function_category_remove_func (category, fn_def);
+	func = gnm_func_lookup (name, NULL);
+
+	g_return_if_fail (func->ref_count == 0);
+
+	gnm_func_group_remove_func (fn_group, func);
 	sym = symbol_lookup (global_symbol_table, name);
 	symbol_unref (sym);
 
-	switch (fn_def->fn_type) {
-	case FUNCTION_ARGS:
-		g_free (fn_def->fn.args.arg_types);
+	switch (func->fn_type) {
+	case GNM_FUNC_TYPE_ARGS:
+		g_free (func->fn.args.arg_types);
 		break;
 	default:
 		/* Nothing.  */
 		;
 	}
-	g_free (fn_def);
+	g_free (func);
 }
 
-static FunctionDefinition *
-fn_def_new (FunctionCategory *category,
-	    char const *name,
-	    char const *arg_names,
-	    char const **help,
-	    FuncRefNotify opt_ref_notify)
+#if 0
+function_add_args
+function_add_nodes
+#endif
+GnmFunc *
+gnm_func_add (GnmFuncGroup *fn_group,
+	      GnmFuncDescriptor const *desc)
 {
-	FunctionDefinition *fn_def;
-
-	fn_def = g_new (FunctionDefinition, 1);
-	fn_def->get_full_info_callback = NULL;
-	fn_def->flags	= 0;
-	fn_def->name    = name;
-	fn_def->help    = help;
-	fn_def->named_arguments = arg_names;
-	fn_def->link	= NULL;
-	fn_def->unlink	= NULL;
-	fn_def->user_data = NULL;
-	fn_def->ref_count = 0;
-	fn_def->ref_notify = opt_ref_notify;
-
-	if (category != NULL)
-		function_category_add_func (category, fn_def);
-	symbol_install (global_symbol_table, name, SYMBOL_FUNCTION, fn_def);
-
-	return fn_def;
-}
-
-FunctionDefinition *
-function_add_args (FunctionCategory *category,
-		   char const *name,
-		   char const *args,
-		   char const *arg_names,
-		   char const **help,
-		   FunctionArgs fn,
-		   FuncRefNotify opt_ref_notify)
-{
-	static char const valid_tokens[] = "fsbraAS?|";
-	FunctionDefinition *fn_def;
+	static char const valid_tokens[] = "fsbraAES?|";
+	GnmFunc *func;
 	char const *ptr;
 
-	g_return_val_if_fail (fn != NULL, NULL);
-	g_return_val_if_fail (args != NULL, NULL);
+	g_return_val_if_fail (fn_group != NULL, NULL);
+	g_return_val_if_fail (desc != NULL, NULL);
 
-	/* Check those arguements */
-	for (ptr = args ; *ptr ; ptr++) {
-		g_return_val_if_fail (strchr (valid_tokens, *ptr), NULL);
+	func = g_new (GnmFunc, 1);
+	if (func == NULL)
+		return NULL;
+
+	func->name		= desc->name;
+	func->arg_names		= desc->arg_names;
+	func->help		= desc->help ? *desc->help : NULL;
+	func->linker		= desc->linker;
+	func->unlinker		= desc->unlinker;
+	func->ref_notify	= desc->ref_notify;
+	func->flags		= desc->flags;
+	func->impl_status	= desc->impl_status;
+	func->test_status	= desc->test_status;
+
+	func->user_data		= NULL;
+	func->ref_count		= 0;
+
+	if (desc->fn_args != NULL) {
+		/* Check those arguements */
+		for (ptr = desc->arg_spec ; *ptr ; ptr++) {
+			g_return_val_if_fail (strchr (valid_tokens, *ptr), NULL);
+		}
+
+		func->fn_type		= GNM_FUNC_TYPE_ARGS;
+		func->fn.args.func	= desc->fn_args;
+		func->fn.args.arg_spec	= desc->arg_spec;
+		extract_arg_types (func);
+	} else if (desc->fn_nodes != NULL) {
+
+		if (desc->arg_spec && *desc->arg_spec) {
+			g_warning ("Arg spec for node function -- why?");
+		}
+
+		func->fn_type  = GNM_FUNC_TYPE_NODES;
+		func->fn.nodes = desc->fn_nodes;
+	} else {
+		g_warning ("Invalid function has neither args nor nodes handler");
+		g_free (func);
+		return NULL;
 	}
 
-	fn_def = fn_def_new (category, name, arg_names, help, opt_ref_notify);
-	if (fn_def != NULL) {
-		fn_def->fn_type = FUNCTION_ARGS;
-		fn_def->fn.args.func = fn;
-		fn_def->fn.args.arg_spec = args;
-		extract_arg_types (fn_def);
-	}
-	return fn_def;
-}
+	func->fn_group = fn_group;
+	if (fn_group != NULL)
+		gnm_func_group_add_func (fn_group, func);
+	symbol_install (global_symbol_table, func->name, SYMBOL_FUNCTION, func);
 
-FunctionDefinition *
-function_add_nodes (FunctionCategory *category,
-		    char const *name,
-		    char const *args,
-		    char const *arg_names,
-		    char const **help,
-		    FunctionNodes fn,
-		    FuncRefNotify opt_ref_notify)
-{
-	FunctionDefinition *fn_def;
-
-	g_return_val_if_fail (fn != NULL, NULL);
-	if (args && *args) {
-		g_warning ("Arg spec for node function -- why?");
-	}
-
-	fn_def = fn_def_new (category, name, arg_names, help, opt_ref_notify);
-	if (fn_def != NULL) {
-		fn_def->fn_type     = FUNCTION_NODES;
-		fn_def->fn.fn_nodes = fn;
-	}
-	return fn_def;
-}
-
-FunctionDefinition *
-function_add_name_only (FunctionCategory *category,
-                        gchar const *name,
-                        FunctionGetFullInfoCallback callback,
-                        FuncRefNotify opt_ref_notify)
-{
-	FunctionDefinition *fn_def;
-
-	fn_def = fn_def_new (category, name, NULL, NULL, opt_ref_notify);
-	if (fn_def != NULL) {
-		fn_def->fn_type = FUNCTION_NAMEONLY;
-		fn_def->get_full_info_callback = callback;
-	}
-
-	return fn_def;
+	return func;
 }
 
 /* Handle unknown functions on import without losing their names */
@@ -474,6 +464,29 @@ unknownFunctionHandler (FunctionEvalInfo *ei, GnmExprList *expr_node_list)
 	return value_new_error (ei->pos, gnumeric_err_NAME);
 }
 
+GnmFunc *
+gnm_func_add_stub (GnmFuncGroup *fn_group,
+		   char const 	    *name,
+		   GnmFuncLoadDesc   load_desc,
+		   GnmFuncRefNotify  opt_ref_notify)
+{
+	GnmFunc *func = g_new0 (GnmFunc, 1);
+	if (func == NULL)
+		return NULL;
+
+	func->name		= name;
+	func->ref_notify	= opt_ref_notify;
+	func->fn_type		= GNM_FUNC_TYPE_STUB;
+	func->fn.load_desc	= load_desc;
+
+	func->fn_group = fn_group;
+	if (fn_group != NULL)
+		gnm_func_group_add_func (fn_group, func);
+	symbol_install (global_symbol_table, func->name, SYMBOL_FUNCTION, func);
+
+	return func;
+}
+
 /*
  * When importing it is useful to keep track of unknown function names.
  * We may be missing a plugin or something similar.
@@ -482,26 +495,34 @@ unknownFunctionHandler (FunctionEvalInfo *ei, GnmExprList *expr_node_list)
  *        and replace them with something else.  Possibly even reordering the
  *        arguments.
  */
-FunctionDefinition *
-function_add_placeholder (char const *name, char const *type)
+GnmFunc *
+gnm_func_add_placeholder (char const *name, char const *type,
+			  gboolean copy_name)
 {
-	FunctionDefinition *func = func_lookup_by_name (name, NULL);
-	const char *unknown_cat_name = N_("Unknown Function");
+	GnmFuncDescriptor desc;
+	GnmFunc *func = gnm_func_lookup (name, NULL);
+	char const *unknown_cat_name = N_("Unknown Function");
 
 	g_return_val_if_fail (func == NULL, func);
 
 	if (!unknown_cat)
-		unknown_cat = function_get_category (unknown_cat_name);
+		unknown_cat = gnm_func_group_fetch (unknown_cat_name);
 
-	/*
-	 * TODO TODO TODO : should add a
-	 *    function_add_{nodes,args}_fake
-	 * This will allow a user to load a missing
-	 * plugin to supply missing functions.
-	 */
-	func = function_add_nodes (unknown_cat, g_strdup (name),
-				   0, "...", NULL,
-				   &unknownFunctionHandler, NULL);
+	memset (&desc, 0, sizeof (GnmFuncDescriptor));
+	/* FIXME: if copied, the name is leaked.  */
+	desc.name	  = copy_name ? g_strdup (name) : name;
+	desc.arg_spec	  = NULL;
+	desc.arg_names	  = "...";
+	desc.help	  = NULL;
+	desc.fn_args	  = NULL;
+	desc.fn_nodes	  = &unknownFunctionHandler;
+	desc.linker	  = NULL;
+	desc.unlinker	  = NULL;
+	desc.flags	  = GNM_FUNC_IS_PLACEHOLDER;
+	desc.impl_status  = GNM_FUNC_IMPL_STATUS_EXISTS;
+	desc.test_status  = GNM_FUNC_TEST_STATUS_UNKNOWN;
+
+	func = gnm_func_add (unknown_cat, &desc);
 	unknown_functions = g_slist_prepend (unknown_functions, func);
 
 	/* WISHLIST : it would be nice to have a log if these. */
@@ -511,33 +532,32 @@ function_add_placeholder (char const *name, char const *type)
 }
 
 gpointer
-function_def_get_user_data (FunctionDefinition const *fn_def)
+gnm_func_get_user_data (GnmFunc const *func)
 {
-	g_return_val_if_fail (fn_def != NULL, NULL);
+	g_return_val_if_fail (func != NULL, NULL);
 
-	return fn_def->user_data;
+	return func->user_data;
 }
 
 void
-function_def_set_user_data (FunctionDefinition *fn_def,
-			    gpointer user_data)
+gnm_func_set_user_data (GnmFunc *func, gpointer user_data)
 {
-	g_return_if_fail (fn_def != NULL);
+	g_return_if_fail (func != NULL);
 
-	fn_def->user_data = user_data;
+	func->user_data = user_data;
 }
 
 char const *
-function_def_get_name (FunctionDefinition const *fn_def)
+gnm_func_get_name (GnmFunc const *func)
 {
-	g_return_val_if_fail (fn_def != NULL, NULL);
+	g_return_val_if_fail (func != NULL, NULL);
 
-	return fn_def->name;
+	return func->name;
 }
 
 /**
  * function_def_count_args:
- * @fn_def: pointer to function definition
+ * @func: pointer to function definition
  * @min: pointer to min. args
  * @max: pointer to max. args
  *
@@ -549,7 +569,7 @@ function_def_get_name (FunctionDefinition const *fn_def)
  *
  **/
 void
-function_def_count_args (FunctionDefinition const *fn_def,
+function_def_count_args (GnmFunc const *fn_def,
                          int *min, int *max)
 {
 	char const *ptr;
@@ -560,14 +580,14 @@ function_def_count_args (FunctionDefinition const *fn_def,
 	g_return_if_fail (max != NULL);
 	g_return_if_fail (fn_def != NULL);
 
-	if (fn_def->fn_type == FUNCTION_NAMEONLY)
-		func_def_load ((FunctionDefinition *) fn_def);
+	if (fn_def->fn_type == GNM_FUNC_TYPE_STUB)
+		gnm_func_load_stub ((GnmFunc *) fn_def);
 
 	/*
 	 * FIXME: clearly for 'nodes' functions many of
 	 * the type fields will need to be filled.
 	 */
-	if (fn_def->fn_type == FUNCTION_NODES) {
+	if (fn_def->fn_type == GNM_FUNC_TYPE_NODES) {
 		*min = 0;
 		*max = G_MAXINT;
 		return;
@@ -586,26 +606,6 @@ function_def_count_args (FunctionDefinition const *fn_def,
 		*min = i;
 }
 
-/**
- * function_set_link_handlers :
- *
- * Add callbacks for a function to perform special handling as each instance
- * of the function is linked or unlinked from the sheet.
- */
-void
-function_set_link_handlers (FunctionDefinition *fn_def,
-			    FuncLinkHandle   link,
-			    FuncUnlinkHandle unlink)
-{
-	/* Be paranoid for now */
-	g_return_if_fail (fn_def != NULL);
-	g_return_if_fail (fn_def->link == NULL);
-	g_return_if_fail (fn_def->unlink == NULL);
-
-	fn_def->link = link;
-	fn_def->unlink = unlink;
-}
-
 /**
  * function_def_get_arg_type:
  * @fn_def: the fn defintion
@@ -614,7 +614,7 @@ function_set_link_handlers (FunctionDefinition *fn_def,
  * Return value: the type of the argument
  **/
 char
-function_def_get_arg_type (FunctionDefinition const *fn_def,
+function_def_get_arg_type (GnmFunc const *fn_def,
                            int arg_idx)
 {
 	char const *ptr;
@@ -622,8 +622,8 @@ function_def_get_arg_type (FunctionDefinition const *fn_def,
 	g_return_val_if_fail (arg_idx >= 0, '?');
 	g_return_val_if_fail (fn_def != NULL, '?');
 
-	if (fn_def->fn_type == FUNCTION_NAMEONLY)
-		func_def_load ((FunctionDefinition *) fn_def);
+	if (fn_def->fn_type == GNM_FUNC_TYPE_STUB)
+		gnm_func_load_stub ((GnmFunc *) fn_def);
 
 	for (ptr = fn_def->fn.args.arg_spec; ptr && *ptr; ptr++) {
 		if (*ptr == '|')
@@ -642,7 +642,7 @@ function_def_get_arg_type (FunctionDefinition const *fn_def,
  * Return value: the type of the argument as a string
  **/
 char const *
-function_def_get_arg_type_string (FunctionDefinition const *fn_def,
+function_def_get_arg_type_string (GnmFunc const *fn_def,
 				  int arg_idx)
 {
 	switch (function_def_get_arg_type (fn_def, arg_idx)) {
@@ -656,6 +656,8 @@ function_def_get_arg_type_string (FunctionDefinition const *fn_def,
 		return _("Cell Range");
 	case 'A':
 		return _("Area");
+	case 'E':
+		return _("Scalar or Error");
 	case 'S':
 		return _("Scalar");
 	case '?':
@@ -675,26 +677,26 @@ function_def_get_arg_type_string (FunctionDefinition const *fn_def,
  * Return value: the name of the argument (must be freed)
  **/
 char*
-function_def_get_arg_name (FunctionDefinition const *fn_def,
+function_def_get_arg_name (GnmFunc const *fn_def,
                            int arg_idx)
 {
-	gchar **names, **o_names;
-	gchar *name;
-	gchar *translated_arguments;
-	gchar delimiter[2];
+	char **names, **o_names;
+	char *name;
+	char *translated_arguments;
+	char delimiter[2];
 
 	g_return_val_if_fail (arg_idx >= 0, NULL);
 	g_return_val_if_fail (fn_def != NULL, NULL);
 
-	if (fn_def->fn_type == FUNCTION_NAMEONLY)
-		func_def_load ((FunctionDefinition *) fn_def);
+	if (fn_def->fn_type == GNM_FUNC_TYPE_STUB)
+		gnm_func_load_stub ((GnmFunc *) fn_def);
 
-	if (!fn_def->named_arguments)
+	if (!fn_def->arg_names)
 		return NULL;
 
-	translated_arguments = _(fn_def->named_arguments);
+	translated_arguments = _(fn_def->arg_names);
 	delimiter[0] = 
-		strcmp (translated_arguments, fn_def->named_arguments) == 0
+		strcmp (translated_arguments, fn_def->arg_names) == 0
 		? ','
 		: format_get_arg_sep ();
 	delimiter[1] = 0;
@@ -738,7 +740,7 @@ Value *
 function_call_with_list (FunctionEvalInfo *ei, GnmExprList *l,
 			 GnmExprEvalFlags flags)
 {
-	FunctionDefinition const *fn_def;
+	GnmFunc const *fn_def;
 	int	  argc, i, optional, iter_count, iter_width = 0, iter_height = 0;
 	char	  arg_type;
 	Value	 **args, *tmp = NULL;
@@ -749,12 +751,12 @@ function_call_with_list (FunctionEvalInfo *ei, GnmExprList *l,
 	g_return_val_if_fail (ei->func_call != NULL, NULL);
 
 	fn_def = ei->func_call->func;
-	if (fn_def->fn_type == FUNCTION_NAMEONLY)
-		func_def_load ((FunctionDefinition *) fn_def);
+	if (fn_def->fn_type == GNM_FUNC_TYPE_STUB)
+		gnm_func_load_stub ((GnmFunc *) fn_def);
 
 	/* Functions that deal with ExprNodes */
-	if (fn_def->fn_type == FUNCTION_NODES)
-		return fn_def->fn.fn_nodes (ei, l);
+	if (fn_def->fn_type == GNM_FUNC_TYPE_NODES)
+		return fn_def->fn.nodes (ei, l);
 
 	/* Functions that take pre-computed Values */
 	argc = gnm_expr_list_length (l);
@@ -772,7 +774,7 @@ function_call_with_list (FunctionEvalInfo *ei, GnmExprList *l,
 		arg_type = fn_def->fn.args.arg_types[i];
 		expr = l->data;
 
-		if (i > fn_def->fn.args.min_args) 
+		if (i >= fn_def->fn.args.min_args) 
 			optional = GNM_EXPR_EVAL_PERMIT_EMPTY;
 
 		if (arg_type == 'A' || arg_type == 'r') {
@@ -802,8 +804,9 @@ function_call_with_list (FunctionEvalInfo *ei, GnmExprList *l,
 
 		/* force scalars whenever we are certain */
 		tmp = args[i] = gnm_expr_eval (expr, ei->pos, optional |
-		       (iter_count >= 0 || arg_type == '?')
-			       ? GNM_EXPR_EVAL_PERMIT_NON_SCALAR : 0);
+		       ((iter_count >= 0 || arg_type == '?')
+			       ? GNM_EXPR_EVAL_PERMIT_NON_SCALAR
+			       : GNM_EXPR_EVAL_SCALAR_NON_EMPTY));
 
 		if (tmp == NULL ||	/* Optional arguments can be empty */
 		    arg_type == '?')	/* '?' arguments are unrestriced */
@@ -811,28 +814,24 @@ function_call_with_list (FunctionEvalInfo *ei, GnmExprList *l,
 
 		/* Handle implicit intersection or iteration depending on flags */
 		if (tmp->type == VALUE_CELLRANGE || tmp->type == VALUE_ARRAY) {
-			if (iter_count < 0) {
-				tmp = value_intersection (tmp, ei->pos);
-				if (tmp == NULL) {
-					free_values (args, i);
+			if (iter_count > 0) {
+				if (iter_width != value_area_get_width (tmp, ei->pos) ||
+				    iter_height != value_area_get_height (tmp, ei->pos)) {
+					/* no need to free inter_vals, there is nothing there yet */
+					free_values (args, i + 1);
 					return value_new_error (ei->pos, gnumeric_err_VALUE);
 				}
 			} else {
-				if (iter_count != 0) {
-					if (iter_width != value_area_get_width (ei->pos, tmp) ||
-					    iter_height != value_area_get_height (ei->pos, tmp)) {
-						/* no need to free inter_vals, there is nothing there yet */
-						free_values (args, i + 1);
-						return value_new_error (ei->pos, gnumeric_err_VALUE);
-					}
-				} else {
-					iter_item = g_alloca (sizeof (int) * argc);
-					iter_width = value_area_get_width (ei->pos, tmp);
-					iter_height = value_area_get_height (ei->pos, tmp);
+				if (iter_count < 0) {
+					g_warning ("Damn I though this was impossible");
+					iter_count = 0;
 				}
-				iter_item[iter_count] = iter_count;
-				iter_count++;
+				iter_item = g_alloca (sizeof (int) * argc);
+				iter_width = value_area_get_width (tmp, ei->pos);
+				iter_height = value_area_get_height (tmp, ei->pos);
 			}
+			iter_item[iter_count] = iter_count;
+			iter_count++;
 		}
 
 		/* All of these argument types must be scalars */
@@ -870,9 +869,17 @@ function_call_with_list (FunctionEvalInfo *ei, GnmExprList *l,
 			}
 			break;
 
-		case 'S': /* nothing necessary */
-		/* case '?': handled above */
+		case 'S':
+			if (tmp->type == VALUE_ERROR) {
+				free_values (args, i);
+				return tmp; 
+			}
 			break;
+
+		case 'E': /* nothing necessary */
+			break;
+
+		/* case '?': handled above */
 		default :
 			g_warning ("Unknown argument type '%c'", arg_type);
 			break;
@@ -898,7 +905,7 @@ function_call_with_list (FunctionEvalInfo *ei, GnmExprList *l,
 				/* marshal the args */
 				err = NULL;
 				for (i = 0 ; i < iter_count; i++) {
-					elem = value_area_fetch_x_y (ei->pos, iter_vals[i], x, y);
+					elem = value_area_fetch_x_y (iter_vals[i], x, y, ei->pos);
 					arg_type = fn_def->fn.args.arg_types[iter_item[i]];
 					if  (arg_type == 'b' || arg_type == 'f') {
 						if (elem->type == VALUE_STRING) {
@@ -952,14 +959,14 @@ Value *
 function_call_with_values (EvalPos const *ep, char const *fn_name,
 			   int argc, Value *values [])
 {
-	FunctionDefinition *fn_def;
+	GnmFunc *fn_def;
 
 	g_return_val_if_fail (ep != NULL, NULL);
 	g_return_val_if_fail (fn_name != NULL, NULL);
 	g_return_val_if_fail (ep->sheet != NULL, NULL);
 
 	/* FIXME : support workbook local functions */
-	fn_def = func_lookup_by_name (fn_name, NULL);
+	fn_def = gnm_func_lookup (fn_name, NULL);
 	if (fn_def == NULL)
 		return value_new_error (ep, _("Function does not exist"));
 	return function_def_call_with_values (ep, fn_def, argc, values);
@@ -967,7 +974,7 @@ function_call_with_values (EvalPos const *ep, char const *fn_name,
 
 Value *
 function_def_call_with_values (EvalPos const *ep,
-                               FunctionDefinition const *fn_def,
+                               GnmFunc const *fn_def,
                                gint    argc,
                                Value  *values [])
 {
@@ -977,12 +984,12 @@ function_def_call_with_values (EvalPos const *ep,
 
 	fs.pos = ep;
 	fs.func_call = &ef;
-	ef.func = (FunctionDefinition *)fn_def;
+	ef.func = (GnmFunc *)fn_def;
 
-	if (fn_def->fn_type == FUNCTION_NAMEONLY)
-		func_def_load (ef.func);
+	if (fn_def->fn_type == GNM_FUNC_TYPE_STUB)
+		gnm_func_load_stub (ef.func);
 
-	if (fn_def->fn_type == FUNCTION_NODES) {
+	if (fn_def->fn_type == GNM_FUNC_TYPE_NODES) {
 		/*
 		 * If function deals with ExprNodes, create some
 		 * temporary ExprNodes with constants.
@@ -1002,7 +1009,7 @@ function_def_call_with_values (EvalPos const *ep,
 			}
 		}
 
-		retval = fn_def->fn.fn_nodes (&fs, l);
+		retval = fn_def->fn.nodes (&fs, l);
 
 		if (l != NULL)
 			gnm_expr_list_free (l);
@@ -1177,27 +1184,27 @@ function_iterate_argument_values (EvalPos const		*ep,
 /* ------------------------------------------------------------------------- */
 
 TokenizedHelp *
-tokenized_help_new (FunctionDefinition const *fn_def)
+tokenized_help_new (GnmFunc const *func)
 {
 	TokenizedHelp *tok;
 
-	g_return_val_if_fail (fn_def != NULL, NULL);
+	g_return_val_if_fail (func != NULL, NULL);
 
-	if (fn_def->fn_type == FUNCTION_NAMEONLY)
-		func_def_load ((FunctionDefinition *) fn_def);
+	if (func->fn_type == GNM_FUNC_TYPE_STUB)
+		gnm_func_load_stub ((GnmFunc *) func);
 
 	tok = g_new (TokenizedHelp, 1);
-	tok->fndef = fn_def;
+	tok->fndef = func;
 	tok->help_copy = NULL;
 	tok->sections = NULL;
 
-	if (fn_def->help != NULL && fn_def->help [0] != '\0') {
+	if (func->help != NULL && func->help [0] != '\0') {
 		char *ptr, *start;
 		gboolean seek_at = TRUE;
 		gboolean last_newline = TRUE;
 
-		ptr = _(fn_def->help [0]);
-		tok->help_is_localized = ptr != fn_def->help [0];
+		ptr = _(func->help);
+		tok->help_is_localized = ptr != func->help;
 		tok->help_copy = g_strdup (ptr);
 		tok->sections = g_ptr_array_new ();
 
