@@ -19,6 +19,7 @@
 #include "workbook-control-gui-priv.h"
 #include "sheet-control-gui-priv.h"
 #include "sheet.h"
+#include "sheet-view.h"
 #include "sheet-style.h"
 #include "sheet-merge.h"
 #include "sheet-object-impl.h"
@@ -301,7 +302,7 @@ item_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 		int draw_x, int draw_y, int width, int height)
 {
 	GnomeCanvas *canvas = item->canvas;
-	GnumericCanvas *gcanvas = GNUMERIC_CANVAS (canvas);
+	GnmCanvas *gcanvas = GNM_CANVAS (canvas);
 	Sheet const *sheet = ((SheetControl *) gcanvas->simple.scg)->sheet;
 	Cell const * const edit_cell = gcanvas->simple.scg->wbcg->wb_control.editing_cell;
 	ItemGrid *ig = ITEM_GRID (item);
@@ -460,6 +461,12 @@ item_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 			}
 		}
 
+		/* its safe to const_cast because only the a non-default row
+		 * will ever get flagged.
+		 */
+		if (ri->needs_respan)
+			row_calc_spans ((ColRowInfo *)ri, sheet);
+
 		for (col = start_col, x = diff_x; col <= end_col ; col++) {
 			MStyle const *style;
 			CellSpanInfo const *span;
@@ -778,13 +785,13 @@ ig_obj_create_finish (ItemGrid *ig, GdkEventButton *event)
  * ig_obj_create_begin :
  *
  * Starts the process of creating a SheetObject.  Handles the initial
- * button press on the GnumericCanvas.
+ * button press on the GnmCanvas.
  */
 static gboolean
 ig_obj_create_begin (ItemGrid *ig, GdkEventButton *event)
 {
 	GnomeCanvasItem *item = GNOME_CANVAS_ITEM (ig);
-	GnumericCanvas  *gcanvas = GNUMERIC_CANVAS (item->canvas);
+	GnmCanvas  *gcanvas = GNM_CANVAS (item->canvas);
 	SheetControlGUI *scg;
 	SheetObject *so;
 
@@ -854,12 +861,13 @@ item_grid_button_press (ItemGrid *ig, GdkEventButton *event)
 {
 	GnomeCanvasItem *item = GNOME_CANVAS_ITEM (ig);
 	GnomeCanvas    *canvas = item->canvas;
-	GnumericCanvas *gcanvas = GNUMERIC_CANVAS (canvas);
+	GnmCanvas *gcanvas = GNM_CANVAS (canvas);
 	SheetControlGUI *scg = ig->scg;
 	SheetControl *sc = (SheetControl *) scg;
 	Sheet *sheet = sc->sheet;
 	CellPos	pos;
 	int x, y;
+	gboolean already_selected;
 
 	gnm_canvas_slide_stop (gcanvas);
 
@@ -928,7 +936,8 @@ item_grid_button_press (ItemGrid *ig, GdkEventButton *event)
 	/* button 1 will always change the selection,  the other buttons will
 	 * only effect things if the target is not already selected.
 	 */
-	if (event->button == 1 || !sv_is_pos_selected (sc->view, pos.col, pos.row)) {
+	already_selected = sv_is_pos_selected (sc->view, pos.col, pos.row);
+	if (event->button == 1 || !already_selected) {
 		if (!(event->state & (GDK_CONTROL_MASK|GDK_SHIFT_MASK)))
 			sv_selection_reset (sc->view);
 
@@ -944,13 +953,20 @@ item_grid_button_press (ItemGrid *ig, GdkEventButton *event)
       	switch (event->button) {
 	case 1: {
 		guint32 double_click_time;
-		g_object_get (gtk_widget_get_settings (GTK_WIDGET (canvas)),
-			"gtk-double-click-time", &double_click_time,
-			NULL);
 
-		if ((ig->last_click_time + double_click_time) > event->time &&
-		    wbcg_edit_start (scg->wbcg, FALSE, FALSE))
-			break;
+		/*
+		 *  If the second click is on a different cell than the
+		 *  first one this can not be a double-click
+		 */
+		if (already_selected) {
+			g_object_get (gtk_widget_get_settings (GTK_WIDGET (canvas)),
+				      "gtk-double-click-time", &double_click_time,
+				      NULL);
+			
+			if ((ig->last_click_time + double_click_time) > event->time &&
+			    wbcg_edit_start (scg->wbcg, FALSE, FALSE))
+				break;
+		}
 
 		ig->last_click_time = event->time;
 		ig->selecting = ITEM_GRID_SELECTING_CELL_RANGE;
@@ -979,21 +995,21 @@ item_grid_button_press (ItemGrid *ig, GdkEventButton *event)
  */
 
 static gboolean
-cb_extend_cell_range (GnumericCanvas *gcanvas, int col, int row, gpointer ignored)
+cb_extend_cell_range (GnmCanvas *gcanvas, int col, int row, gpointer ignored)
 {
 	sv_selection_extend_to (((SheetControl *) gcanvas->simple.scg)->view, col, row);
 	return TRUE;
 }
 
 static gboolean
-cb_extend_expr_range (GnumericCanvas *gcanvas, int col, int row, gpointer ignored)
+cb_extend_expr_range (GnmCanvas *gcanvas, int col, int row, gpointer ignored)
 {
 	scg_rangesel_extend_to (gcanvas->simple.scg, col, row);
 	return TRUE;
 }
 
 static gboolean
-cb_extend_object_creation (GnumericCanvas *gcanvas, int col, int row, gpointer ignored)
+cb_extend_object_creation (GnmCanvas *gcanvas, int col, int row, gpointer ignored)
 {
 	int x, y;
 	gdouble new_x, new_y;
@@ -1014,7 +1030,7 @@ cb_cursor_come_to_rest (ItemGrid *ig)
 {
 	Sheet const *sheet = ((SheetControl *) ig->scg)->view->sheet;
 	GnomeCanvas *canvas = ig->canvas_item.canvas;
-	GnumericCanvas *gcanvas = GNUMERIC_CANVAS (canvas);
+	GnmCanvas *gcanvas = GNM_CANVAS (canvas);
 	GnmHLink *link;
 	int x, y;
 	CellPos pos;
@@ -1046,7 +1062,7 @@ cb_cursor_motion (ItemGrid *ig)
 {
 	Sheet const *sheet = ((SheetControl *) ig->scg)->view->sheet;
 	GnomeCanvas *canvas = ig->canvas_item.canvas;
-	GnumericCanvas *gcanvas = GNUMERIC_CANVAS (canvas);
+	GnmCanvas *gcanvas = GNM_CANVAS (canvas);
 	int x, y, cursor;
 	CellPos pos;
 	GnmHLink *old_link;
@@ -1075,7 +1091,7 @@ static gint
 item_grid_event (GnomeCanvasItem *item, GdkEvent *event)
 {
 	GnomeCanvas *canvas = item->canvas;
-	GnumericCanvas *gcanvas = GNUMERIC_CANVAS (canvas);
+	GnmCanvas *gcanvas = GNM_CANVAS (canvas);
 	ItemGrid *ig = ITEM_GRID (item);
 	SheetControlGUI *scg = ig->scg;
 	SheetControl *sc = (SheetControl *) scg;
@@ -1139,7 +1155,7 @@ item_grid_event (GnomeCanvasItem *item, GdkEvent *event)
 	}
 
 	case GDK_MOTION_NOTIFY: {
-		GnumericCanvasSlideHandler slide_handler = NULL;
+		GnmCanvasSlideHandler slide_handler = NULL;
 		switch (ig->selecting) {
 		case ITEM_GRID_NO_SELECTION:
 			if (ig->cursor_timer == 0)
@@ -1170,7 +1186,7 @@ item_grid_event (GnomeCanvasItem *item, GdkEvent *event)
 			 * that is more work than I want to do right now.
 			 */
 			if (gnm_canvas_handle_motion (gcanvas, canvas, &event->motion,
-						      GNM_SLIDE_X | GNM_SLIDE_Y,
+						      GNM_CANVAS_SLIDE_X | GNM_CANVAS_SLIDE_Y,
 						      &cb_extend_object_creation, NULL))
 				ig_obj_create_motion (ig, event->motion.x, event->motion.y);
 
@@ -1180,7 +1196,8 @@ item_grid_event (GnomeCanvasItem *item, GdkEvent *event)
 		};
 
 		gnm_canvas_handle_motion (gcanvas, canvas, &event->motion,
-			GNM_SLIDE_X | GNM_SLIDE_Y | GNM_SLIDE_AT_COLROW_BOUND,
+			GNM_CANVAS_SLIDE_X | GNM_CANVAS_SLIDE_Y |
+			GNM_CANVAS_SLIDE_AT_COLROW_BOUND,
 			slide_handler, NULL);
 		return TRUE;
 	}
