@@ -23,6 +23,7 @@
 #include "cell.h"
 #include "selection.h"
 #include "style.h"
+#include "sheet-style.h"
 #include "sheet-object-impl.h"
 #include "sheet-object-cell-comment.h"
 #include "gui-util.h"
@@ -54,7 +55,7 @@ static SheetControlClass *scg_parent_class;
 static void scg_ant                    (SheetControl *sc);
 static void scg_unant                  (SheetControl *sc);
 
-GnumericCanvas *
+GnmCanvas *
 scg_pane (SheetControlGUI *scg, int p)
 {
 	/* it is ok to request a pane when we are not frozen */
@@ -118,7 +119,7 @@ scg_redraw_headers (SheetControl *sc,
 {
 	SheetControlGUI *scg = (SheetControlGUI *)sc;
 	GnumericPane *pane;
- 	GnumericCanvas *gcanvas;
+ 	GnmCanvas *gcanvas;
 	int i;
 
 	for (i = scg->active_panes; i-- > 0 ; ) {
@@ -220,7 +221,7 @@ scg_resize (SheetControl *sc, gboolean force_scroll)
 {
 	SheetControlGUI *scg = (SheetControlGUI *)sc;
 	Sheet const *sheet;
-	GnumericCanvas *gcanvas;
+	GnmCanvas *gcanvas;
 	int h, w, btn_h, btn_w, tmp;
 	double zoom;
 
@@ -276,37 +277,45 @@ scg_resize (SheetControl *sc, gboolean force_scroll)
 		/* pane 0 has already been done */
 		for (i = scg->active_panes; i-- > 1 ; ) {
 			GnumericPane const *p = scg->pane + i;
-			p->gcanvas->first_offset.col = scg_colrow_distance_get (
-				scg, TRUE, 0, p->gcanvas->first.col);
-			p->gcanvas->first_offset.row = scg_colrow_distance_get (
-				scg, FALSE, 0, p->gcanvas->first.row);
+			if (p->is_active) {
+				p->gcanvas->first_offset.col = scg_colrow_distance_get (
+					scg, TRUE, 0, p->gcanvas->first.col);
+				p->gcanvas->first_offset.row = scg_colrow_distance_get (
+					scg, FALSE, 0, p->gcanvas->first.row);
+			}
 		}
 
-		gtk_widget_set_usize (GTK_WIDGET (scg->pane[1].gcanvas), r - l, -1);
-		gtk_widget_set_usize (GTK_WIDGET (scg->pane[2].gcanvas), r - l, b - t);
-		gtk_widget_set_usize (GTK_WIDGET (scg->pane[3].gcanvas), -1,    b - t);
+		if (scg->pane[1].is_active) {
+			gtk_widget_set_usize (GTK_WIDGET (scg->pane[1].gcanvas), r - l, -1);
+			/* The item_bar_calcs should be equal */
+			/* FIXME : The canvas gets confused when the initial scroll
+			 * region is set too early in its life cycle.
+			 * It likes it to be at the origin, we can live with that for now.
+			 * However, we really should track the bug eventually.
+			 */
+			h = item_bar_calc_size (scg->pane[1].col.item);
+			gtk_widget_set_usize (GTK_WIDGET (scg->pane[1].col.canvas), r - l, h+1);
+			gnome_canvas_set_scroll_region (scg->pane[1].col.canvas,
+				0, 0, GNUMERIC_CANVAS_FACTOR_X / zoom, h / zoom);
+				/* l / zoom, 0, r / zoom, h / zoom); */
+		}
+
+		if (scg->pane[3].is_active) {
+			gtk_widget_set_usize (GTK_WIDGET (scg->pane[3].gcanvas), -1,    b - t);
+			/* The item_bar_calcs should be equal */
+			w = item_bar_calc_size (scg->pane[3].row.item);
+			gtk_widget_set_usize (GTK_WIDGET (scg->pane[3].row.canvas), w+1, b - t);
+			gnome_canvas_set_scroll_region (scg->pane[3].row.canvas,
+				0, 0, w / zoom, GNUMERIC_CANVAS_FACTOR_Y / zoom);
+				/* 0, t / zoom, w / zoom, b / zoom); */
+		}
+
+		if (scg->pane[2].is_active)
+			gtk_widget_set_usize (GTK_WIDGET (scg->pane[2].gcanvas), r - l, b - t);
 
-		/* The item_bar_calcs should be equal */
-		/* FIXME : The canvas gets confused when the initial scroll
-		 * region is set too early in its life cycle.
-		 * It likes it to be at the origin, we can live with that for now.
-		 * However, we really should track the bug eventually.
-		 */
-		h = item_bar_calc_size (scg->pane[2].col.item);
-		gtk_widget_set_usize (GTK_WIDGET (scg->pane[2].col.canvas), r - l, h+1);
-		gnome_canvas_set_scroll_region (scg->pane[2].col.canvas,
-			0, 0, GNUMERIC_CANVAS_FACTOR_X / zoom, h / zoom);
-			/* l / zoom, 0, r / zoom, h / zoom); */
 		gnome_canvas_set_scroll_region (scg->pane[0].col.canvas,
 			0, 0, GNUMERIC_CANVAS_FACTOR_X / zoom, h / zoom);
 			/* r / zoom, 0, GNUMERIC_CANVAS_FACTOR_X / zoom, h / zoom); */
-
-		/* The item_bar_calcs should be equal */
-		w = item_bar_calc_size (scg->pane[2].row.item);
-		gtk_widget_set_usize (GTK_WIDGET (scg->pane[2].row.canvas), w+1, b - t);
-		gnome_canvas_set_scroll_region (scg->pane[2].row.canvas,
-			0, 0, w / zoom, GNUMERIC_CANVAS_FACTOR_Y / zoom);
-			/* 0, t / zoom, w / zoom, b / zoom); */
 		gnome_canvas_set_scroll_region (scg->pane[0].row.canvas,
 			0, 0, w / zoom, GNUMERIC_CANVAS_FACTOR_Y / zoom);
 			/* 0, b / zoom, w / zoom, GNUMERIC_CANVAS_FACTOR_Y / zoom); */
@@ -361,7 +370,7 @@ scg_scrollbar_config (SheetControl const *sc)
 	SheetControlGUI *scg = SHEET_CONTROL_GUI (sc);
 	GtkAdjustment *va = GTK_ADJUSTMENT (scg->va);
 	GtkAdjustment *ha = GTK_ADJUSTMENT (scg->ha);
-	GnumericCanvas *gcanvas = scg_pane (scg, 0);
+	GnmCanvas *gcanvas = scg_pane (scg, 0);
 	Sheet const    *sheet = sc->sheet;
 	SheetView const*sv = sc->view;
 	int const last_col = gcanvas->last_full.col;
@@ -474,14 +483,14 @@ scg_colrow_select (SheetControlGUI *scg, gboolean is_cols,
 				scg_rangesel_bound (scg,
 					0, index, SHEET_MAX_COLS-1, index);
 		} else if (is_cols) {
-			GnumericCanvas *gcanvas =
+			GnmCanvas *gcanvas =
 				scg_pane (scg, scg->pane[3].is_active ? 3 : 0);
 			sv_selection_add_range (sv,
 				index, gcanvas->first.row,
 				index, 0,
 				index, SHEET_MAX_ROWS-1);
 		} else {
-			GnumericCanvas *gcanvas =
+			GnmCanvas *gcanvas =
 				scg_pane (scg, scg->pane[1].is_active ? 1 : 0);
 			sv_selection_add_range (sv,
 				gcanvas->first.col, index,
@@ -491,7 +500,8 @@ scg_colrow_select (SheetControlGUI *scg, gboolean is_cols,
 	}
 
 	/* The edit pos, and the selection may have changed */
-	sheet_update (sv->sheet);
+	if (!rangesel)
+		sheet_update (sv->sheet);
 	return TRUE;
 }
 
@@ -620,17 +630,16 @@ scg_init (SheetControlGUI *scg)
  * A convenience routine to store the new topleft back in the view.
  */
 static void
-gnm_canvas_update_inital_top_left (GnumericCanvas const *gcanvas)
+gnm_canvas_update_inital_top_left (GnmCanvas const *gcanvas)
 {
 	if (gcanvas->pane->index == 0) {
 		SheetView *sv = gcanvas->simple.scg->sheet_control.view;
-		sv->initial_top_left.col = gcanvas->first.col;
-		sv->initial_top_left.row = gcanvas->first.row;
+		sv->initial_top_left = gcanvas->first;
 	}
 }
 
 static int
-bar_set_left_col (GnumericCanvas *gcanvas, int new_first_col)
+bar_set_left_col (GnmCanvas *gcanvas, int new_first_col)
 {
 	GnomeCanvas *colc;
 	int col_offset;
@@ -649,7 +658,7 @@ bar_set_left_col (GnumericCanvas *gcanvas, int new_first_col)
 }
 
 static void
-gnm_canvas_set_left_col (GnumericCanvas *gcanvas, int new_first_col)
+gnm_canvas_set_left_col (GnmCanvas *gcanvas, int new_first_col)
 {
 	g_return_if_fail (gcanvas != NULL);
 	g_return_if_fail (0 <= new_first_col && new_first_col < SHEET_MAX_COLS);
@@ -679,17 +688,18 @@ scg_set_left_col (SheetControlGUI *scg, int col)
 	else if (col > bound->end.col)
 		col = bound->end.col;
 
-	if (scg->pane[3].is_active) {
+	if (scg->pane[1].is_active) {
 		int right = scg->sheet_control.view->unfrozen_top_left.col;
 		if (col < right)
 			col = right;
-		gnm_canvas_set_left_col (scg_pane (scg, 3), col);
+		if (scg->pane[3].is_active)
+			gnm_canvas_set_left_col (scg_pane (scg, 3), col);
 	}
 	gnm_canvas_set_left_col (scg_pane (scg, 0), col);
 }
 
 static int
-bar_set_top_row (GnumericCanvas *gcanvas, int new_first_row)
+bar_set_top_row (GnmCanvas *gcanvas, int new_first_row)
 {
 	GnomeCanvas *rowc;
 	int row_offset;
@@ -708,7 +718,7 @@ bar_set_top_row (GnumericCanvas *gcanvas, int new_first_row)
 }
 
 static void
-gnm_canvas_set_top_row (GnumericCanvas *gcanvas, int new_first_row)
+gnm_canvas_set_top_row (GnmCanvas *gcanvas, int new_first_row)
 {
 	g_return_if_fail (gcanvas != NULL);
 	g_return_if_fail (0 <= new_first_row && new_first_row < SHEET_MAX_ROWS);
@@ -738,17 +748,18 @@ scg_set_top_row (SheetControlGUI *scg, int row)
 	else if (row > bound->end.row)
 		row = bound->end.row;
 
-	if (scg->pane[1].is_active) {
+	if (scg->pane[3].is_active) {
 		int bottom = scg->sheet_control.view->unfrozen_top_left.row;
 		if (row < bottom)
 			row = bottom;
-		gnm_canvas_set_top_row (scg_pane (scg, 1), row);
+		if (scg->pane[1].is_active)
+			gnm_canvas_set_top_row (scg_pane (scg, 1), row);
 	}
 	gnm_canvas_set_top_row (scg_pane (scg, 0), row);
 }
 
 static void
-gnm_canvas_set_top_left (GnumericCanvas *gcanvas,
+gnm_canvas_set_top_left (GnmCanvas *gcanvas,
 			 int col, int row, gboolean force_scroll)
 {
 	gboolean changed = FALSE;
@@ -797,14 +808,14 @@ scg_set_top_left (SheetControl *sc, int col, int row)
 }
 
 static void
-gnm_canvas_make_cell_visible (GnumericCanvas *gcanvas, int col, int row,
+gnm_canvas_make_cell_visible (GnmCanvas *gcanvas, int col, int row,
 			      gboolean const force_scroll)
 {
 	GnomeCanvas *canvas;
 	Sheet *sheet;
 	int   new_first_col, new_first_row;
 
-	g_return_if_fail (IS_GNUMERIC_CANVAS (gcanvas));
+	g_return_if_fail (GNM_IS_CANVAS (gcanvas));
 
 	/* Avoid calling this before the canvas is realized: We do not know the
 	 * visible area, and would unconditionally scroll the cell to the top
@@ -898,7 +909,7 @@ scg_make_cell_visible (SheetControlGUI *scg, int col, int row,
 				couple_panes ? br->col : scg->pane[0].gcanvas->first.col,
 				scg->pane[1].gcanvas->first.row,
 				force_scroll);
-			if (couple_panes)
+			if (couple_panes && scg->pane[3].is_active)
 				gnm_canvas_set_left_col (scg->pane[3].gcanvas, br->col);
 		} else if (couple_panes) { /* pane 2 */
 			/* FIXME : We may need to change the way this routine
@@ -925,7 +936,7 @@ scg_make_cell_visible (SheetControlGUI *scg, int col, int row,
 			? br->row
 			: scg->pane[0].gcanvas->first.row,
 			force_scroll);
-		if (couple_panes)
+		if (couple_panes && scg->pane[1].is_active)
 			gnm_canvas_set_top_row (scg->pane[1].gcanvas,
 				br->row);
 	} else {			 /* pane 0 */
@@ -958,64 +969,78 @@ scg_set_panes (SheetControl *sc)
 {
 	SheetControlGUI *scg = (SheetControlGUI *) sc;
 	gboolean const being_frozen = sv_is_frozen (sc->view);
-	gboolean const was_frozen = scg->pane[2].gcanvas != NULL;
+	gboolean const was_frozen =
+		scg->pane[1].gcanvas != NULL ||
+		scg->pane[3].gcanvas != NULL;
 
 	if (!being_frozen && !was_frozen)
 		return;
 
-	/* TODO : support just h or v split */
 	if (being_frozen) {
 		CellPos const *tl = &sc->view->frozen_top_left;
 		CellPos const *br = &sc->view->unfrozen_top_left;
+		gboolean const freeze_h = br->col > tl->col;
+		gboolean const freeze_v = br->row > tl->row;
 
-		gnm_pane_init (scg->pane + 1, scg, FALSE, 1);
-		gnm_pane_init (scg->pane + 2, scg, TRUE,  2);
-		gnm_pane_init (scg->pane + 3, scg, FALSE, 3);
-		scg->active_panes = 4;
 		gnm_pane_bound_set (scg->pane + 0,
 			br->col, br->row,
 			SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1);
-		gnm_pane_bound_set (scg->pane + 1,
-			tl->col, br->row, br->col-1, SHEET_MAX_ROWS-1);
-		gnm_pane_bound_set (scg->pane + 2,
-			tl->col, tl->row, br->col-1, br->row-1);
-		gnm_pane_bound_set (scg->pane + 3,
-			br->col, tl->row, SHEET_MAX_COLS-1, br->row-1);
 
-		gtk_table_attach (scg->inner_table,
-			GTK_WIDGET (scg->pane[2].col.canvas),
-			1, 2, 0, 1,
-			GTK_FILL | GTK_SHRINK,
-			GTK_FILL,
-			0, 0);
-		gtk_table_attach (scg->inner_table,
-			GTK_WIDGET (scg->pane[2].row.canvas),
-			0, 1, 1, 2,
-			GTK_FILL | GTK_SHRINK,
-			GTK_FILL,
-			0, 0);
-		gtk_table_attach (scg->inner_table,
-			GTK_WIDGET (scg->pane[2].gcanvas),
-			1, 2, 1, 2,
-			GTK_FILL | GTK_SHRINK,
-			GTK_FILL,
-			0, 0);
-		gtk_table_attach (scg->inner_table,
-			GTK_WIDGET (scg->pane[3].gcanvas),
-			2, 3, 1, 2,
-			GTK_EXPAND | GTK_FILL | GTK_SHRINK,
-			GTK_FILL | GTK_SHRINK,
-			0, 0);
-		gtk_table_attach (scg->inner_table,
-			GTK_WIDGET (scg->pane[1].gcanvas),
-			1, 2, 2, 3,
-			GTK_FILL | GTK_SHRINK,
-			GTK_EXPAND | GTK_FILL | GTK_SHRINK,
-			0, 0);
+		if (freeze_h) {
+			scg->active_panes = 2;
+			gnm_pane_init (scg->pane + 1, scg, TRUE, FALSE, 1);
+			gnm_pane_bound_set (scg->pane + 1,
+				tl->col, br->row, br->col-1, SHEET_MAX_ROWS-1);
+			gtk_table_attach (scg->inner_table,
+				GTK_WIDGET (scg->pane[1].gcanvas),
+				1, 2, 2, 3,
+				GTK_FILL | GTK_SHRINK,
+				GTK_EXPAND | GTK_FILL | GTK_SHRINK,
+				0, 0);
+			gtk_table_attach (scg->inner_table,
+				GTK_WIDGET (scg->pane[1].col.canvas),
+				1, 2, 0, 1,
+				GTK_FILL | GTK_SHRINK,
+				GTK_FILL,
+				0, 0);
+		}
+		if (freeze_h && freeze_v) {
+			scg->active_panes = 4;
+			gnm_pane_init (scg->pane + 2, scg, FALSE, FALSE,  2);
+			gnm_pane_bound_set (scg->pane + 2,
+				tl->col, tl->row, br->col-1, br->row-1);
+			gtk_table_attach (scg->inner_table,
+				GTK_WIDGET (scg->pane[2].gcanvas),
+				1, 2, 1, 2,
+				GTK_FILL | GTK_SHRINK,
+				GTK_FILL,
+				0, 0);
+		}
+		if (freeze_v) {
+			scg->active_panes = 4;
+			gnm_pane_init (scg->pane + 3, scg, FALSE, TRUE, 3);
+			gnm_pane_bound_set (scg->pane + 3,
+				br->col, tl->row, SHEET_MAX_COLS-1, br->row-1);
+			gtk_table_attach (scg->inner_table,
+				GTK_WIDGET (scg->pane[3].gcanvas),
+				2, 3, 1, 2,
+				GTK_EXPAND | GTK_FILL | GTK_SHRINK,
+				GTK_FILL | GTK_SHRINK,
+				0, 0);
+			gtk_table_attach (scg->inner_table,
+				GTK_WIDGET (scg->pane[3].row.canvas),
+				0, 1, 1, 2,
+				GTK_FILL | GTK_SHRINK,
+				GTK_FILL,
+				0, 0);
+		}
 	} else {
-		gnm_pane_release (scg->pane + 1);
-		gnm_pane_release (scg->pane + 2);
-		gnm_pane_release (scg->pane + 3);
+		if (scg->pane[1].is_active)
+			gnm_pane_release (scg->pane + 1);
+		if (scg->pane[2].is_active)
+			gnm_pane_release (scg->pane + 2);
+		if (scg->pane[3].is_active)
+			gnm_pane_release (scg->pane + 3);
 		scg->active_panes = 1;
 		gnm_pane_bound_set (scg->pane + 0,
 			0, 0, SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1);
@@ -1030,10 +1055,12 @@ scg_set_panes (SheetControl *sc)
 	if (being_frozen) {
 		CellPos const *tl = &sc->view->frozen_top_left;
 
-		gnm_canvas_set_left_col (scg->pane[1].gcanvas, tl->col);
-		gnm_canvas_set_top_row (scg->pane[3].gcanvas, tl->row);
-		gnm_canvas_set_top_left (scg->pane[2].gcanvas,
-					 tl->col, tl->row, TRUE);
+		if (scg->pane[1].is_active)
+			gnm_canvas_set_left_col (scg->pane[1].gcanvas, tl->col);
+		if (scg->pane[2].is_active)
+			gnm_canvas_set_top_left (scg->pane[2].gcanvas, tl->col, tl->row, TRUE);
+		if (scg->pane[3].is_active)
+			gnm_canvas_set_top_row (scg->pane[3].gcanvas, tl->row);
 	}
 }
 
@@ -1080,7 +1107,7 @@ sheet_control_gui_new (SheetView *sv)
 		0,
 		0, 0);
 
-	gnm_pane_init (scg->pane + 0, scg, TRUE, 0);
+	gnm_pane_init (scg->pane + 0, scg, TRUE, TRUE, 0);
 	scg->inner_table = GTK_TABLE (gtk_table_new (3, 3, FALSE));
 	gtk_table_attach (scg->inner_table, GTK_WIDGET (scg->corner),
 		0, 1, 0, 1,
@@ -1151,9 +1178,9 @@ sheet_control_gui_new (SheetView *sv)
 	g_signal_connect_after (G_OBJECT (scg->table),
 		"size_allocate",
 		G_CALLBACK (cb_table_size_allocate), scg);
-	g_signal_connect (G_OBJECT (scg->table),
+	g_signal_connect_object (G_OBJECT (scg->table),
 		"destroy",
-		G_CALLBACK (cb_table_destroy), scg);
+		G_CALLBACK (cb_table_destroy), G_OBJECT (scg), 0);
 
 	sv_attach_control (sv, SHEET_CONTROL (scg));
 
@@ -1170,7 +1197,7 @@ scg_finalize (GObject *object)
 
 	/* remove the object view before we disappear */
 	for (ptr = sheet->sheet_objects; ptr != NULL ; ptr = ptr->next )
-		sc_object_destroy_view	(sc, SHEET_OBJECT (ptr->data));
+		sc_object_destroy_view (sc, SHEET_OBJECT (ptr->data));
 
 	g_ptr_array_free (scg->col_group.buttons, TRUE);
 	g_ptr_array_free (scg->row_group.buttons, TRUE);
@@ -1178,8 +1205,10 @@ scg_finalize (GObject *object)
 	if (sc->view)
 		sv_detach_control (sc);
 
-	if (scg->table)
-		gtk_object_unref (GTK_OBJECT (scg->table));
+	if (scg->table) {
+		gtk_object_destroy (GTK_OBJECT (scg->table));
+		scg->table =NULL;
+	}
 
 	if (G_OBJECT_CLASS (scg_parent_class)->finalize)
 		(*G_OBJECT_CLASS (scg_parent_class)->finalize)(object);
@@ -1303,7 +1332,10 @@ enum {
 	CONTEXT_ROW_HEIGHT,
 	CONTEXT_ROW_HIDE,
 	CONTEXT_ROW_UNHIDE,
-	CONTEXT_COMMENT_EDIT
+	CONTEXT_COMMENT_EDIT,
+	CONTEXT_HYPERLINK_EDIT,
+	CONTEXT_HYPERLINK_ADD,
+	CONTEXT_HYPERLINK_REMOVE
 };
 static gboolean
 context_menu_handler (GnumericPopupMenuElement const *element,
@@ -1368,6 +1400,19 @@ context_menu_handler (GnumericPopupMenuElement const *element,
 	case CONTEXT_COMMENT_EDIT:
 		dialog_cell_comment (wbcg, sheet, &sv->edit_pos);
 		break;
+
+	case CONTEXT_HYPERLINK_EDIT:
+	case CONTEXT_HYPERLINK_ADD:
+		dialog_hyperlink (wbcg, sc);
+		break;
+
+	case CONTEXT_HYPERLINK_REMOVE: {
+		MStyle *style = mstyle_new ();
+		mstyle_set_hlink (style, NULL);
+		cmd_selection_format (wbc, style, NULL,
+			_("Remove Hyperlink"));
+		break;
+	}
 	default :
 		break;
 	};
@@ -1378,17 +1423,20 @@ void
 scg_context_menu (SheetControlGUI *scg, GdkEventButton *event,
 		  gboolean is_col, gboolean is_row)
 {
-	SheetControl *sc = (SheetControl *) scg;
+	SheetControl *sc = SHEET_CONTROL (scg);
+	Sheet *sheet = sc_sheet (sc);
 
 	enum {
 		CONTEXT_DISPLAY_FOR_CELLS = 1,
 		CONTEXT_DISPLAY_FOR_ROWS = 2,
-		CONTEXT_DISPLAY_FOR_COLS = 4
+		CONTEXT_DISPLAY_FOR_COLS = 4,
+		CONTEXT_DISPLAY_WITH_HYPERLINK	  = 8,
+		CONTEXT_DISPLAY_WITHOUT_HYPERLINK = 16
 	};
 	enum {
-		CONTEXT_DISABLE_PASTE_SPECIAL = 1,
-		CONTEXT_DISABLE_FOR_ROWS = 2,
-		CONTEXT_DISABLE_FOR_COLS = 4
+		CONTEXT_DISABLE_PASTE_SPECIAL	= 1,
+		CONTEXT_DISABLE_FOR_ROWS	= 2,
+		CONTEXT_DISABLE_FOR_COLS	= 4
 	};
 
 	static GnumericPopupMenuElement const popup_elements[] = {
@@ -1450,11 +1498,21 @@ scg_context_menu (SheetControlGUI *scg, GdkEventButton *event,
 		{ N_("_Unhide"),	  "Gnumeric_RowUnhide",
 		    CONTEXT_DISPLAY_FOR_ROWS, 0, CONTEXT_ROW_UNHIDE },
 
+		{ N_("_Hyperlink"),	  "Gnumeric_Link_Add",
+		    CONTEXT_DISPLAY_WITHOUT_HYPERLINK, 0,
+		    CONTEXT_HYPERLINK_ADD },
+		{ N_("Edit _Hyperlink"),	  "Gnumeric_Link_Edit",
+		    CONTEXT_DISPLAY_WITH_HYPERLINK, 0,
+		    CONTEXT_HYPERLINK_EDIT },
+		{ N_("_Remove Hyperlink"),	  "Gnumeric_Link_Delete",
+		    CONTEXT_DISPLAY_WITH_HYPERLINK, 0,
+		    CONTEXT_HYPERLINK_REMOVE },
+
 		{ NULL, NULL, 0, 0, 0 },
 	};
 
 	/* row and column specific operations */
-	int const display_filter =
+	int display_filter =
 		((!is_col && !is_row) ? CONTEXT_DISPLAY_FOR_CELLS : 0) |
 		(is_col ? CONTEXT_DISPLAY_FOR_COLS : 0) |
 		(is_row ? CONTEXT_DISPLAY_FOR_ROWS : 0);
@@ -1468,12 +1526,12 @@ scg_context_menu (SheetControlGUI *scg, GdkEventButton *event,
 		? 0 : CONTEXT_DISABLE_PASTE_SPECIAL;
 
 	GList *l;
+	gboolean has_link = FALSE;
 
 	wbcg_edit_finish (scg->wbcg, FALSE);
 
-	/*
-	 * Now see if there is some selection which selects a
-	 * whole row or a whole column and disable the insert/delete col/row menu items
+	/* Now see if there is some selection which selects a whole row or a
+	 * whole column and disable the insert/delete col/row menu items
 	 * accordingly
 	 */
 	for (l = sc->view->selections; l != NULL; l = l->next) {
@@ -1484,8 +1542,15 @@ scg_context_menu (SheetControlGUI *scg, GdkEventButton *event,
 
 		if (r->start.col == 0 && r->end.col == SHEET_MAX_COLS - 1)
 			sensitivity_filter |= CONTEXT_DISABLE_FOR_COLS;
+
+		if (!has_link && sheet_style_region_contains_link (sheet, r))
+			has_link = TRUE;
 	}
 
+	if (display_filter & CONTEXT_DISPLAY_FOR_CELLS)
+		display_filter |= ((has_link) ?
+				   CONTEXT_DISPLAY_WITH_HYPERLINK : CONTEXT_DISPLAY_WITHOUT_HYPERLINK);
+
 	gnumeric_create_popup_menu (popup_elements, &context_menu_handler,
 				    scg, display_filter,
 				    sensitivity_filter, event);
@@ -1615,8 +1680,6 @@ scg_mode_edit_object (SheetControlGUI *scg, SheetObject *so)
 		scg_set_display_cursor (scg);
 	}
 	g_object_unref (G_OBJECT (so));
-
-	puts (g_type_name_from_instance	(so));
 }
 
 /**
@@ -1699,7 +1762,7 @@ scg_object_update_bbox (SheetControlGUI *scg, SheetObject *so,
 }
 
 static int
-calc_obj_place (GnumericCanvas *gcanvas, int pixel, gboolean is_col,
+calc_obj_place (GnmCanvas *gcanvas, int pixel, gboolean is_col,
 		SheetObjectAnchorType anchor_type, float *offset)
 {
 	int origin;
@@ -1725,7 +1788,7 @@ calc_obj_place (GnumericCanvas *gcanvas, int pixel, gboolean is_col,
 void
 scg_object_calc_position (SheetControlGUI *scg, SheetObject *so, double const *coords)
 {
-	GnumericCanvas *gcanvas;
+	GnmCanvas *gcanvas;
 	SheetObjectAnchor anchor;
 	int i, pixels [4];
 	double tmp [4];
@@ -2203,12 +2266,12 @@ scg_rangesel_move (SheetControlGUI *scg, int n, gboolean jump_to_bound,
 	SheetView *sv = sc_view ((SheetControl *) scg);
 	CellPos tmp;
 
-	if (!scg->rangesel.active)
-		scg_rangesel_start (scg,
-			sv->edit_pos_real.col, sv->edit_pos_real.row,
-			sv->edit_pos_real.col, sv->edit_pos_real.row);
+	if (!scg->rangesel.active) {
+		tmp.col = sv->edit_pos_real.col;
+		tmp.row = sv->edit_pos_real.row;
+	} else
+		tmp = scg->rangesel.base_corner;
 
-	tmp = scg->rangesel.base_corner;
 	if (horiz)
 		tmp.col = sheet_find_boundary_horizontal (
 			sv_sheet (sv), tmp.col, tmp.row, tmp.row, n, jump_to_bound);
@@ -2216,7 +2279,10 @@ scg_rangesel_move (SheetControlGUI *scg, int n, gboolean jump_to_bound,
 		tmp.row = sheet_find_boundary_vertical (
 			sv_sheet (sv), tmp.col, tmp.row, tmp.col, n, jump_to_bound);
 
-	scg_rangesel_changed (scg, tmp.col, tmp.row, tmp.col, tmp.row);
+	if (scg->rangesel.active)
+		scg_rangesel_changed (scg, tmp.col, tmp.row, tmp.col, tmp.row);
+	else
+		scg_rangesel_start   (scg, tmp.col, tmp.row, tmp.col, tmp.row);
 	scg_make_cell_visible (scg, tmp.col, tmp.row, FALSE, TRUE);
 }
 
@@ -2420,7 +2486,7 @@ scg_class_init (GObjectClass *object_class)
 
 	g_return_if_fail (sc_class != NULL);
 
-	scg_parent_class = gtk_type_class (sheet_control_get_type ());
+	scg_parent_class = g_type_class_peek (sheet_control_get_type ());
 	object_class->finalize = scg_finalize;
 
 	sc_class->resize                 = scg_resize;
