@@ -18,12 +18,12 @@
  *    to keep track of the view frames.
  */
 #include <gnumeric-config.h>
+#include <gnumeric-i18n.h>
 #include "gnumeric.h"
 #include "sheet-object-bonobo.h"
 
 #include "workbook.h"
 #include "sheet.h"
-#include "workbook-private.h"
 #include "gui-util.h"
 
 #include <math.h>
@@ -83,13 +83,14 @@ get_file_name (void)
 
         filename = gtk_file_selection_get_filename (fs);
 
-	if (!(basename = g_basename (filename)) ||
+	if (!(basename = g_path_get_basename (filename)) ||
 	    basename [0] == '\0')
 		filename = NULL;
 	else
 		filename = g_strdup (filename);
 
 	gtk_object_destroy (GTK_OBJECT (fs));
+	g_free (basename);
 
 	return filename;
 }
@@ -291,7 +292,7 @@ sheet_object_bonobo_write_xml (SheetObject const *so,
 		if (BONOBO_EX (&ev)) {
 			/* TODO : Generate a decent message when sheetobjects
 			 * get user visible ids */
-			gnumeric_io_error_save (ctxt->io_context,
+			gnumeric_error_save (COMMAND_CONTEXT (ctxt)->io_context,
 				   bonobo_exception_get_text (&ev));
 		} else
 			res = TRUE;
@@ -312,7 +313,7 @@ sheet_object_bonobo_class_init (GtkObjectClass *object_class)
 {
 	SheetObjectClass *sheet_object_class;
 
-	sheet_object_bonobo_parent_class = gtk_type_class (sheet_object_get_type ());
+	sheet_object_bonobo_parent_class = g_type_class_peek (sheet_object_get_type ());
 
 	/* GtkObject class method overrides */
 	object_class->destroy = sheet_object_bonobo_destroy;
