@@ -9,19 +9,21 @@
  *
  * (C) 1998, 1999, 2000 Miguel de Icaza
  * (C) 2000-2001 Ximian, Inc.
+ * (C) 2002 Jody Goldberg
  */
 #include <gnumeric-config.h>
+#include <gnumeric-i18n.h>
 #include "gnumeric.h"
-#include "workbook.h"
+#include "workbook-priv.h"
 
 #include "workbook-view.h"
 #include "workbook-control.h"
-#include "workbook-private.h"
 #include "command-context.h"
 #include "application.h"
 #include "sheet.h"
-#include "cell.h"
+#include "sheet-view.h"
 #include "sheet-control.h"
+#include "cell.h"
 #include "expr.h"
 #include "expr-name.h"
 #include "dependent.h"
@@ -36,18 +38,10 @@
 #include "gnm-marshalers.h"
 #include "style-color.h"
 
-#ifdef WITH_BONOBO
-#include <bonobo/bonobo-persist-file.h>
-#include "sheet-object-container.h"
-#include "sheet-object-bonobo.h"
-#endif
-
-#include <gtk/gtk.h>
-#include <libgnome/gnome-i18n.h>
-#include <gal/util/e-util.h>
+#include <gtk/gtkmain.h> /* for gtk_main_quit */
+#include <gsf/gsf-impl-utils.h>
 #include <ctype.h>
 #include <string.h>
-#include <stdlib.h>
 
 static GObjectClass *workbook_parent_class;
 
@@ -55,12 +49,14 @@ static GObjectClass *workbook_parent_class;
 enum {
 	SUMMARY_CHANGED,
 	FILENAME_CHANGED,
+	SHEET_ORDER_CHANGED,
+	SHEET_ADDED,
+	SHEET_DELETED,
 	LAST_SIGNAL
 };
 
 static GQuark signals [LAST_SIGNAL] = { 0 };
 
-
 /*
  * We introduced numbers in front of the the history file names for two
  * reasons:
@@ -126,7 +122,7 @@ workbook_finalize (GObject *wb_object)
 	Workbook *wb = WORKBOOK (wb_object);
 	GList *sheets, *ptr;
 
-	wb->priv->during_destruction = TRUE;
+	wb->during_destruction = TRUE;
 
 	if (wb->file_saver != NULL) {
 		g_object_weak_unref (G_OBJECT (wb->file_saver),
@@ -201,19 +197,9 @@ workbook_finalize (GObject *wb_object)
 	g_hash_table_destroy (wb->sheet_hash_private);
 	wb->sheet_hash_private = NULL;
 
-	if (wb->sheet_order_dependents != NULL) {
-		g_hash_table_destroy (wb->sheet_order_dependents);
-		wb->sheet_order_dependents = NULL;
-	}
-
 	g_ptr_array_free (wb->sheets, TRUE);
 	wb->sheets = NULL;
 
-	wb->names = expr_name_list_destroy (wb->names);
-
-	workbook_private_delete (wb->priv);
-	wb->priv = NULL;
-
 	if (wb->file_format_level >= FILE_FL_MANUAL_REMEMBER)
 		workbook_history_update (application_workbook_list (), wb->filename);
 
@@ -248,7 +234,7 @@ workbook_set_dirty (Workbook *wb, gboolean is_dirty)
 	if (wb->summary_info != NULL)
 		wb->summary_info->modified = is_dirty;
 	g_hash_table_foreach (wb->sheet_hash_private,
-			      cb_sheet_mark_dirty, GINT_TO_POINTER (is_dirty));
+		cb_sheet_mark_dirty, GINT_TO_POINTER (is_dirty));
 }
 
 static void
@@ -275,12 +261,38 @@ workbook_is_dirty (Workbook const *wb)
 	if (wb->summary_info != NULL && wb->summary_info->modified)
 		return TRUE;
 
-	g_hash_table_foreach (wb->sheet_hash_private, cb_sheet_check_dirty,
-			      &dirty);
+	g_hash_table_foreach (wb->sheet_hash_private,
+		cb_sheet_check_dirty, &dirty);
 
 	return dirty;
 }
 
+/**
+ * workbook_set_placeholder :
+ * @wb :
+ * @is_placeholder :
+ **/
+void
+workbook_set_placeholder (Workbook *wb, gboolean is_placeholder)
+{
+	g_return_if_fail (IS_WORKBOOK (wb));
+	wb->is_placeholder = is_placeholder;
+}
+
+/**
+ * workbook_is_placeholder :
+ * @wb :
+ *
+ * Returns TRUE if @wb has no views and was created as a placeholder for data
+ * in an external reference.
+ **/
+gboolean
+workbook_is_placeholder	(Workbook const *wb)
+{
+	g_return_val_if_fail (IS_WORKBOOK (wb), FALSE);
+	return wb->is_placeholder;
+}
+
 static void
 cb_sheet_check_pristine (gpointer key, gpointer value, gpointer user_data)
 {
@@ -315,8 +327,8 @@ workbook_is_pristine (Workbook const *wb)
 		return FALSE;
 
 	/* Check if we seem to contain anything */
-	g_hash_table_foreach (wb->sheet_hash_private, cb_sheet_check_pristine,
-			      &pristine);
+	g_hash_table_foreach (wb->sheet_hash_private,
+		cb_sheet_check_pristine, &pristine);
 
 	return pristine;
 }
@@ -326,11 +338,12 @@ workbook_init (GObject *object)
 {
 	Workbook *wb = WORKBOOK (object);
 
-	wb->priv = workbook_private_new ();
+	wb->is_placeholder = FALSE;
+	wb->modified	   = FALSE;
+
 	wb->wb_views = NULL;
 	wb->sheets = g_ptr_array_new ();
-	wb->sheet_hash_private = g_hash_table_new (gnumeric_strcase_hash,
-						   gnumeric_strcase_equal);
+	wb->sheet_hash_private = g_hash_table_new (g_str_hash, g_str_equal);
 	wb->sheet_order_dependents = NULL;
 	wb->names        = NULL;
 	wb->summary_info = summary_info_new ();
@@ -341,20 +354,19 @@ workbook_init (GObject *object)
 	wb->undo_commands = wb->redo_commands = NULL;
 
 	/* default to no iteration */
-	wb->iteration.enabled = FALSE;
+	wb->iteration.enabled = TRUE;
 	wb->iteration.max_number = 100;
 	wb->iteration.tolerance = .001;
+	wb->recalc_auto = TRUE;
+
+	wb->file_format_level = FILE_FL_NEW;
+	wb->file_saver        = NULL;
+
+	wb->during_destruction = FALSE;
+	wb->being_reordered    = FALSE;
+	wb->recursive_dirty_enabled = TRUE;
 
 	application_workbook_list_add (wb);
-
-#if 0
-	workbook_corba_setup (wb);
-#endif
-#ifdef WITH_BONOBO
-#ifdef GNOME2_CONVERSION_COMPLETE
-	workbook_bonobo_setup (wb);
-#endif
-#endif
 }
 
 static void
@@ -372,7 +384,7 @@ workbook_class_init (GObjectClass *object_class)
 		gnm__VOID__VOID,
 		G_TYPE_NONE,
 		0, G_TYPE_NONE);
-	
+
 	signals [FILENAME_CHANGED] = g_signal_new ("filename_changed",
 		WORKBOOK_TYPE,
 		G_SIGNAL_RUN_LAST,
@@ -380,7 +392,34 @@ workbook_class_init (GObjectClass *object_class)
 		(GSignalAccumulator) NULL, NULL,
 		gnm__VOID__VOID,
 		G_TYPE_NONE,
-		0, G_TYPE_NONE);	
+		0, G_TYPE_NONE);
+
+	signals [SHEET_ORDER_CHANGED] = g_signal_new ("sheet_order_changed",
+		WORKBOOK_TYPE,
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (WorkbookClass, sheet_order_changed),
+		(GSignalAccumulator) NULL, NULL,
+		gnm__VOID__VOID,
+		G_TYPE_NONE,
+		0, G_TYPE_NONE);
+
+	signals [SHEET_ADDED] = g_signal_new ("sheet_added",
+		WORKBOOK_TYPE,
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (WorkbookClass, sheet_added),
+		(GSignalAccumulator) NULL, NULL,
+		gnm__VOID__VOID,
+		G_TYPE_NONE,
+		0, G_TYPE_NONE);
+
+	signals [SHEET_DELETED] = g_signal_new ("sheet_deleted",
+		WORKBOOK_TYPE,
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (WorkbookClass, sheet_deleted),
+		(GSignalAccumulator) NULL, NULL,
+		gnm__VOID__VOID,
+		G_TYPE_NONE,
+		0, G_TYPE_NONE);
 }
 
 /**
@@ -411,10 +450,6 @@ workbook_new (void)
 		is_unique = workbook_set_filename (wb, name);
 		g_free (name);
 	} while (!is_unique);
-	wb->file_format_level = FILE_FL_NEW;
-	wb->file_saver        = NULL;
-
-	wb->priv->during_destruction = FALSE;
 	return wb;
 }
 
@@ -431,21 +466,33 @@ static void
 workbook_sheet_name_strip_number (char *name, int* number)
 {
 	char *end;
-
+	int p10 = 1;
+	int n = 0;
 	*number = 1;
 
-	end = strrchr (name, ')');
-	if (end == NULL || end[1] != '\0')
+	g_return_if_fail (*name != 0);
+
+	end = name + strlen (name) - 1;
+	if (*end != ')')
 		return;
 
-	while (--end >= name) {
+	while (end > name) {
+		int dig;
+		end = g_utf8_prev_char (end);
+
 		if (*end == '(') {
-			*number = atoi (end + 1);
-			*end = '\0';
+			*number = n;
+			*end = 0;
 			return;
 		}
-		if (!isdigit ((unsigned char)*end))
+
+		dig = g_unichar_digit_value (g_utf8_get_char (end));
+		if (dig == -1)
 			return;
+
+		/* FIXME: check for overflow.  */
+		n += p10 * dig;
+		p10 *= 10;
 	}
 }
 
@@ -479,9 +526,9 @@ workbook_new_with_sheets (int sheet_count)
  * FIXME : Add a check to ensure the name is unique.
  */
 gboolean
-workbook_set_filename (Workbook *wb, const char *name)
+workbook_set_filename (Workbook *wb, char const *name)
 {
-	char const *base_name;
+	char *base_name;
 	g_return_val_if_fail (wb != NULL, FALSE);
 	g_return_val_if_fail (name != NULL, FALSE);
 
@@ -489,10 +536,11 @@ workbook_set_filename (Workbook *wb, const char *name)
 		g_free (wb->filename);
 
 	wb->filename = g_strdup (name);
-	base_name = g_basename (name);
+	base_name = g_path_get_basename (name);
 
 	WORKBOOK_FOREACH_CONTROL (wb, view, control,
 		wb_control_title_set (control, base_name););
+	g_free (base_name);
 
 	g_signal_emit (G_OBJECT (wb), signals [FILENAME_CHANGED], 0);
 	return TRUE;
@@ -506,12 +554,20 @@ workbook_get_filename (Workbook *wb)
 	return wb->filename;
 }
 
-void workbook_add_summary_info (Workbook *wb, SummaryItem *sit)
+void
+workbook_add_summary_info (Workbook *wb, SummaryItem *sit)
 {
 	if (summary_info_add (wb->summary_info, sit))
 		g_signal_emit (G_OBJECT (wb), signals [SUMMARY_CHANGED], 0);
 }
 
+SummaryInfo *
+workbook_metadata (Workbook *wb)
+{
+	g_return_val_if_fail (IS_WORKBOOK (wb), NULL);
+	return wb->summary_info;
+}
+
 /**
  * workbook_set_saveinfo:
  * @wb: the workbook to modify
@@ -559,20 +615,6 @@ workbook_get_file_saver (Workbook *wb)
 	return wb->file_saver;
 }
 
-static void
-cb_sheet_calc_spans (gpointer key, gpointer value, gpointer flags)
-{
-	sheet_calc_spans (value, GPOINTER_TO_INT(flags));
-}
-void
-workbook_calc_spans (Workbook *wb, SpanCalcFlags const flags)
-{
-	g_return_if_fail (wb != NULL);
-
-	g_hash_table_foreach (wb->sheet_hash_private,
-			      &cb_sheet_calc_spans, GINT_TO_POINTER (flags));
-}
-
 void
 workbook_unref (Workbook *wb)
 {
@@ -602,44 +644,43 @@ workbook_unref (Workbook *wb)
 Value *
 workbook_foreach_cell_in_range (EvalPos const *pos,
 				Value const	*cell_range,
-				gboolean	 only_existing,
-				ForeachCellCB	 handler,
-				void		*closure)
+				CellIterFlags	 flags,
+				CellIterFunc	 handler,
+				gpointer	 closure)
 {
 	Range  r;
 	Sheet *start_sheet, *end_sheet;
 
 	g_return_val_if_fail (pos != NULL, NULL);
 	g_return_val_if_fail (cell_range != NULL, NULL);
-
 	g_return_val_if_fail (cell_range->type == VALUE_CELLRANGE, NULL);
 
-	value_cellrange_normalize (pos, cell_range, &start_sheet, &end_sheet, &r);
+	rangeref_normalize (&cell_range->v_range.cell, pos,
+			    &start_sheet, &end_sheet, &r);
 
 	if (start_sheet != end_sheet) {
 		Value *res;
 		Workbook const *wb = start_sheet->workbook;
 		int i = start_sheet->index_in_wb;
 		int stop = end_sheet->index_in_wb;
-		if (i < stop) { int tmp = i; i = stop ; stop = tmp; }
+		if (i > stop) { int tmp = i; i = stop ; stop = tmp; }
 
-		g_return_val_if_fail (start_sheet->workbook == wb, VALUE_TERMINATE);
+		g_return_val_if_fail (end_sheet->workbook == wb, VALUE_TERMINATE);
 
-		while (i <= stop) {
-			res = sheet_foreach_cell_in_range (g_ptr_array_index (wb->sheets, i),
-							   only_existing,
-							   r.start.col, r.start.row,
-							   r.end.col, r.end.row,
-							   handler, closure);
+		for (; i <= stop ; i++) {
+			res = sheet_foreach_cell_in_range (
+				g_ptr_array_index (wb->sheets, i), flags,
+				r.start.col, r.start.row, r.end.col, r.end.row,
+				handler, closure);
 			if (res != NULL)
 				return res;
 		}
+		return NULL;
 	}
 
-	return sheet_foreach_cell_in_range (start_sheet, only_existing,
-					    r.start.col, r.start.row,
-					    r.end.col, r.end.row,
-					    handler, closure);
+	return sheet_foreach_cell_in_range (start_sheet, flags,
+		r.start.col, r.start.row, r.end.col, r.end.row,
+		handler, closure);
 }
 
 /**
@@ -683,11 +724,22 @@ workbook_cells (Workbook *wb, gboolean comments)
 gboolean
 workbook_enable_recursive_dirty (Workbook *wb, gboolean enable)
 {
-	gboolean old = wb->priv->recursive_dirty_enabled;
-	wb->priv->recursive_dirty_enabled = enable;
+	gboolean old;
+
+	g_return_val_if_fail (IS_WORKBOOK (wb), FALSE);
+
+	old = wb->recursive_dirty_enabled;
+	wb->recursive_dirty_enabled = enable;
 	return old;
 }
 
+void
+workbook_autorecalc_enable (Workbook *wb, gboolean enable)
+{
+	g_return_if_fail (IS_WORKBOOK (wb));
+	wb->recalc_auto = enable;
+}
+
 void
 workbook_iteration_enabled (Workbook *wb, gboolean enable)
 {
@@ -719,25 +771,26 @@ workbook_attach_view (Workbook *wb, WorkbookView *wbv)
 	g_return_if_fail (IS_WORKBOOK_VIEW (wbv));
 	g_return_if_fail (wb_view_workbook (wbv) == NULL);
 
+	if (wb->wb_views == NULL)
+		wb->wb_views = g_ptr_array_new ();
+	g_ptr_array_add (wb->wb_views, wbv);
 	wbv->wb = wb;
-	if (wbv->wb->wb_views == NULL)
-		wbv->wb->wb_views = g_ptr_array_new ();
-	g_ptr_array_add (wbv->wb->wb_views, wbv);
-
-	/* Set the titles of the newly connected view's controls */
-	if (wbv->wb != NULL) {
-		char const *base_name = g_basename (wb->filename);
-		WORKBOOK_VIEW_FOREACH_CONTROL (wbv, wbc,
-			wb_control_title_set (wbc, base_name););
-	}
 }
 
 void
 workbook_detach_view (WorkbookView *wbv)
 {
+	SheetView *sv;
+
 	g_return_if_fail (IS_WORKBOOK_VIEW (wbv));
 	g_return_if_fail (IS_WORKBOOK (wbv->wb));
 
+	WORKBOOK_FOREACH_SHEET (wbv->wb, sheet, {
+		sv = sheet_get_view (sheet, wbv);
+		sheet_detach_view (sv);
+		g_object_unref (G_OBJECT (sv));
+	});
+
 	g_ptr_array_remove (wbv->wb->wb_views, wbv);
 	if (wbv->wb->wb_views->len == 0) {
 		g_ptr_array_free (wbv->wb->wb_views, TRUE);
@@ -785,13 +838,19 @@ cb_dep_unlink (Dependent *dep, gpointer value, gpointer user_data)
 		pos = &DEP_TO_CELL (dep)->pos;
 	dependent_unlink (dep, pos);
 }
+
 static void
 pre_sheet_index_change (Workbook *wb)
 {
+	g_return_if_fail (!wb->being_reordered);
+
+	wb->being_reordered = TRUE;
+
 	if (wb->sheet_order_dependents != NULL)
 		g_hash_table_foreach (wb->sheet_order_dependents,
 			(GHFunc) cb_dep_unlink, NULL);
 }
+
 static void
 cb_dep_link (Dependent *dep, gpointer value, gpointer user_data)
 {
@@ -803,9 +862,18 @@ cb_dep_link (Dependent *dep, gpointer value, gpointer user_data)
 static void
 post_sheet_index_change (Workbook *wb)
 {
+	g_return_if_fail (wb->being_reordered);
+
 	if (wb->sheet_order_dependents != NULL)
 		g_hash_table_foreach (wb->sheet_order_dependents,
 			(GHFunc) cb_dep_link, NULL);
+
+	wb->being_reordered = FALSE;
+
+	if (wb->during_destruction)
+		return;
+
+	g_signal_emit (G_OBJECT (wb), signals [SHEET_ORDER_CHANGED], 0);
 }
 
 static void
@@ -831,18 +899,25 @@ workbook_sheet_by_index (Workbook const *wb, int i)
 /**
  * workbook_sheet_by_name:
  * @wb: workbook to lookup the sheet on
- * @sheet_name: the sheet name we are looking for.
+ * @name: the sheet name we are looking for.
  *
  * Returns a pointer to a Sheet or NULL if the sheet
  * was not found.
  */
 Sheet *
-workbook_sheet_by_name (Workbook const *wb, char const *sheet_name)
+workbook_sheet_by_name (Workbook const *wb, char const *name)
 {
-	g_return_val_if_fail (wb != NULL, NULL);
-	g_return_val_if_fail (sheet_name != NULL, NULL);
+	Sheet *sheet;
+	char *tmp;
 
-	return g_hash_table_lookup (wb->sheet_hash_private, sheet_name);
+	g_return_val_if_fail (IS_WORKBOOK (wb), NULL);
+	g_return_val_if_fail (name != NULL, NULL);
+
+	tmp = g_utf8_casefold (name, -1);
+	sheet = g_hash_table_lookup (wb->sheet_hash_private, tmp);
+	g_free (tmp);
+
+	return sheet;
 }
 
 /**
@@ -872,7 +947,7 @@ workbook_sheet_attach (Workbook *wb, Sheet *new_sheet,
 	}
 
 	g_hash_table_insert (wb->sheet_hash_private,
-			     new_sheet->name_unquoted, new_sheet);
+		new_sheet->name_case_insensitive, new_sheet);
 	post_sheet_index_change (wb);
 
 	WORKBOOK_FOREACH_VIEW (wb, view,
@@ -899,14 +974,14 @@ workbook_sheet_detach (Workbook *wb, Sheet *sheet)
 			      == sheet, FALSE);
 
 	/* Finish any object editing */
-	SHEET_FOREACH_CONTROL (sheet, control,
+	SHEET_FOREACH_CONTROL (sheet, view, control,
 		sc_mode_edit (control););
 
 	sheet_index = sheet->index_in_wb;
 
 	/* If not exiting, adjust the focus for any views whose focus sheet
 	 * was the one being deleted, and prepare to recalc */
-	if (!wb->priv->during_destruction) {
+	if (!wb->during_destruction) {
 		if (sheet_index > 0)
 			focus = g_ptr_array_index (wb->sheets, sheet_index-1);
 		else if ((sheet_index+1) < (int)wb->sheets->len)
@@ -925,13 +1000,20 @@ workbook_sheet_detach (Workbook *wb, Sheet *sheet)
 	WORKBOOK_FOREACH_CONTROL (wb, view, control,
 		wb_control_sheet_remove (control, sheet););
 
+	/* If we are not destroying things,
+	 * Check for 3d refs that start or end on this sheet
+	 */
+	if (wb->sheet_order_dependents != NULL) {
+#warning TODO
+		puts ("foo");
+	}
+
 	/* Remove our reference to this sheet */
-#warning TODO : check for 3d refs that start or end on this sheet
 	pre_sheet_index_change (wb);
 	g_ptr_array_remove_index (wb->sheets, sheet_index);
 	workbook_sheet_index_update (wb, sheet_index);
 	sheet->index_in_wb = -1;
-	g_hash_table_remove (wb->sheet_hash_private, sheet->name_unquoted);
+	g_hash_table_remove (wb->sheet_hash_private, sheet->name_case_insensitive);
 	sheet_destroy (sheet);
 	post_sheet_index_change (wb);
 
@@ -960,6 +1042,8 @@ workbook_sheet_add (Workbook *wb, Sheet const *insert_after, gboolean make_dirty
 	if (make_dirty)
 		sheet_set_dirty (new_sheet, TRUE);
 
+	g_signal_emit (G_OBJECT (wb), signals [SHEET_ADDED], 0);
+
 	return new_sheet;
 }
 
@@ -980,6 +1064,9 @@ workbook_sheet_delete (Sheet *sheet)
 	wb = sheet->workbook;
 
 	if (!sheet->pristine) {
+		Sheet *new_focus = NULL;
+		int i;
+
 		/*
 		 * FIXME : Deleting a sheet plays havoc with our data structures.
 		 * Be safe for now and empty the undo/redo queues
@@ -993,6 +1080,22 @@ workbook_sheet_delete (Sheet *sheet)
 			wb_control_undo_redo_clear (control, FALSE);
 			wb_control_undo_redo_labels (control, NULL, NULL);
 		);
+
+		i = sheet->index_in_wb - 1;
+		if (i < 0)
+			i = sheet->index_in_wb + 1;
+		if (i < workbook_sheet_count (wb))
+			new_focus = workbook_sheet_by_index (wb, i);
+
+		WORKBOOK_FOREACH_VIEW (wb, wbv,
+			if (sheet == wb_view_cur_sheet (wbv))
+				wb_view_sheet_focus (wbv, new_focus);
+		);
+		WORKBOOK_FOREACH_CONTROL (wb, view, control,
+			wb_control_undo_redo_clear (control, TRUE);
+			wb_control_undo_redo_clear (control, FALSE);
+			wb_control_undo_redo_labels (control, NULL, NULL);
+		);
 	}
 
 	/* Important to do these BEFORE detaching the sheet */
@@ -1000,6 +1103,8 @@ workbook_sheet_delete (Sheet *sheet)
 
 	/* All is fine, remove the sheet */
 	workbook_sheet_detach (wb, sheet);
+
+	g_signal_emit (G_OBJECT (wb), signals [SHEET_DELETED], 0);
 }
 
 /**
@@ -1021,7 +1126,7 @@ workbook_sheet_move (Sheet *sheet, int direction)
 	if (0 <= new_pos && new_pos < workbook_sheet_count (wb)) {
 		int min_pos = MIN (old_pos, new_pos);
 		int max_pos = MAX (old_pos, new_pos);
-		
+
 		g_ptr_array_remove_index (wb->sheets, old_pos);
 		g_ptr_array_insert (wb->sheets, sheet, new_pos);
 
@@ -1042,8 +1147,8 @@ workbook_sheet_move (Sheet *sheet, int direction)
  * workbook_sheet_get_free_name:
  * @wb:   workbook to look for
  * @base: base for the name, e. g. "Sheet"
- * @name_format : optionally null format for handling dupilicates.
  * @always_suffix: if true, add suffix even if the name "base" is not in use.
+ * @handle_counter : strip counter if necessary
  *
  * Gets a new unquoted name for a sheets such that it does not exist on the
  * workbook.
@@ -1092,21 +1197,80 @@ workbook_sheet_get_free_name (Workbook *wb,
 	return name;
 }
 
+/**
+ * workbook_sheet_reorder:
+ * @wb:          workbook to look for
+ * @new_order:   list of sheets in order (new sheets are marked NULL)
+ * @new_sheets:  list of new sheets
+ *
+ * Adjusts the order of the sheets.
+ *
+ * Returns FALSE when it was successful
+ **/
 gboolean
-workbook_sheet_reorganize (WorkbookControl *wbc, 
-			   GSList *changed_names, GSList *new_order,  
-			   GSList *new_names,  GSList *old_names,
-			   GSList **new_sheets, GSList *color_changed,
-			   GSList *colors_fore, GSList *colors_back)
+workbook_sheet_reorder (Workbook *wb, GSList *new_order, GSList *new_sheets)
 {
 	GSList *this_sheet;
-	GSList *new_sheet = NULL;
 	gint old_pos, new_pos = 0;
+
+	g_return_val_if_fail (IS_WORKBOOK (wb), FALSE);
+
+	if (new_order == NULL)
+		return TRUE;
+
+	pre_sheet_index_change (wb);
+	this_sheet = new_order;
+	while (this_sheet) {
+		gboolean an_old_sheet = TRUE;
+		Sheet *sheet = this_sheet->data;
+
+		if (new_sheets && sheet == NULL) {
+			sheet = new_sheets->data;
+			new_sheets = new_sheets->next;
+			an_old_sheet = FALSE;
+		}
+
+		if (sheet != NULL) {
+			old_pos = sheet->index_in_wb;
+			if (new_pos != old_pos) {
+				int max_pos = MAX (old_pos, new_pos);
+				int min_pos = MIN (old_pos, new_pos);
+
+				g_ptr_array_remove_index (wb->sheets, old_pos);
+				g_ptr_array_insert (wb->sheets, sheet, new_pos);
+				for (; max_pos >= min_pos ; max_pos--) {
+					Sheet *sheet = g_ptr_array_index (wb->sheets, max_pos);
+					sheet->index_in_wb = max_pos;
+				}
+				WORKBOOK_FOREACH_CONTROL (wb, view, control,
+							  wb_control_sheet_move (control,
+										 sheet, new_pos););
+				if (an_old_sheet)
+					sheet_set_dirty (sheet, TRUE);
+			}
+			new_pos++;
+		}
+		this_sheet = this_sheet->next;
+	}
+	post_sheet_index_change (wb);
+	return FALSE;
+}
+
+gboolean
+workbook_sheet_reorganize (Workbook *wb,
+			   GSList *changed_names, GSList *new_order,
+			   GSList *new_names,  GSList *old_names,
+			   GSList **new_sheets, GSList *color_changed,
+			   GSList *colors_fore, GSList *colors_back,
+			   GSList *protection_changed, GSList *new_locks,
+			   CommandContext *cc)
+{
+	GSList *new_sheet = NULL;
 	GSList *the_names;
 	GSList *the_sheets;
 	GSList *the_fore;
 	GSList *the_back;
-	Workbook *wb = wb_control_workbook (wbc);
+	GSList *the_lock;
 
 /* We need to verify validity of the new names */
 	the_names = new_names;
@@ -1120,37 +1284,36 @@ workbook_sheet_reorganize (WorkbookControl *wbc,
 
 		if (new_name != NULL ) {
 			sheet = the_sheets->data;
-			
+
 			/* Is the sheet name to short ?*/
 			if (1 > strlen (new_name)) {
-				gnumeric_error_invalid (COMMAND_CONTEXT (wbc), 
-							_("Sheet name must have at least 1 letter"),
-							new_name);
+				gnumeric_error_invalid (cc,
+					_("Sheet name must have at least 1 letter"),
+					new_name);
 				return TRUE;
 			}
-			
+
 			/* Is the sheet name already in use ?*/
-			tmp = (Sheet *) g_hash_table_lookup (wb->sheet_hash_private, 
-							     new_name);
-			
+			tmp = workbook_sheet_by_name (wb, new_name);
+
 			if (tmp != NULL) {
 				/* Perhaps it is a sheet also to be renamed */
 				GSList *tmp_sheets = g_slist_find (changed_names, tmp);
 				if (NULL == tmp_sheets) {
-					gnumeric_error_invalid (COMMAND_CONTEXT (wbc),
-							 _("There is already a sheet named"),
-								new_name);
+					gnumeric_error_invalid (cc,
+						_("There is already a sheet named"),
+						new_name);
 					return TRUE;
 				}
 			}
-			
+
 			/* Will we try to use the same name a second time ?*/
-			if (the_names->next != NULL && 
+			if (the_names->next != NULL &&
 			    g_slist_find_custom (the_names->next, new_name, g_str_compare) != NULL) {
-				gnumeric_error_invalid (COMMAND_CONTEXT (wbc),
-							_("You may not use this name twice"),
-							new_name);
-				return TRUE;			
+				gnumeric_error_invalid (cc,
+					_("You may not use this name twice"),
+					new_name);
+				return TRUE;
 			}
 		}
 		the_names = the_names->next;
@@ -1175,10 +1338,10 @@ workbook_sheet_reorganize (WorkbookControl *wbc,
 		if (sheet != NULL) {
 			sheet_rename (sheet, the_names->data);
 			g_hash_table_insert (wb->sheet_hash_private,
-					     sheet->name_unquoted, sheet);
-			
+					     sheet->name_case_insensitive, sheet);
+
 			sheet_set_dirty (sheet, TRUE);
-			
+
 			WORKBOOK_FOREACH_CONTROL (wb, view, control,
 						  wb_control_sheet_rename (control, sheet););
 		}
@@ -1198,9 +1361,9 @@ workbook_sheet_reorganize (WorkbookControl *wbc,
 				char *name = the_names->data;
 				Sheet *a_new_sheet ;
 				gboolean free_name = (name == NULL);
-				
+
 				if (free_name)
-					name = workbook_sheet_get_free_name 
+					name = workbook_sheet_get_free_name
 						(wb, _("Sheet"), TRUE, FALSE);
 				a_new_sheet = sheet_new (wb, name);
 				if (free_name)
@@ -1224,15 +1387,15 @@ workbook_sheet_reorganize (WorkbookControl *wbc,
 			sheet = new_sheet->data;
 			new_sheet = new_sheet->next;
 		}
-		
+
 		if (sheet != NULL) {
 			GdkColor *back = (GdkColor *) the_back->data;
 			GdkColor *fore = (GdkColor *) the_fore->data;
-			StyleColor *tab_color = back ? 
-				style_color_new (back->red, back->green, 
+			StyleColor *tab_color = back ?
+				style_color_new (back->red, back->green,
 						 back->blue) : NULL;
 			StyleColor *text_color = fore ?
-				style_color_new (fore->red, fore->green, 
+				style_color_new (fore->red, fore->green,
 						 fore->blue) : NULL;
 			sheet_set_tab_color (sheet, tab_color, text_color);
 		}
@@ -1240,44 +1403,28 @@ workbook_sheet_reorganize (WorkbookControl *wbc,
 		the_back = the_back->next;
 		the_sheets = the_sheets->next;
 	}
-	
-/* reordering */
-	new_sheet = new_sheets ? *new_sheets : NULL;
-	this_sheet = new_order;
-	while (this_sheet) {
-		gboolean an_old_sheet = TRUE;
-		Sheet *sheet = this_sheet->data;
 
+/* Changing Protection */
+	the_lock = new_locks;
+	the_sheets = protection_changed;
+	while (the_sheets) {
+		Sheet *sheet = the_sheets->data;
 		if (new_sheet && sheet == NULL) {
 			sheet = new_sheet->data;
 			new_sheet = new_sheet->next;
-			an_old_sheet = FALSE;
 		}
-		if (sheet != NULL) {
-			old_pos = sheet->index_in_wb;
-			if (new_pos != old_pos) {
-				int max_pos = MAX (old_pos, new_pos);
-				int min_pos = MIN (old_pos, new_pos);
-
-				g_ptr_array_remove_index (wb->sheets, old_pos);
-				g_ptr_array_insert (wb->sheets, sheet, new_pos);
-				for (; max_pos >= min_pos ; max_pos--) {
-					Sheet *sheet = g_ptr_array_index (wb->sheets, max_pos);
-					sheet->index_in_wb = max_pos;
-				}
-				WORKBOOK_FOREACH_CONTROL (wb, view, control,
-							  wb_control_sheet_move (control, 
-										 sheet, new_pos););
-				if (an_old_sheet)
-					sheet_set_dirty (sheet, TRUE);
-			}
-			new_pos++;
-		}
-		this_sheet = this_sheet->next;
+		if (sheet != NULL)
+			sheet->is_protected = GPOINTER_TO_INT (the_lock->data);;
+		the_lock = the_lock->next;
+		the_sheets = the_sheets->next;
 	}
+
+/* reordering */
+	workbook_sheet_reorder (wb, new_order, new_sheets ? *new_sheets : NULL);
+
 	return FALSE;
 }
 
-E_MAKE_TYPE (workbook, "Workbook", Workbook,
-	     workbook_class_init, workbook_init,
-	     G_TYPE_OBJECT);
+GSF_CLASS (Workbook, workbook,
+	   workbook_class_init, workbook_init,
+	   G_TYPE_OBJECT);
